@@ -1,7 +1,7 @@
 /* bar.c - 2000/05/05 */
 /*
- *  EasyTAG - Tag editor for MP3 and OGG files
- *  Copyright (C) 2000-2002  Jerome Couderc <j.couderc@ifrance.com>
+ *  EasyTAG - Tag editor for MP3 and Ogg Vorbis files
+ *  Copyright (C) 2000-2003  Jerome Couderc <j.couderc@ifrance.com>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -58,7 +58,7 @@ void     Statusbar_Remove_Timer (void);
 /*
  * Menu bar
  */
-void Menu_Item_Action (GtkWidget *widget, guint action)
+void Menu_Item_Action (GtkWidget *widget ATTRIBUTE_UNUSED, guint action)
 {
     switch (action)
     {
@@ -90,60 +90,61 @@ void Menu_Item_Action (GtkWidget *widget, guint action)
         case AM_SORT_DESCENDING_FILE_BITRATE:  ET_Sort_File_List_And_Update_UI(SORTING_BY_DESCENDING_FILE_BITRATE);  break;
         case AM_SORT_ASCENDING_FILE_SAMPLERATE: ET_Sort_File_List_And_Update_UI(SORTING_BY_ASCENDING_FILE_SAMPLERATE); break;
         case AM_SORT_DESCENDING_FILE_SAMPLERATE:ET_Sort_File_List_And_Update_UI(SORTING_BY_DESCENDING_FILE_SAMPLERATE);break;
-        case AM_FIRST:            Action_Select_First_File();   break;
-        case AM_PREV:             Action_Select_Prev_File();    break;
-        case AM_NEXT:             Action_Select_Next_File();    break;
-        case AM_LAST:             Action_Select_Last_File();    break;
-        case AM_SCAN:             Action_Scan_File();           break;
-        case AM_REMOVE:           Action_Remove_Tag();          break;
-        case AM_UNDO:             Action_Undo_File();           break;
-        case AM_REDO:             Action_Redo_File();           break;
-        case AM_SAVE:             Action_Save_File();           break;
-        case AM_SCAN_ALL:         Action_Scan_All_Files();      break;
-        case AM_REMOVE_ALL:       Action_Remove_All_Tags();     break;
-        case AM_UNDO_ALL:         Action_Undo_All_Files();      break;
-        case AM_REDO_ALL:         Action_Redo_All_Files();      break;
-        case AM_SAVE_ALL:         Action_Save_All_Files();      break;
-        case AM_LOAD_HOME_DIR:          Browser_Load_Home_Directory();     break;
-        case AM_LOAD_DEFAULT_DIR:       Browser_Load_Default_Directory();  break;
-        case AM_SET_PATH_AS_DEFAULT:    Set_Current_Path_As_Default();     break;
+        case AM_SELECT_ALL_FILES:       Action_Select_All_Files();              break;
+        case AM_UNSELECT_ALL_FILES:     Action_Unselect_All_Files();            break;
+        case AM_INVERT_SELECTION:       Action_Invert_Files_Selection();        break;
+        case AM_DELETE_FILE:            Action_Delete_Selected_Files();         break;
+        case AM_FIRST:                  Action_Select_First_File();             break;
+        case AM_PREV:                   Action_Select_Prev_File();              break;
+        case AM_NEXT:                   Action_Select_Next_File();              break;
+        case AM_LAST:                   Action_Select_Last_File();              break;
+        case AM_SCAN:                   Action_Scan_Selected_Files();           break;
+        case AM_REMOVE:                 Action_Remove_Selected_Tags();          break;
+        case AM_UNDO:                   Action_Undo_Selected_Files();           break;
+        case AM_REDO:                   Action_Redo_Selected_File();            break;
+        case AM_SAVE:                   Action_Save_Selected_Files();           break;
+        case AM_UNDO_HISTORY:           Action_Undo_From_History_List();        break;
+        case AM_REDO_HISTORY:           Action_Redo_From_History_List();        break;
+        case AM_LOAD_HOME_DIR:          Browser_Load_Home_Directory();          break;
+        case AM_LOAD_DEFAULT_DIR:       Browser_Load_Default_Directory();       break;
+        case AM_SET_PATH_AS_DEFAULT:    Set_Current_Path_As_Default();          break;
         case AM_RENAME_DIR:             Browser_Open_Rename_Directory_Window(); break;
-        case AM_COLLAPSE_TREE:          Browser_Tree_Collapse();           break;
-        case AM_INITIALIZE_TREE:        Browser_Tree_Rebuild(NULL);        break;
-        case AM_RELOAD_DIRECTORY:       Browser_Reload_Directory();        break;
+        case AM_COLLAPSE_TREE:          Browser_Tree_Collapse();                break;
+        case AM_INITIALIZE_TREE:        Browser_Tree_Rebuild(NULL);             break;
+        case AM_RELOAD_DIRECTORY:       Browser_Reload_Directory();             break;
         case AM_BROWSE_DIRECTORY_WITH:  Browser_Open_Run_Program_Tree_Window(); break;
         case AM_BROWSE_FILE_WITH:       Browser_Open_Run_Program_List_Window(); break;
-        case AM_OPEN_OPTIONS_WINDOW:    Open_OptionsWindow();              break;
-        case AM_SAVE_CONFIG:            Save_Config_Now();                 break;
+        case AM_OPEN_OPTIONS_WINDOW:    Open_OptionsWindow();                   break;
+        case AM_SAVE_CONFIG:            Save_Config_Now();                      break;
         case AM_SCANNER_FILL_TAG:
             if (!Scanner_Option_Menu_Get_Active_Menu_Item(SCANNER_FILL_TAG))
                 Open_ScannerWindow(SCANNER_FILL_TAG);
             else
                 if (ScannerWindow)
-                    Scan_Select_Mode_And_Run_Scanner((ET_File *)ETFileList->data);
+                    Scan_Select_Mode_And_Run_Scanner(ETFileDisplayed);
             break;
         case AM_SCANNER_RENAME_FILE:
             if (!Scanner_Option_Menu_Get_Active_Menu_Item(SCANNER_RENAME_FILE))
                 Open_ScannerWindow(SCANNER_RENAME_FILE);
             else
                 if (ScannerWindow)
-                    Scan_Select_Mode_And_Run_Scanner((ET_File *)ETFileList->data);
+                    Scan_Select_Mode_And_Run_Scanner(ETFileDisplayed);
             break;
         case AM_SCANNER_PROCESS_FIELDS:
             if (!Scanner_Option_Menu_Get_Active_Menu_Item(SCANNER_PROCESS_FIELDS))
                 Open_ScannerWindow(SCANNER_PROCESS_FIELDS);
             else
                 if (ScannerWindow)
-                    Scan_Select_Mode_And_Run_Scanner((ET_File *)ETFileList->data);
+                    Scan_Select_Mode_And_Run_Scanner(ETFileDisplayed);
             break;
-        case AM_SEARCH_FILE:            Open_Search_File_Window();         break;
-        case AM_CDDB_SEARCH:            Open_Cddb_Window();                break;
-        case AM_FILENAME_FROM_TXT:      Open_Load_Filename_Window();       break;
-        case AM_WRITE_PLAYLIST:         Open_Write_Playlist_Window();      break;
-        case AM_RUN_AUDIO_PLAYER:       Run_Audio_Player_With_File_List(); break;
-        case AM_OPEN_ABOUT_WINDOW:      Show_About_Window();               break;
-        case AM_QUIT:                   Quit_MainWindow();                 break;
-        default: g_warning("No menu action defined for this item!");    break;
+        case AM_SEARCH_FILE:            Open_Search_File_Window();          break;
+        case AM_CDDB_SEARCH:            Open_Cddb_Window();                 break;
+        case AM_FILENAME_FROM_TXT:      Open_Load_Filename_Window();        break;
+        case AM_WRITE_PLAYLIST:         Open_Write_Playlist_Window();       break;
+        case AM_RUN_AUDIO_PLAYER:       Run_Audio_Player_Using_Selection(); break;
+        case AM_OPEN_ABOUT_WINDOW:      Show_About_Window();                break;
+        case AM_QUIT:                   Quit_MainWindow();                  break;
+        default: g_warning("No menu action defined for this item!");        break;
     }
 }
 
@@ -198,26 +199,29 @@ GtkWidget *Create_Menu_Bar (void)
       {_("/File/Sort List/sep"),                           NULL,     NULL,             0,                               "<Separator>"},
       {_("/File/Sort List/Ascending by samplerate"),       NULL,     Menu_Item_Action, AM_SORT_ASCENDING_FILE_SAMPLERATE, "<Item>"},
       {_("/File/Sort List/Descending by samplerate"),      NULL,     Menu_Item_Action, AM_SORT_DESCENDING_FILE_SAMPLERATE,"<Item>"},
-      {_("/File/Open File with ..."),NULL,             Menu_Item_Action,     AM_BROWSE_FILE_WITH,  "<Item>"},
-      {_("/File/sep"),               NULL,             NULL,                 0,                    "<Separator>"},
-      {_("/File/_First File"),       "<control>F",     Menu_Item_Action,     AM_FIRST,             "<Item>"},
-      {_("/File/_Previous File"),    "<control>P",     Menu_Item_Action,     AM_PREV,              "<Item>"},
-      {_("/File/_Next File"),        "<control>N",     Menu_Item_Action,     AM_NEXT,              "<Item>"},
-      {_("/File/_Last File"),        "<control>L",     Menu_Item_Action,     AM_LAST,              "<Item>"},
-      {_("/File/sep"),               NULL,             NULL,                 0,                    "<Separator>"},
-      {_("/File/_Scan File"),        "<control>S",     Menu_Item_Action,     AM_SCAN,              "<Item>"},
-      {_("/File/_Remove Tag"),       "<control>R",     Menu_Item_Action,     AM_REMOVE,            "<Item>"},
-      {_("/File/_Undo Last File Changes"),"<control>U",Menu_Item_Action,     AM_UNDO,              "<Item>"},
-      {_("/File/R_edo Last File Changes"),"<control>E",Menu_Item_Action,     AM_REDO,              "<Item>"},
-      {_("/File/S_ave File"),        "<control>A",     Menu_Item_Action,     AM_SAVE,              "<Item>"},
-      {_("/File/sep"),               NULL,             NULL,                 0,                    "<Separator>"},
-      {_("/File/Scan All Files"),    "<control><alt>S",Menu_Item_Action,     AM_SCAN_ALL,          "<Item>"},
-      {_("/File/Remove All Tags"),   "<control><alt>R",Menu_Item_Action,     AM_REMOVE_ALL,        "<Item>"},
-      {_("/File/Undo Last Changes"), "<control><alt>U",Menu_Item_Action,     AM_UNDO_ALL,          "<Item>"},
-      {_("/File/Redo Last Changes"), "<control><alt>E",Menu_Item_Action,     AM_REDO_ALL,          "<Item>"},
-      {_("/File/Save All Files"),    "<control><alt>A",Menu_Item_Action,     AM_SAVE_ALL,          "<Item>"},
-      {_("/File/sep"),               NULL,             NULL,                 0,                    "<Separator>"},
-      {_("/File/_Quit"),             "<control>Q",     Menu_Item_Action,     AM_QUIT,              "<Item>"},
+      {_("/File/Open File(s) with ..."),                   NULL,     Menu_Item_Action,     AM_BROWSE_FILE_WITH,  "<Item>"},
+      {_("/File/sep"),                                     NULL,     NULL,                 0,                    "<Separator>"},
+      {_("/File/Select All Files"),         "<control>A",             Menu_Item_Action,     AM_SELECT_ALL_FILES,  "<Item>"},
+      {_("/File/Unselect All Files"),       "<control><shift>A",      Menu_Item_Action,     AM_UNSELECT_ALL_FILES,"<Item>"},
+      {_("/File/Invert Files Selection"),   "<control>I",             Menu_Item_Action,     AM_INVERT_SELECTION,  "<Item>"},
+      {_("/File/sep"),                      NULL,             NULL,                 0,                    "<Separator>"},
+      {_("/File/Delete File(s)"),           NULL,             Menu_Item_Action,     AM_DELETE_FILE,       "<Item>"},
+      {_("/File/sep"),                      NULL,             NULL,                 0,                    "<Separator>"},
+      {_("/File/_First File"),              "<control>F",     Menu_Item_Action,     AM_FIRST,             "<Item>"},
+      {_("/File/_Previous File"),           "<control>P",     Menu_Item_Action,     AM_PREV,              "<Item>"},
+      {_("/File/_Next File"),               "<control>N",     Menu_Item_Action,     AM_NEXT,              "<Item>"},
+      {_("/File/_Last File"),               "<control>L",     Menu_Item_Action,     AM_LAST,              "<Item>"},
+      {_("/File/sep"),                      NULL,             NULL,                 0,                    "<Separator>"},
+      {_("/File/S_can File(s)"),            "<control>C",     Menu_Item_Action,     AM_SCAN,              "<Item>"},
+      {_("/File/_Remove Tag(s)"),           "<control>R",     Menu_Item_Action,     AM_REMOVE,            "<Item>"},
+      {_("/File/_Undo Last File(s) Changes"),"<control>U",    Menu_Item_Action,     AM_UNDO,              "<Item>"},
+      {_("/File/R_edo Last File(s) Changes"),"<control>E",    Menu_Item_Action,     AM_REDO,              "<Item>"},
+      {_("/File/_Save File(s)"),            "<control>S",     Menu_Item_Action,     AM_SAVE,              "<Item>"},
+      {_("/File/sep"),                      NULL,             NULL,                 0,                    "<Separator>"},
+      {_("/File/Undo Last Changes"),        "<control><alt>U",Menu_Item_Action,     AM_UNDO_HISTORY,      "<Item>"},
+      {_("/File/Redo Last Changes"),        "<control><alt>E",Menu_Item_Action,     AM_REDO_HISTORY,      "<Item>"},
+      {_("/File/sep"),                      NULL,             NULL,                 0,                    "<Separator>"},
+      {_("/File/_Quit"),                    "<control>Q",     Menu_Item_Action,     AM_QUIT,              "<Item>"},
 
       {_("/_Browser"),                            NULL,               NULL,            0,                       "<Branch>"},
       {_("/Browser/tearoff"),                     NULL,               NULL,            0,                       "<Tearoff>"},
@@ -225,9 +229,9 @@ GtkWidget *Create_Menu_Bar (void)
       {_("/Browser/Go to _Default Directory"),    "<control>D",       Menu_Item_Action,AM_LOAD_DEFAULT_DIR,     "<Item>"},
       {_("/Browser/Set _Current Path as Default"),NULL,               Menu_Item_Action,AM_SET_PATH_AS_DEFAULT,  "<Item>"},
       {_("/Browser/sep"),                         NULL,               NULL,            0,                       "<Separator>"},
-      {_("/Browser/Rename Directory..."),         "<control><shift>D",Menu_Item_Action,AM_RENAME_DIR,           "<Item>"},
+      {_("/Browser/Rename Directory ..."),        "<control><shift>D",Menu_Item_Action,AM_RENAME_DIR,           "<Item>"},
       {_("/Browser/Reload Directory"),            NULL,               Menu_Item_Action,AM_RELOAD_DIRECTORY,     "<Item>"},
-      {_("/Browser/Browse Directory with..."),    NULL,               Menu_Item_Action,AM_BROWSE_DIRECTORY_WITH,"<Item>"},
+      {_("/Browser/Browse Directory with ..."),   NULL,               Menu_Item_Action,AM_BROWSE_DIRECTORY_WITH,"<Item>"},
       {_("/Browser/sep"),                         NULL,               NULL,            0,                       "<Separator>"},
       {_("/Browser/Browse _Sub-directories"),     "<control><shift>S",NULL,            AM_RECURSE,              "<CheckItem>"},
       {_("/Browser/sep"),                         NULL,               NULL,            0,                       "<Separator>"},
@@ -236,28 +240,28 @@ GtkWidget *Create_Menu_Bar (void)
    
       {_("/S_canner"),                         NULL,        NULL,             0,                        "<Branch>"},
       {_("/Scanner/tearoff"),                  NULL,        NULL,             0,                        "<Tearoff>"},
-      {_("/Scanner/_Fill Tag..."),             NULL,        Menu_Item_Action, AM_SCANNER_FILL_TAG,      "<Item>"},
-      {_("/Scanner/_Rename File..."),          NULL,        Menu_Item_Action, AM_SCANNER_RENAME_FILE,   "<Item>"},
-      {_("/Scanner/_Process Fields..."),       NULL,        Menu_Item_Action, AM_SCANNER_PROCESS_FIELDS,"<Item>"},
+      {_("/Scanner/_Fill Tag(s) ..."),         NULL,        Menu_Item_Action, AM_SCANNER_FILL_TAG,      "<Item>"},
+      {_("/Scanner/_Rename File(s) ..."),      NULL,        Menu_Item_Action, AM_SCANNER_RENAME_FILE,   "<Item>"},
+      {_("/Scanner/_Process Fields ..."),      NULL,        Menu_Item_Action, AM_SCANNER_PROCESS_FIELDS,"<Item>"},
 
       {_("/_Misc"),                            NULL,        NULL,             0,                        "<Branch>"},
       {_("/Misc/tearoff"),                     NULL,        NULL,             0,                        "<Tearoff>"},
-      {_("/Misc/S_earch File..."),             "<alt>E",    Menu_Item_Action, AM_SEARCH_FILE,           "<Item>"},
-      {_("/Misc/_CDDB Search..."),             NULL,        Menu_Item_Action, AM_CDDB_SEARCH,           "<Item>"},
+      {_("/Misc/S_earch File(s) ..."),         "<alt>E",    Menu_Item_Action, AM_SEARCH_FILE,           "<Item>"},
+      {_("/Misc/_CDDB Search ..."),            NULL,        Menu_Item_Action, AM_CDDB_SEARCH,           "<Item>"},
       {_("/Misc/sep"),                         NULL,        NULL,             0,                        "<Separator>"},
-      {_("/Misc/Load Filenames from TXT..."),  "<alt>T",    Menu_Item_Action, AM_FILENAME_FROM_TXT,     "<Item>"},
-      {_("/Misc/Write Playlist..."),           "<alt>W",    Menu_Item_Action, AM_WRITE_PLAYLIST,        "<Item>"},
+      {_("/Misc/Load Filenames from TXT ..."), "<alt>T",    Menu_Item_Action, AM_FILENAME_FROM_TXT,     "<Item>"},
+      {_("/Misc/Write Playlist ..."),          "<alt>W",    Menu_Item_Action, AM_WRITE_PLAYLIST,        "<Item>"},
       {_("/Misc/Run Audio Player"),            "<alt>X",    Menu_Item_Action, AM_RUN_AUDIO_PLAYER,      "<Item>"},
     
       {_("/_Settings"),                        NULL,        NULL,             0,                        "<Branch>"},
       {_("/Settings/tearoff"),                 NULL,        NULL,             0,                        "<Tearoff>"},
-      {_("/Settings/_Preferences..."),         "<alt>P",    Menu_Item_Action, AM_OPEN_OPTIONS_WINDOW,   "<Item>"},
+      {_("/Settings/_Preferences ..."),        "<alt>P",    Menu_Item_Action, AM_OPEN_OPTIONS_WINDOW,   "<Item>"},
       {_("/Settings/sep"),                     NULL,        NULL,             0,                        "<Separator>"},
       {_("/Settings/Save _Configuration Now"), "<alt>C",    Menu_Item_Action, AM_SAVE_CONFIG,           "<Item>"},
     
       {_("/_Help"),                            NULL,        NULL,             0,                        "<LastBranch>"},
       {_("/Help/tearoff"),                     NULL,        NULL,             0,                        "<Tearoff>"},
-      {_("/Help/_About..."),                   "<alt>A",    Menu_Item_Action, AM_OPEN_ABOUT_WINDOW,     "<Item>"}
+      {_("/Help/_About .."),                   "<alt>A",    Menu_Item_Action, AM_OPEN_ABOUT_WINDOW,     "<Item>"}
     };
 
     GtkWidget *HandleBar;
@@ -425,15 +429,12 @@ GtkWidget *Create_Progress_Bar (void)
 #include "../pixmaps/next.xpm"
 #include "../pixmaps/last.xpm"
 #include "../pixmaps/scan.xpm"
-#include "../pixmaps/scan_all.xpm"
 #include "../pixmaps/delete.xpm"
-#include "../pixmaps/delete_all.xpm"
 #include "../pixmaps/undo.xpm"
-#include "../pixmaps/undo_all.xpm"
 #include "../pixmaps/redo.xpm"
-#include "../pixmaps/redo_all.xpm"
 #include "../pixmaps/save.xpm"
-#include "../pixmaps/save_all.xpm"
+#include "../pixmaps/select_all.xpm"
+#include "../pixmaps/invert_selection.xpm"
 #include "../pixmaps/stop.xpm"
 #include "../pixmaps/exit.xpm"
 GtkWidget *Create_Tool_Bar (void)
@@ -473,49 +474,38 @@ GtkWidget *Create_Tool_Bar (void)
     gtk_toolbar_append_space(GTK_TOOLBAR(ToolBar));
 
     /*
-     * Buttons for actions for only ONE file.
+     * Buttons for actions for selected files.
      */
     Icon = Create_Pixmap_Icon(scan_xpm);
-    TBScanButton = gtk_toolbar_append_item(GTK_TOOLBAR(ToolBar),NULL,_("Scan this file"),
-                                           NULL,Icon,GTK_SIGNAL_FUNC(Action_Scan_File),NULL);
+    TBScanButton = gtk_toolbar_append_item(GTK_TOOLBAR(ToolBar),NULL,_("Scan selected files"),
+                                           NULL,Icon,GTK_SIGNAL_FUNC(Action_Scan_Selected_Files),NULL);
     Icon = Create_Pixmap_Icon(delete_xpm);
-    TBRemoveButton = gtk_toolbar_append_item(GTK_TOOLBAR(ToolBar),NULL,_("Remove tag of this file"),
-                                             NULL,Icon,GTK_SIGNAL_FUNC(Action_Remove_Tag),NULL);
+    TBRemoveButton = gtk_toolbar_append_item(GTK_TOOLBAR(ToolBar),NULL,_("Remove tag of selected files"),
+                                             NULL,Icon,GTK_SIGNAL_FUNC(Action_Remove_Selected_Tags),NULL);
     Icon = Create_Pixmap_Icon(undo_xpm);
-    TBUndoButton = gtk_toolbar_append_item(GTK_TOOLBAR(ToolBar),NULL,_("Undo last changes of this file"),
-                                           NULL,Icon,GTK_SIGNAL_FUNC(Action_Undo_File),NULL);
+    TBUndoButton = gtk_toolbar_append_item(GTK_TOOLBAR(ToolBar),NULL,_("Undo last changes of selected files"),
+                                           NULL,Icon,GTK_SIGNAL_FUNC(Action_Undo_Selected_Files),NULL);
     //gtk_signal_connect(GTK_OBJECT(TBUndoButton),"enter",GTK_SIGNAL_FUNC(Undo_Display_Last_Info),NULL);
 
     Icon = Create_Pixmap_Icon(redo_xpm);
-    TBRedoButton = gtk_toolbar_append_item(GTK_TOOLBAR(ToolBar),NULL,_("Redo last changes of this file"),
-                                           NULL,Icon,GTK_SIGNAL_FUNC(Action_Redo_File),NULL);
+    TBRedoButton = gtk_toolbar_append_item(GTK_TOOLBAR(ToolBar),NULL,_("Redo last changes of selected files"),
+                                           NULL,Icon,GTK_SIGNAL_FUNC(Action_Redo_Selected_File),NULL);
     //gtk_signal_connect(GTK_OBJECT(TBUndoButton),"enter",GTK_SIGNAL_FUNC(Undo_Display_Last_Info),NULL);
 
     Icon = Create_Pixmap_Icon(save_xpm);
-    TBSaveButton = gtk_toolbar_append_item(GTK_TOOLBAR(ToolBar),NULL,_("Save this file"),
-                                            NULL,Icon,GTK_SIGNAL_FUNC(Action_Save_File),NULL);
+    TBSaveButton = gtk_toolbar_append_item(GTK_TOOLBAR(ToolBar),NULL,_("Save selected files"),
+                                            NULL,Icon,GTK_SIGNAL_FUNC(Action_Save_Selected_Files),NULL);
 
     /* Separator */    
     gtk_toolbar_append_space(GTK_TOOLBAR(ToolBar));
 
-    /*
-     * Buttons for actions for ALL files.
-     */
-    Icon = Create_Pixmap_Icon(scan_all_xpm);
-    TBScanAllButton = gtk_toolbar_append_item(GTK_TOOLBAR(ToolBar),NULL,_("Scan all files"),NULL,
-                                              Icon,GTK_SIGNAL_FUNC(Action_Scan_All_Files),NULL);
-    Icon = Create_Pixmap_Icon(delete_all_xpm);
-    TBRemoveAllButton = gtk_toolbar_append_item(GTK_TOOLBAR(ToolBar),NULL,_("Remove all tags"),NULL,
-                                                Icon,GTK_SIGNAL_FUNC(Action_Remove_All_Tags),NULL);
-    Icon = Create_Pixmap_Icon(undo_all_xpm);
-    TBUndoAllButton = gtk_toolbar_append_item(GTK_TOOLBAR(ToolBar),NULL,_("Undo last changes"),NULL,
-                                              Icon,GTK_SIGNAL_FUNC(Action_Undo_All_Files),NULL);
-    Icon = Create_Pixmap_Icon(redo_all_xpm);
-    TBRedoAllButton = gtk_toolbar_append_item(GTK_TOOLBAR(ToolBar),NULL,_("Redo last changes"),NULL,
-                                              Icon,GTK_SIGNAL_FUNC(Action_Redo_All_Files),NULL);
-    Icon = Create_Pixmap_Icon(save_all_xpm);
-    TBSaveAllButton = gtk_toolbar_append_item(GTK_TOOLBAR(ToolBar),NULL,_("Save all files"),NULL,
-                                               Icon,GTK_SIGNAL_FUNC(Action_Save_All_Files),NULL);
+    Icon = Create_Pixmap_Icon(select_all_xpm);
+    TBSelectAllButton = gtk_toolbar_append_item(GTK_TOOLBAR(ToolBar),NULL,_("Select all files"),NULL,
+                                               Icon,GTK_SIGNAL_FUNC(Action_Select_All_Files),NULL);
+
+    Icon = Create_Pixmap_Icon(invert_selection_xpm);
+    TBInvertSelectionButton = gtk_toolbar_append_item(GTK_TOOLBAR(ToolBar),NULL,_("Invert files selection"),NULL,
+                                               Icon,GTK_SIGNAL_FUNC(Action_Invert_Files_Selection),NULL);
 
     /* Separator */    
     gtk_toolbar_append_space(GTK_TOOLBAR(ToolBar));
@@ -540,4 +530,3 @@ GtkWidget *Create_Tool_Bar (void)
     gtk_widget_show_all(HandleBar);
     return HandleBar;
 }
-
