@@ -1,7 +1,7 @@
 /* cddb.c - 2000/09/15 */
 /*
- *  EasyTAG - Tag editor for MP3 and OGG files
- *  Copyright (C) 2000-2002  Jerome Couderc <j.couderc@ifrance.com>
+ *  EasyTAG - Tag editor for MP3 and Ogg Vorbis files
+ *  Copyright (C) 2000-2003  Jerome Couderc <j.couderc@ifrance.com>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -66,6 +66,10 @@ guint      CddbStatusBarContext;
 GtkWidget *CddbStopSearchButton;
 GtkWidget *CddbSearchStringInResultNextButton;
 GtkWidget *CddbSearchStringInResultPrevButton;
+GtkWidget *CddbDisplayRedLinesButton;
+GtkWidget *CddbSelectAllInResultButton;
+GtkWidget *CddbUnselectAllInResultButton;
+GtkWidget *CddbInvertSelectionInResultButton;
 
 gboolean   CddbStopSearch = FALSE;
 
@@ -75,7 +79,7 @@ gboolean   CddbStopSearch = FALSE;
  **************/
 void Cddb_Destroy_Window   (void);
 void Cddb_Window_Key_Press (GtkWidget *window, GdkEvent *event);
-void Cddb_Show_Album_Info  (GtkCList *clist, gint row, gint column);
+void Cddb_Show_Album_Info  (GtkCList *clist, guint row, gint column);
 void Cddb_Run_Search       (void);
 
 gboolean Cddb_Free_Album_List       (void);
@@ -91,28 +95,35 @@ gboolean Cddb_Search_Album_List_From_String (void);
 gboolean Cddb_Get_Album_Tracks_List_CB     (GtkCList *clist, gint row, gint column);
 gboolean Cddb_Get_Album_Tracks_List        (GtkCList *clist, gint row, gint column);
 
-void     Cddb_Load_Album_List              (void);
+void     Cddb_Load_Album_List              (gboolean only_red_lines);
 void     Cddb_Load_Track_Album_List        (GList *track_list);
 gboolean Cddb_Set_Track_Infos_To_File_List (void);
+void     Cddb_Album_List_Set_Row_Color     (GtkCList *clist, gint row);
 
 void Cddb_Search_In_All_Fields_Check_Button_Toggled     (void);
 void Cddb_Search_In_All_Categories_Check_Button_Toggled (void);
 void Cddb_Set_To_All_Fields_Check_Button_Toggled        (void);
 void Cddb_Stop_Search                                   (void);
 void Cddb_Search_String_In_Result                       (GtkWidget *entry, GtkButton *button);
+void Cddb_Display_Red_Lines_In_Result      (void);
 
 void   Cddb_Set_Apply_Button_Sensivity     (void);
 void   Cddb_Set_Search_Button_Sensivity    (void);
 void   Cddb_Show_Categories_Button_Toggled (void) ;
 gchar *Cddb_Generate_Request_String_With_Fields_And_Categories_Options (void);
-void   Cddb_Album_List_Set_Row_Visible     (GtkCList *clist, gint row);
+void   Cddb_Album_List_Set_Row_Visible     (GtkCList *clist, guint row);
 
-void Cddb_Track_List_Row_Selected (GtkCList *clist, gint row, gint column);
+void Cddb_Track_List_Row_Selected   (GtkCList *clist, guint row, gint column);
+void Cddb_Track_List_Row_Unselected (GtkCList *clist, guint row, gint column);
 GtkWidget *Create_Cddb_Track_Clist_Popup_Menu (GtkCList *clist);
-void Cddb_Track_Clist_Select_All_Lines   (void);
-void Cddb_Track_Clist_Unselect_All_Lines (void);
-void Cddb_Track_Clist_Invert_Selection   (void);
-
+void Cddb_Track_Clist_Select_All_Lines     (void);
+void Cddb_Track_Clist_Unselect_All_Lines   (void);
+void Cddb_Track_Clist_Invert_Selection     (void);
+void Cddb_Track_Clist_Sort_By_Ascending_Track_Number  (void);
+void Cddb_Track_Clist_Sort_By_Descending_Track_Number (void);
+void Cddb_Track_Clist_Sort_By_Ascending_Track_Name    (void);
+void Cddb_Track_Clist_Sort_By_Descending_Track_Name   (void);
+void Cddb_Track_List_Column_Selected                  (GtkCList *clist, gint column, gpointer user_data);
 
 /*************
  * Functions *
@@ -129,6 +140,10 @@ void Init_CddbWindow (void)
 #include "../pixmaps/stop.xpm"
 #include "../pixmaps/up.xpm"
 #include "../pixmaps/down.xpm"
+#include "../pixmaps/red_lines.xpm"
+#include "../pixmaps/select_all.xpm"
+#include "../pixmaps/invert_selection.xpm"
+#include "../pixmaps/unselect_all.xpm"
 void Open_Cddb_Window (void)
 {
     GtkWidget *MainVBox, *VBox, *vbox, *hbox;
@@ -139,6 +154,7 @@ void Open_Cddb_Window (void)
     GtkWidget *Separator;
     GtkWidget *ScrollWindow;
     GtkWidget *PopupMenu;
+    GtkWidget *Icon;
     GtkTooltips *Tips;
     GList *History_List;
     //gint mw_x, mw_y;
@@ -205,6 +221,10 @@ void Open_Cddb_Window (void)
         (GtkSignalFunc)Cddb_Search_Album_List_From_String,NULL);
     gtk_entry_set_text_1(CddbSearchStringEntry,"");
 
+    // Set content of the clipboard if available
+    gtk_editable_paste_clipboard(GTK_EDITABLE(GTK_COMBO(CddbSearchStringEntry)->entry));
+    gtk_signal_connect(GTK_OBJECT(GTK_COMBO(CddbSearchStringEntry)->entry),"grab-focus",Focus_Select_Text_In_Editable,NULL);
+    
     CddbStopSearchButton = Create_Button_With_Icon_And_Label(stop_xpm,NULL);
     gtk_box_pack_start(GTK_BOX(hbox),CddbStopSearchButton,FALSE,FALSE,0);
     gtk_button_set_relief(GTK_BUTTON(CddbStopSearchButton),GTK_RELIEF_NONE);
@@ -368,6 +388,36 @@ void Open_Cddb_Window (void)
     gtk_signal_connect_object(GTK_OBJECT(CddbSearchStringInResultPrevButton),"clicked",(GtkSignalFunc)Add_To_Combo_Box_History,GTK_OBJECT(CddbSearchStringInResultEntry));
     gtk_tooltips_set_tip(Tips,CddbSearchStringInResultPrevButton,_("Search Previous"),NULL);
 
+    // Separator line
+    Separator = gtk_vseparator_new();
+    gtk_box_pack_start(GTK_BOX(hbox),Separator,FALSE,FALSE,2);
+
+    CddbDisplayRedLinesButton = gtk_toggle_button_new();
+    Icon = Create_Pixmap_Icon(red_lines_xpm);
+    gtk_container_add(GTK_CONTAINER(CddbDisplayRedLinesButton),Icon);
+    gtk_box_pack_start(GTK_BOX(hbox),CddbDisplayRedLinesButton,FALSE,FALSE,0);
+    gtk_button_set_relief(GTK_BUTTON(CddbDisplayRedLinesButton),GTK_RELIEF_NONE);
+    gtk_tooltips_set_tip(Tips,CddbDisplayRedLinesButton,_("Show only red lines/ Show all lines"),NULL);
+    gtk_signal_connect(GTK_OBJECT(CddbDisplayRedLinesButton),"toggled",(GtkSignalFunc)Cddb_Display_Red_Lines_In_Result,NULL);
+
+    CddbUnselectAllInResultButton = Create_Button_With_Icon_And_Label(unselect_all_xpm,NULL);
+    gtk_box_pack_end(GTK_BOX(hbox),CddbUnselectAllInResultButton,FALSE,FALSE,0);
+    gtk_button_set_relief(GTK_BUTTON(CddbUnselectAllInResultButton),GTK_RELIEF_NONE);
+    gtk_tooltips_set_tip(Tips,CddbUnselectAllInResultButton,_("Unselect all lines"),NULL);
+    gtk_signal_connect(GTK_OBJECT(CddbUnselectAllInResultButton),"clicked",(GtkSignalFunc)Cddb_Track_Clist_Unselect_All_Lines,NULL);
+
+    CddbInvertSelectionInResultButton = Create_Button_With_Icon_And_Label(invert_selection_xpm,NULL);
+    gtk_box_pack_end(GTK_BOX(hbox),CddbInvertSelectionInResultButton,FALSE,FALSE,0);
+    gtk_button_set_relief(GTK_BUTTON(CddbInvertSelectionInResultButton),GTK_RELIEF_NONE);
+    gtk_tooltips_set_tip(Tips,CddbInvertSelectionInResultButton,_("Invert lines selection"),NULL);
+    gtk_signal_connect(GTK_OBJECT(CddbInvertSelectionInResultButton),"clicked",(GtkSignalFunc)Cddb_Track_Clist_Invert_Selection,NULL);
+
+    CddbSelectAllInResultButton = Create_Button_With_Icon_And_Label(select_all_xpm,NULL);
+    gtk_box_pack_end(GTK_BOX(hbox),CddbSelectAllInResultButton,FALSE,FALSE,0);
+    gtk_button_set_relief(GTK_BUTTON(CddbSelectAllInResultButton),GTK_RELIEF_NONE);
+    gtk_tooltips_set_tip(Tips,CddbSelectAllInResultButton,_("Select all lines"),NULL);
+    gtk_signal_connect(GTK_OBJECT(CddbSelectAllInResultButton),"clicked",(GtkSignalFunc)Cddb_Track_Clist_Select_All_Lines,NULL);
+
 
     /*
      * Result of research
@@ -405,8 +455,9 @@ void Open_Cddb_Window (void)
     gtk_container_add(GTK_CONTAINER(ScrollWindow),CddbTrackCList);
     gtk_clist_set_selection_mode(GTK_CLIST(CddbTrackCList),GTK_SELECTION_EXTENDED);
     gtk_clist_column_titles_passive(GTK_CLIST(CddbTrackCList));
-    //gtk_clist_set_reorderable(GTK_CLIST(CddbTrackCList),FALSE);
+    gtk_clist_set_reorderable(GTK_CLIST(CddbTrackCList),TRUE);
     gtk_signal_connect(GTK_OBJECT(CddbTrackCList),"select_row",(GtkSignalFunc)Cddb_Track_List_Row_Selected,NULL);
+    gtk_signal_connect(GTK_OBJECT(CddbTrackCList),"unselect_row",(GtkSignalFunc)Cddb_Track_List_Row_Unselected,NULL);
     gtk_tooltips_set_tip(Tips,CddbTrackCList,_("Select lines to 'apply' to your files list. "
         "Note that all lines will be processed if no line is selected."),NULL);
 
@@ -416,6 +467,10 @@ void Open_Cddb_Window (void)
     gtk_clist_set_column_justification(GTK_CLIST(CddbTrackCList),0,GTK_JUSTIFY_RIGHT);
     gtk_clist_set_column_justification(GTK_CLIST(CddbTrackCList),2,GTK_JUSTIFY_RIGHT);
 
+    gtk_clist_column_title_active (GTK_CLIST(CddbTrackCList),0);
+    gtk_clist_column_title_active (GTK_CLIST(CddbTrackCList),1);
+    gtk_signal_connect(GTK_OBJECT(CddbTrackCList),"click_column",(GtkSignalFunc)Cddb_Track_List_Column_Selected,NULL);
+
     // Create Popup Menu on CddbTrackCList
     PopupMenu = Create_Cddb_Track_Clist_Popup_Menu(GTK_CLIST (CddbTrackCList));
 
@@ -486,6 +541,7 @@ void Open_Cddb_Window (void)
     CddbApplyButton = Create_Button_With_Pixmap(BUTTON_APPLY);
     gtk_box_pack_end(GTK_BOX(hbox),CddbApplyButton,FALSE,FALSE,2);
     gtk_signal_connect(GTK_OBJECT(CddbApplyButton),"clicked",(GtkSignalFunc)Cddb_Set_Track_Infos_To_File_List,NULL);
+    gtk_tooltips_set_tip(Tips,CddbApplyButton,_("Load the selected lines or all lines (if no line selected)."),NULL);
 
 
     /*
@@ -718,7 +774,7 @@ void Cddb_Search_String_In_Result (GtkWidget *entry, GtkButton *button)
         // Search in the album list (from top to bottom)
         if ( (selection=GTK_CLIST(CddbAlbumCList)->selection) != NULL )
         {
-            row = (gint)selection->data + 1;
+            row = GPOINTER_TO_INT(selection->data) + 1;
             gtk_clist_unselect_row(GTK_CLIST(CddbAlbumCList),row-1,0);
         }else
         {
@@ -754,7 +810,7 @@ void Cddb_Search_String_In_Result (GtkWidget *entry, GtkButton *button)
         // Search in the album list (from bottom to top)
         if ( (selection=GTK_CLIST(CddbAlbumCList)->selection) != NULL )
         {
-            row = (gint)selection->data - 1;
+            row = GPOINTER_TO_INT(selection->data) - 1;
             gtk_clist_unselect_row(GTK_CLIST(CddbAlbumCList),row+1,0);
         }else
         {
@@ -792,7 +848,7 @@ void Cddb_Search_String_In_Result (GtkWidget *entry, GtkButton *button)
 /*
  * Set a row visible in the clist (by scrolling the list) if the row is hidden
  */
-void Cddb_Album_List_Set_Row_Visible (GtkCList *clist, gint row)
+void Cddb_Album_List_Set_Row_Visible (GtkCList *clist, guint row)
 {
     GtkVisibility visible = gtk_clist_row_is_visible(clist,row);
     switch (visible)
@@ -810,7 +866,7 @@ void Cddb_Album_List_Set_Row_Visible (GtkCList *clist, gint row)
 /*
  * Show collected infos of the album in the status bar
  */
-void Cddb_Show_Album_Info (GtkCList *clist, gint row, gint column)
+void Cddb_Show_Album_Info (GtkCList *clist, guint row, gint column)
 {
     CddbAlbum *cddbalbum = NULL;
     gchar *msg;
@@ -832,9 +888,26 @@ void Cddb_Show_Album_Info (GtkCList *clist, gint row, gint column)
 /*
  * Select the corresponding file into the main file list
  */
-void Cddb_Track_List_Row_Selected (GtkCList *clist, gint row, gint column)
+void Cddb_Track_List_Row_Selected (GtkCList *clist, guint row, gint column)
 {
-    Action_Select_Nth_File_By_Position(row+1);
+    GList *SelectedRow = NULL;
+
+    Browser_List_Unselect_All_Files();
+    
+    SelectedRow = ((GtkCList*)clist)->selection;
+    while (SelectedRow)
+    {
+        Browser_List_Select_File_By_Row(row,TRUE);
+        Action_Select_Nth_File_By_Position(GPOINTER_TO_INT(SelectedRow->data)+1);
+        SelectedRow = SelectedRow->next;
+    }
+}
+/*
+ * Callback to unselect-row event
+ */
+void Cddb_Track_List_Row_Unselected (GtkCList *clist, guint row, gint column)
+{
+    Browser_List_Unselect_File_By_Row(row);
 }
 
 
@@ -870,13 +943,21 @@ GtkWidget *Create_Cddb_Track_Clist_Popup_Menu (GtkCList *clist)
     gtk_menu_append(GTK_MENU(CddbPopupMenu),MenuItem);
     gtk_signal_connect(GTK_OBJECT(MenuItem),"activate",(GtkSignalFunc)Cddb_Track_Clist_Unselect_All_Lines,NULL);
 
-    MenuItem = gtk_menu_item_new();
-    gtk_menu_append(GTK_MENU(CddbPopupMenu),MenuItem);
-
     MenuItem = gtk_menu_item_new_with_label(_("Invert selection"));
     gtk_menu_append(GTK_MENU(CddbPopupMenu),MenuItem);
     gtk_signal_connect(GTK_OBJECT(MenuItem),"activate",(GtkSignalFunc)Cddb_Track_Clist_Invert_Selection,NULL);
 
+    MenuItem = gtk_menu_item_new();
+    gtk_menu_append(GTK_MENU(CddbPopupMenu),MenuItem);
+
+    MenuItem = gtk_menu_item_new_with_label(_("Sort by Track Number"));
+    gtk_menu_append(GTK_MENU(CddbPopupMenu),MenuItem);
+    gtk_signal_connect(GTK_OBJECT(MenuItem),"activate",(GtkSignalFunc)Cddb_Track_Clist_Sort_By_Ascending_Track_Number,NULL);
+
+    MenuItem = gtk_menu_item_new_with_label(_("Sort by Track Name"));
+    gtk_menu_append(GTK_MENU(CddbPopupMenu),MenuItem);
+    gtk_signal_connect(GTK_OBJECT(MenuItem),"activate",(GtkSignalFunc)Cddb_Track_Clist_Sort_By_Ascending_Track_Name,NULL);
+
     gtk_widget_show_all(CddbPopupMenu);
     return CddbPopupMenu;
 }
@@ -887,6 +968,7 @@ void Cddb_Track_Clist_Select_All_Lines (void)
     gtk_signal_handler_block_by_func(GTK_OBJECT(CddbTrackCList),GTK_SIGNAL_FUNC(Cddb_Track_List_Row_Selected),NULL);
     gtk_clist_select_all(GTK_CLIST(CddbTrackCList));
     gtk_signal_handler_unblock_by_func(GTK_OBJECT(CddbTrackCList),GTK_SIGNAL_FUNC(Cddb_Track_List_Row_Selected),NULL);
+    gtk_signal_emit_by_name(GTK_OBJECT(CddbTrackCList),"select-row");
 }
 void Cddb_Track_Clist_Unselect_All_Lines (void)
 {
@@ -909,6 +991,67 @@ void Cddb_Track_Clist_Invert_Selection (void)
         list = list->next;
     }
     gtk_signal_handler_unblock_by_func(GTK_OBJECT(CddbTrackCList),GTK_SIGNAL_FUNC(Cddb_Track_List_Row_Selected),NULL);
+
+    // To select lines in the BrowserList
+    if ( (list = ((GtkCList*)CddbTrackCList)->selection) )
+        gtk_clist_select_row(GTK_CLIST(CddbTrackCList),GPOINTER_TO_INT(list->data),0);
+        //gtk_signal_emit_by_name(GTK_OBJECT(CddbTrackCList),"select-row");
+    else
+        gtk_signal_emit_by_name(GTK_OBJECT(CddbTrackCList),"unselect-row");
+}
+
+void Cddb_Track_Clist_Sort_By_Ascending_Track_Number (void)
+{
+    // Sort ascending by column '#'
+    gtk_clist_set_sort_column(GTK_CLIST(CddbTrackCList),0);
+    gtk_clist_set_sort_type(GTK_CLIST(CddbTrackCList),GTK_SORT_ASCENDING);
+    gtk_clist_sort(GTK_CLIST(CddbTrackCList));
+}
+void Cddb_Track_Clist_Sort_By_Descending_Track_Number (void)
+{
+    // Sort ascending by column '#'
+    gtk_clist_set_sort_column(GTK_CLIST(CddbTrackCList),0);
+    gtk_clist_set_sort_type(GTK_CLIST(CddbTrackCList),GTK_SORT_DESCENDING);
+    gtk_clist_sort(GTK_CLIST(CddbTrackCList));
+}
+
+void Cddb_Track_Clist_Sort_By_Ascending_Track_Name (void)
+{
+    // Sort ascending by column 'Track Name'
+    gtk_clist_set_sort_column(GTK_CLIST(CddbTrackCList),1);
+    gtk_clist_set_sort_type(GTK_CLIST(CddbTrackCList),GTK_SORT_ASCENDING);
+    gtk_clist_sort(GTK_CLIST(CddbTrackCList));
+}
+void Cddb_Track_Clist_Sort_By_Descending_Track_Name (void)
+{
+    // Sort ascending by column 'Track Name'
+    gtk_clist_set_sort_column(GTK_CLIST(CddbTrackCList),1);
+    gtk_clist_set_sort_type(GTK_CLIST(CddbTrackCList),GTK_SORT_DESCENDING);
+    gtk_clist_sort(GTK_CLIST(CddbTrackCList));
+}
+
+void Cddb_Track_List_Column_Selected (GtkCList *clist, gint column, gpointer user_data)
+{
+    static gboolean col0 = FALSE;
+    static gboolean col1 = FALSE;
+    
+    if (column == 0) // Sort by column '#'
+    {
+        if (col0)
+            Cddb_Track_Clist_Sort_By_Descending_Track_Number();
+        else
+            Cddb_Track_Clist_Sort_By_Ascending_Track_Number();
+        col0 = !col0;
+        col1 = FALSE;
+    }else if (column == 1) // Sort by column 'Track Name'
+    {
+        if (col1)
+            Cddb_Track_Clist_Sort_By_Descending_Track_Name();
+        else
+            Cddb_Track_Clist_Sort_By_Ascending_Track_Name();
+        col1 = !col1;
+        col0 = FALSE;
+    }    
 }
 
 
@@ -1152,13 +1295,11 @@ gboolean Cddb_Free_Track_Album_List (GList *track_list)
 /*
  * Load the CddbAlbumList into the corresponding CList
  */
-void Cddb_Load_Album_List (void)
+void Cddb_Load_Album_List (gboolean only_red_lines)
 {
     if (CddbAlbumList && CddbAlbumCList)
     {
-        GdkColor GREY = {0, 0xaa00, 0xaa00, 0xaa00};
         GList *cddbalbumlist = g_list_first(CddbAlbumList);
-        GtkStyle *style = gtk_style_copy(gtk_widget_get_style(CddbAlbumCList));
         
         gtk_clist_clear(GTK_CLIST(CddbAlbumCList));
         while (cddbalbumlist)
@@ -1166,21 +1307,19 @@ void Cddb_Load_Album_List (void)
             gchar *row_text[2];
             gint   new_row;
             CddbAlbum *cddbalbum = cddbalbumlist->data;
-    
-            row_text[0] = cddbalbum->artist_album;
-            row_text[1] = cddbalbum->category;
-            
-            // Load the row in the list
-            new_row = gtk_clist_append(GTK_CLIST(CddbAlbumCList),row_text);
-            gtk_clist_set_row_data(GTK_CLIST(CddbAlbumCList),new_row,cddbalbum);
-            
-            // For other versions of the same album
-            if ( cddbalbum->other_version == TRUE )
+        
+            if ( (only_red_lines && cddbalbum->track_list) || !only_red_lines)
             {
-                style->fg[GTK_STATE_NORMAL] = GREY;
-                gtk_clist_set_row_style(GTK_CLIST(CddbAlbumCList),new_row,style);
+                row_text[0] = cddbalbum->artist_album;
+                row_text[1] = cddbalbum->category;
+                
+                // Load the row in the list
+                new_row = gtk_clist_append(GTK_CLIST(CddbAlbumCList),row_text);
+                gtk_clist_set_row_data(GTK_CLIST(CddbAlbumCList),new_row,cddbalbum);
+                
+                // Set color of the selected row
+                Cddb_Album_List_Set_Row_Color(GTK_CLIST(CddbAlbumCList),new_row);
             }
-
             cddbalbumlist = cddbalbumlist->next;
         }
     }
@@ -1478,8 +1617,11 @@ gboolean Cddb_Search_Album_List_From_String (void)
     gtk_statusbar_push(GTK_STATUSBAR(CddbStatusBar),CddbStatusBarContext,msg);
     g_free(msg);
 
+    // Initialize the button
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(CddbDisplayRedLinesButton),FALSE);
+    
     // Load the albums found in the list
-    Cddb_Load_Album_List();
+    Cddb_Load_Album_List(FALSE);
 
     return TRUE;
 }
@@ -1504,12 +1646,12 @@ gboolean Cddb_Get_Album_Tracks_List_CB (GtkCList *clist, gint row, gint column)
 }
 gboolean Cddb_Get_Album_Tracks_List (GtkCList *clist, gint row, gint column)
 {
-    gint   socket_id;
+    gint       socket_id;
     CddbAlbum *cddbalbum = NULL;
-    GList *TrackOffsetList = NULL;
-    gchar *cddb_in, *cddb_out, *msg;
-    gint   bytes_written, bytes_read, bytes_read1, bytes_read_total;
-    gboolean read_track_offset = FALSE;
+    GList     *TrackOffsetList = NULL;
+    gchar     *cddb_in, *cddb_out, *msg;
+    gint       bytes_written, bytes_read, bytes_read1, bytes_read_total;
+    gboolean   read_track_offset = FALSE;
 
 
     gtk_clist_clear(GTK_CLIST(CddbTrackCList));
@@ -1531,7 +1673,7 @@ gboolean Cddb_Get_Album_Tracks_List (GtkCList *clist, gint row, gint column)
         return FALSE;
 
     // http://www.freedb.org/~cddb/cddb.cgi?cmd...
-	cddb_in = g_strdup_printf("GET http://%s%s?cmd=cddb+read+"
+    cddb_in = g_strdup_printf("GET http://%s%s?cmd=cddb+read+"
                               "%s+%s"
                               "&hello=noname+localhost+%s+%s"
                               "&proto=1 HTTP/1.1\r\n"
@@ -1674,21 +1816,8 @@ gboolean Cddb_Get_Album_Tracks_List (GtkCList *clist, gint row, gint column)
     // Close connection
     Cddb_Close_Connection(socket_id);
 
-    // Set forecolor of the selected row to red (light red for an other version of the album)
-    // to show that we've already the track list for this album
-    if ( cddbalbum->other_version == TRUE )
-    {
-        GdkColor LIGHT_RED = {0, 0xffff, 0x7f00, 0x7f00};
-        GtkStyle *style = gtk_style_copy(gtk_widget_get_style(CddbAlbumCList));
-        style->fg[GTK_STATE_NORMAL] = LIGHT_RED;
-        gtk_clist_set_row_style(GTK_CLIST(CddbAlbumCList),row,style);
-    }else
-    {
-        GdkColor RED = {0, 0xffff, 0x0000, 0x0000};
-        GtkStyle *style = gtk_style_copy(gtk_widget_get_style(CddbAlbumCList));
-        style->fg[GTK_STATE_NORMAL] = RED;
-        gtk_clist_set_row_style(GTK_CLIST(CddbAlbumCList),row,style);
-    }
+    // Set color of the selected row (without reloading the whole list)
+    Cddb_Album_List_Set_Row_Color(GTK_CLIST(CddbAlbumCList),row);
 
     // Load the track list of the album
     gtk_statusbar_push(GTK_STATUSBAR(CddbStatusBar),CddbStatusBarContext,_("Loading album track list ..."));
@@ -1711,7 +1840,45 @@ gboolean Cddb_Get_Album_Tracks_List (GtkCList *clist, gint row, gint column)
     return TRUE;
 }
 
+void Cddb_Album_List_Set_Row_Color (GtkCList *clist, gint row)
+{
+    CddbAlbum *cddbalbum = NULL;
+    GtkStyle *style;
+    GdkColor LIGHT_RED = {0, 0xffff, 0x7f00, 0x7f00};
+    GdkColor RED       = {0, 0xffff, 0x0000, 0x0000};
+    GdkColor GREY      = {0, 0xaa00, 0xaa00, 0xaa00};
 
+    cddbalbum = gtk_clist_get_row_data(GTK_CLIST(clist),row);
+    style = gtk_style_copy(gtk_widget_get_style(CddbAlbumCList));
+
+    // Set forecolor of the selected row to red (light red for an other version of the album)
+    // to show that we've already the track list for this album
+    if (cddbalbum->track_list != NULL)
+    {
+        // The line have infos about the album...
+        if ( cddbalbum->other_version == TRUE )
+        {
+            style->fg[GTK_STATE_NORMAL]   = LIGHT_RED;
+            style->bg[GTK_STATE_SELECTED] = LIGHT_RED;
+            gtk_clist_set_row_style(GTK_CLIST(CddbAlbumCList),row,style);
+        }else
+        {
+            style->fg[GTK_STATE_NORMAL]   = RED;
+            style->bg[GTK_STATE_SELECTED] = RED;
+            gtk_clist_set_row_style(GTK_CLIST(CddbAlbumCList),row,style);
+        }
+    }else
+    {
+        if ( cddbalbum->other_version == TRUE )
+        {
+            style->fg[GTK_STATE_NORMAL] = GREY;
+            gtk_clist_set_row_style(GTK_CLIST(CddbAlbumCList),row,style);
+        }else
+        {
+            gtk_clist_set_row_style(GTK_CLIST(CddbAlbumCList),row,NULL);
+        }
+    }
+}
 
 
 /*
@@ -1732,7 +1899,7 @@ gboolean Cddb_Set_Track_Infos_To_File_List (void)
     if (!CddbTrackCList || !ETFileList) return FALSE;
     
     // Save the current displayed data
-    ET_Save_File_Data_From_UI((ET_File *)ETFileList->data);
+    ET_Save_File_Data_From_UI(ETFileDisplayed);
 
     etfilelist = g_list_first(ETFileList);
     list_length = GTK_CLIST(CddbTrackCList)->rows;
@@ -1777,7 +1944,7 @@ gboolean Cddb_Set_Track_Infos_To_File_List (void)
     }
 
     etfilelist = g_list_first(ETFileList);
-    for (clist_row=0;clist_row<GTK_CLIST(CddbTrackCList)->rows;clist_row++)
+    for (clist_row=0;clist_row<(guint)GTK_CLIST(CddbTrackCList)->rows;clist_row++)
     {
         list = g_list_nth(GTK_CLIST(CddbTrackCList)->row_list,clist_row);
         if (CddbTrackCList_Line_Selected == FALSE || GTK_CLIST_ROW(list)->state == GTK_STATE_SELECTED)
@@ -1787,60 +1954,72 @@ gboolean Cddb_Set_Track_Infos_To_File_List (void)
             if (cddbtrackalbum && etfilelist)
             {
                 ET_File   *etfile   = (ET_File *)etfilelist->data;
-                File_Name *FileName = ET_File_Name_Item_New();
-                File_Tag  *FileTag  = ET_File_Tag_Item_New();
-                ET_Copy_File_Tag_Item(etfilelist->data,FileTag);
+                File_Name *FileName = NULL;
+                File_Tag  *FileTag  = NULL;
 
+                /*
+                 * Tag fields
+                 */
+                if (cddbsettoallfields || cddbsettotitle      || cddbsettotitle
+                ||  cddbsettoartist    || cddbsettoalbum      || cddbsettoyear
+                ||  cddbsettotrack     || cddbsettotracktotal || cddbsettogenre)
+                {
+                    // Allocation of a new FileTag
+                    FileTag = ET_File_Tag_Item_New();
+                    ET_Copy_File_Tag_Item(etfilelist->data,FileTag);
+                }
+                
                 if (cddbsettoallfields || cddbsettotitle)
-                    ET_Set_Field_File_Tag_Item((gint *)&FileTag->title,cddbtrackalbum->track_name);
+                    ET_Set_Field_File_Tag_Item(&FileTag->title,cddbtrackalbum->track_name);
+
+                if ( (cddbsettoallfields || cddbsettoartist) && cddbtrackalbum->cddbalbum->artist)
+                    ET_Set_Field_File_Tag_Item(&FileTag->artist,cddbtrackalbum->cddbalbum->artist);
+
+                if ( (cddbsettoallfields || cddbsettoalbum) && cddbtrackalbum->cddbalbum->album)
+                    ET_Set_Field_File_Tag_Item(&FileTag->album, cddbtrackalbum->cddbalbum->album);
+
+                if ( (cddbsettoallfields || cddbsettoyear) && cddbtrackalbum->cddbalbum->year)
+                    ET_Set_Field_File_Tag_Item(&FileTag->year,  cddbtrackalbum->cddbalbum->year);
 
                 if (cddbsettoallfields || cddbsettotrack)
                 {
-                    if (NUMBER_TRACK_FORMATED) snprintf(buffer,sizeof(buffer),"%.2d",cddbtrackalbum->track_number);
+                    if (NUMBER_TRACK_FORMATED) snprintf(buffer,sizeof(buffer),"%.*d",NUMBER_TRACK_FORMATED_SPIN_BUTTON,cddbtrackalbum->track_number);
                     else                       snprintf(buffer,sizeof(buffer),"%d",  cddbtrackalbum->track_number);
-                    ET_Set_Field_File_Tag_Item((gint *)&FileTag->track,buffer);
+                    ET_Set_Field_File_Tag_Item(&FileTag->track,buffer);
                 }            
 
                 if (cddbsettoallfields || cddbsettotracktotal)
                 {
-                    if (NUMBER_TRACK_FORMATED) snprintf(buffer,sizeof(buffer),"%.2d",list_length);
+                    if (NUMBER_TRACK_FORMATED) snprintf(buffer,sizeof(buffer),"%.*d",NUMBER_TRACK_FORMATED_SPIN_BUTTON,list_length);
                     else                       snprintf(buffer,sizeof(buffer),"%d",  list_length);
-                    ET_Set_Field_File_Tag_Item((gint *)&FileTag->track_total,buffer);
+                    ET_Set_Field_File_Tag_Item(&FileTag->track_total,buffer);
                 }
             
-                if ( (cddbsettoallfields || cddbsettoalbum) && cddbtrackalbum->cddbalbum->album)
-                    ET_Set_Field_File_Tag_Item((gint *)&FileTag->album, cddbtrackalbum->cddbalbum->album);
-
-                if ( (cddbsettoallfields || cddbsettoartist) && cddbtrackalbum->cddbalbum->artist)
-                    ET_Set_Field_File_Tag_Item((gint *)&FileTag->artist,cddbtrackalbum->cddbalbum->artist);
-
                 if ( (cddbsettoallfields || cddbsettogenre) && cddbtrackalbum->cddbalbum->genre)
-                    ET_Set_Field_File_Tag_Item((gint *)&FileTag->genre, cddbtrackalbum->cddbalbum->genre);
-
-                if ( (cddbsettoallfields || cddbsettoyear) && cddbtrackalbum->cddbalbum->year)
-                    ET_Set_Field_File_Tag_Item((gint *)&FileTag->year,  cddbtrackalbum->cddbalbum->year);
+                    ET_Set_Field_File_Tag_Item(&FileTag->genre, cddbtrackalbum->cddbalbum->genre);
 
+                /*
+                 * Filename field
+                 */
                 if ( (cddbsettoallfields || cddbsettofilename) )
                 {
+                    gchar *filename_generated, *filename_new = NULL;
+
+                    // Allocation of a new FileName
+                    FileName = ET_File_Name_Item_New();
+                    
                     // Build the filename with the path
-                    gchar *dirname, *filename_generated, *filename_new = NULL;
-                    if (NUMBER_TRACK_FORMATED) snprintf(buffer,sizeof(buffer),"%.2d",cddbtrackalbum->track_number);
+                    if (NUMBER_TRACK_FORMATED) snprintf(buffer,sizeof(buffer),"%.*d",NUMBER_TRACK_FORMATED_SPIN_BUTTON,cddbtrackalbum->track_number);
                     else                       snprintf(buffer,sizeof(buffer),"%d",  cddbtrackalbum->track_number);
                     filename_generated = g_strconcat(buffer," - ",cddbtrackalbum->track_name,NULL);
                     ET_File_Name_Convert_Character(filename_generated); // Replace invalid characters
-                    dirname = g_dirname(((File_Name *)etfile->FileNameNew->data)->value);
-                    if (dirname)
-                    {
-                        if ( strcmp(dirname,"/")==0 ) // Root directory?
-                            filename_new = g_strconcat("/",filename_generated,etfile->ETFileDescription->Extension,NULL);
-                        else
-                            filename_new = g_strconcat(dirname,"/",filename_generated,etfile->ETFileDescription->Extension,NULL);
-                        ET_Set_Field_File_Name_Item((gint *)&FileName->value,filename_new);
-                    }
+                    filename_new = ET_Generate_File_Name(etfile,filename_generated);
+                    ET_Set_Field_File_Name_Item(&FileName->value,filename_new);
                     g_free(filename_generated);
                     if (filename_new) g_free(filename_new);
                 }
 
+
                 ET_Manage_Changes_Of_File_Data(etfile,FileName,FileTag);
 
                 // Then run current scanner if asked...
@@ -1852,9 +2031,23 @@ gboolean Cddb_Set_Track_Infos_To_File_List (void)
     }
     
     Browser_List_Refresh_Whole_List();
-    ET_Display_File_Data_To_UI((ET_File *)ETFileList->data);
+    ET_Display_File_Data_To_UI(ETFileDisplayed);
 
     return TRUE;
 }
 
 
+void Cddb_Display_Red_Lines_In_Result (void)
+{
+    if (!CddbDisplayRedLinesButton) return;
+
+    if ( gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(CddbDisplayRedLinesButton)) )
+    {
+        // Show only red lines
+        Cddb_Load_Album_List(TRUE);
+    }else
+    {
+        // Show all lines
+        Cddb_Load_Album_List(FALSE);
+    }
+}
