@@ -1,7 +1,7 @@
 /* et_core.c - 2001/10/21 */
 /*
- *  EasyTAG - Tag editor for MP3 and OGG files
- *  Copyright (C) 2000-2002  Jerome Couderc <j.couderc@ifrance.com>
+ *  EasyTAG - Tag editor for MP3 and Ogg Vorbis files
+ *  Copyright (C) 2000-2003  Jerome Couderc <j.couderc@ifrance.com>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -32,7 +32,10 @@
 #include "et_core.h"
 #include "mpeg_header.h"
 #include "flac_header.h"
+#include "monkeyaudio_header.h"
+#include "musepack_header.h"
 #include "id3_tag.h"
+#include "ape_tag.h"
 #ifdef ENABLE_OGG
 #    include "ogg_tag.h"
 #endif
@@ -48,6 +51,7 @@
  * Declaration *
  ***************/
 
+ET_File *ETFileDisplayed = NULL;
 
 
 /**************
@@ -80,11 +84,14 @@ gboolean ET_Copy_File_Tag_Item_Other_Field (ET_File *ETFile, File_Tag *FileTag);
 //gboolean ET_Set_Field_File_Name_Item (gint *FileNameField, gchar *value);
 //gboolean ET_Set_Field_File_Tag_Item  (gint *FileTagField, gchar *value);
 
-gulong   ET_File_Key_New (void);
-gulong   ET_Undo_Key_New (void);
+guint    ET_File_Key_New     (void);
+guint    ET_Undo_Key_New     (void);
 
-void     ET_Display_File_And_List_Status_To_UI (gchar *filename);
-void     ET_Display_Filename_To_UI             (gchar *filename);
+void     ET_File_List_Number (void);
+GList   *ET_File_List_Remove (GList *item_to_remove);
+
+void     ET_Display_File_And_List_Status_To_UI (ET_File *ETFile);
+void     ET_Display_Filename_To_UI             (ET_File *ETFile);
 gboolean ET_Display_File_Tag_To_UI             (ET_File *ETFile);
 gboolean ET_Display_File_Info_To_UI            (ET_File_Info *ETFileInfo);
 
@@ -106,6 +113,8 @@ gboolean ET_Add_File_To_History_List    (ET_File *ETFile);
 gboolean ET_Read_File_Info (gchar *filename, ET_File_Info *ETFileInfo);
 //gboolean ET_File_Name_Convert_Character (gchar *filename);
 
+gint ET_Comp_Func_Sort_File_By_Ascending_Index_Key (ET_File *ETFile1, ET_File *ETFile2);
+
 
 /*******************
  * Basic functions *
@@ -129,7 +138,7 @@ gchar *ET_Get_File_Extension (gchar *filename)
  */
 ET_File_Description *ET_Get_File_Description_From_Extension (gchar *extension)
 {
-    gint i;
+    guint i;
     
     if (!extension)
         return &ETFileDescription[ET_FILE_DESCRIPTION_SIZE];
@@ -248,6 +257,7 @@ void ET_Initialize_File_Item (ET_File *ETFile)
 {
     if (ETFile)
     {
+        ETFile->IndexKey          = 0;
         ETFile->ETFileKey         = 0;
         ETFile->ETFileDescription = NULL;    
         ETFile->ETFileInfo        = NULL;
@@ -263,16 +273,16 @@ void ET_Initialize_File_Item (ET_File *ETFile)
 
 
 /* Key for each item of ETFileList */
-gulong ET_File_Key_New (void)
+guint ET_File_Key_New (void)
 {
-    static gulong ETFileKey = 0;
+    static guint ETFileKey = 0;
     return ++ETFileKey;
 }
 
 /* Key for Undo */
-gulong ET_Undo_Key_New (void)
+guint ET_Undo_Key_New (void)
 {
-    static gulong ETUndoKey = 0;
+    static guint ETUndoKey = 0;
     return ++ETUndoKey;
 }
 
@@ -293,8 +303,8 @@ GList *ET_Add_File_To_File_List (gchar *filename)
     File_Name    *FileName;
     File_Tag     *FileTag;
     ET_File_Info *ETFileInfo;
-    gulong        ETFileKey;
-    gulong        undo_key;
+    guint         ETFileKey;
+    guint         undo_key;
 
     
     if (!filename)
@@ -325,9 +335,12 @@ GList *ET_Add_File_To_File_List (gchar *filename)
             Ogg_Tag_Read_File_Tag(filename,FileTag);
             break;
 #endif
+        case APE_TAG:
+            Ape_Tag_Read_File_Tag(filename,FileTag);
+            break;
         case UNKNOWN_TAG:
         default:
-            g_print("FileTag: Undefined tag type %d for file %s.\n",ETFileDescription->TagType,filename);
+            g_print("FileTag: Undefined tag type (%d) for file %s\n",ETFileDescription->TagType,filename);
             break;
     }
 
@@ -347,16 +360,23 @@ GList *ET_Add_File_To_File_List (gchar *filename)
 #endif
         case FLAC_FILE:
             Flac_Header_Read_File_Info(filename,ETFileInfo);
-            //ET_Read_File_Info(filename,ETFileInfo); // To get al least the file size
+            break;
+        case MPC_FILE:
+            Mpc_Header_Read_File_Info(filename,ETFileInfo);
+            break;
+        case MAC_FILE:
+            Mac_Header_Read_File_Info(filename,ETFileInfo);
             break;
         case UNKNOWN_FILE:
         default:
-            g_print("ETFileInfo: Undefined file type %d for file %s.\n",ETFileDescription->FileType,filename);
+            g_print("ETFileInfo: Undefined file type (%d) for file %s\n",ETFileDescription->FileType,filename);
+            ET_Read_File_Info(filename,ETFileInfo); // To get at least the file size
             break;
     }
 
     /* Attach all data defined above to this ETFile item */
     ETFile = ET_File_Item_New();
+    ETFile->IndexKey          = 0; // Will be renumered after...
     ETFile->ETFileKey         = ETFileKey;
     ETFile->ETFileDescription = ETFileDescription;
     ETFile->FileNameList      = g_list_append(NULL,FileName);
@@ -372,6 +392,9 @@ GList *ET_Add_File_To_File_List (gchar *filename)
     /* Sort the file list */
     ET_Sort_File_List(SORTING_FILE_MODE);
 
+    /* To number the ETFile in the list */
+    ET_File_List_Number();
+
 
     /*
      * Process the filename and tag to generate undo if needed...
@@ -400,6 +423,35 @@ GList *ET_Add_File_To_File_List (gchar *filename)
 }
 
 
+/*
+ * Delete the corresponding file. Return TRUE if deleted.
+ */
+gboolean ET_Remove_File_From_File_List (ET_File *ETFile)
+{
+    if (ETFile)
+    {
+        // Now, we delete the file in the list
+        GList *list = g_list_first(ETFileList);
+        while(list)
+        {
+            if ( (ET_File *)list->data == ETFile )
+            {
+                ET_File_List_Remove(list);
+                break;
+            }
+            list = list->next;
+        }
+        
+        // To number the ETFile in the list
+        ET_File_List_Number();
+        return TRUE;
+    }else
+    {
+        return FALSE;
+    }
+}
+
+
 
 
 /**************************
@@ -411,6 +463,9 @@ GList *ET_Add_File_To_File_List (gchar *filename)
  */
 void ET_Sort_File_List (ET_Sorting_Type Sorting_Type)
 {
+    // Important to rewind before
+    ETFileList = ET_File_List_First();
+
     switch (Sorting_Type)
     {
         case SORTING_UNKNOWN:
@@ -499,6 +554,7 @@ void ET_Sort_File_List (ET_Sorting_Type Sorting_Type)
             ETFileList = g_list_sort(ETFileList,(GCompareFunc)ET_Comp_Func_Sort_File_By_Descending_File_Samplerate);
             break;
     }
+    
 }
 
 
@@ -509,16 +565,20 @@ void ET_Sort_File_List_And_Update_UI (ET_Sorting_Type Sorting_Type)
 {
     if (!ETFileList) return;
     
-    ET_Save_File_Data_From_UI((ET_File *)ETFileList->data);
+    ET_Save_File_Data_From_UI(ETFileDisplayed);
 
-    /* Go to the first item of the list */
-    ET_File_List_First();
+    /* Sort the list */
     ET_Sort_File_List(Sorting_Type);
-    
+
+    /* To number the ETFile in the list */
+    ET_File_List_Number();
+
     /* Reload files in browser list */
     Browser_List_Load_Files();
 
-    ET_Display_File_Data_To_UI((ET_File *)ETFileList->data);
+    ET_File_List_By_Etfile(ETFileDisplayed);  // Just to update 'ETFileList'
+    Browser_List_Select_File(ETFileDisplayed,TRUE);
+    ET_Display_File_Data_To_UI(ETFileDisplayed);
 
     Update_Command_Buttons_Sensivity();
 }
@@ -965,6 +1025,15 @@ gint ET_Comp_Func_Sort_File_By_Descending_File_Samplerate (ET_File *ETFile1, ET_
 }
 
 
+/*
+ * Comparison function for sorting by ascending IndexKey (used mainly to have the ETFileSelectionList already sorted.)
+ */
+gint ET_Comp_Func_Sort_File_By_Ascending_Index_Key (ET_File *ETFile1, ET_File *ETFile2)
+{
+    return (ETFile1->IndexKey - ETFile2->IndexKey);
+}
+
+
 
 
 /***************************
@@ -977,7 +1046,6 @@ gint ET_Comp_Func_Sort_File_By_Descending_File_Samplerate (ET_File *ETFile1, ET_
 GList *ET_File_List_First (void)
 {
     ETFileList = g_list_first(ETFileList);
-    ETFileList_Index = 1;
     return ETFileList;
 }
 
@@ -987,14 +1055,9 @@ GList *ET_File_List_First (void)
 GList *ET_File_List_Previous (void)
 {
     if (ETFileList && ETFileList->prev)
-    {
-        ETFileList_Index--;
         return ETFileList = ETFileList->prev;
-    }else
-    {
-        ETFileList_Index = 1;
+    else
         return NULL;
-    }
 }
 
 /*
@@ -1004,14 +1067,9 @@ GList *ET_File_List_Previous (void)
 GList *ET_File_List_Next (void)
 {
     if (ETFileList && ETFileList->next)
-    {
-        ETFileList_Index++;
         return ETFileList = ETFileList->next;
-    }else
-    {
-        ETFileList_Index = ETFileList_Length;
+    else
         return NULL;
-    }
 }
 
 /*
@@ -1020,10 +1078,57 @@ GList *ET_File_List_Next (void)
 GList *ET_File_List_Last (void)
 {
     ETFileList = g_list_last(ETFileList);
-    ETFileList_Index = ETFileList_Length;
     return ETFileList;
 }
 
+/*
+ * Returns the item of the "main list" which correspond to the given 'ETFile' (used into browser list).
+ */
+GList *ET_File_List_By_Etfile (ET_File *ETFile)
+{
+    GList *etfilelist;
+
+    etfilelist = ET_File_List_First();
+    while (etfilelist)
+    {
+        if ( ETFile == (ET_File *)etfilelist->data )
+            break;
+        etfilelist = ET_File_List_Next();
+    }
+    ETFileList = etfilelist; // To "save" the position like in ET_File_List_Next... (not very good - FIX ME)
+    return etfilelist;
+}
+
+/*
+ * Returns the item of the "main list" which have the position 'pos_in_list' in the list (used into CDDB result list).
+ * Range for 'pos_in_list' is 1 to ETFileList_Length.
+ */
+GList *ET_File_List_By_Position (gulong pos_in_list)
+{
+    GList *etfilelist;
+    guint i = 1;
+
+    etfilelist = ET_File_List_First();
+    while (etfilelist)
+    {
+        if (i == pos_in_list)
+            break;
+        etfilelist = ET_File_List_Next();
+        i++;
+    }
+    ETFileList = etfilelist; // To "save" the position like in ET_File_List_Next... (not very good - FIX ME)
+    return etfilelist;
+}
+
+/*
+ * Just returns the current item of the "main list"
+ */
+GList *ET_File_List_Current (void)
+{
+    return ETFileList;
+}
+
+
 /*
  * Remove an item from the "main list"
  */
@@ -1035,36 +1140,40 @@ GList *ET_File_List_Remove (GList *item_to_remove)
         return ETFileList;
 
     /* Get the item that will become the new current item, when 'item_to_remove' will be removed */
-    if (item_to_remove->next)
+    if (item_to_remove->next)       // Select the next item by default
         etfilelist = item_to_remove->next;
-    else if (item_to_remove->prev)
+    else if (item_to_remove->prev)  // It was the last, select the previous one
         etfilelist = item_to_remove->prev;
-    else
+    else                            // There is no more item
         etfilelist = NULL;
 
     /* Remove infos of the file */
     ETFileList_TotalSize     -= ((ET_File_Info *)((ET_File *)item_to_remove->data)->ETFileInfo)->size;
     ETFileList_TotalDuration -= ((ET_File_Info *)((ET_File *)item_to_remove->data)->ETFileInfo)->duration;
     
+    /* If the current displayed file is just removing, it will be unable to display it again! */
+    if (ETFileDisplayed == (ET_File *)item_to_remove->data)
+        ETFileDisplayed = NULL;
+    
     /* Free data of the file */
     g_list_remove_link(g_list_first(ETFileList),item_to_remove);
     ET_Free_File_List_Item(item_to_remove->data);
 
-    ETFileList = etfilelist;
+    ETFileList = etfilelist; // Put main list to the new position
      
     /* Recalculate length of list */
     ET_File_List_Get_Length();
     if (ETFileList && ETFileList->data)
     {
         /* Select the new file (synchronize index,...) */
-        ET_File_List_Nth_By_Key(((ET_File *)ETFileList->data)->ETFileKey);
+        ET_File_List_By_Etfile((ET_File *)ETFileList->data);
     }else
     {
         /* Reinit the tag and file area */
         Clear_File_Entry_Field();
         Clear_Header_Fields();
         Clear_Tag_Entry_Fields();
-        //ETFileList_Index = ETFileList_Length = 0;
+        ETFileList_Length = 0;
         gtk_label_set_text(GTK_LABEL(FileIndex),"0/0:");
         Update_Command_Buttons_Sensivity();
     }
@@ -1072,22 +1181,6 @@ GList *ET_File_List_Remove (GList *item_to_remove)
     return ETFileList;
 }
 
-/*
- * Returns the item of the "main list" which correspond to the given 'etfilekey' (was used into browser list).
- */
-GList *ET_File_List_Nth_By_Key (gulong etfilekey)
-{
-    ET_File_List_First();
-    while (ETFileList)
-    {
-        if ( etfilekey == ((ET_File *)ETFileList->data)->ETFileKey )
-            break;
-        if (ET_File_List_Next() == NULL)
-            return NULL;    /* Not found */
-    }
-    return ETFileList;
-}
-
 /*
  * Returns the length of the "main list"
  */
@@ -1100,6 +1193,39 @@ guint ET_File_List_Get_Length (void)
     return ETFileList_Length;
 }
 
+/*
+ * Renumberate the whole list (IndexKey) from 1 to n
+ */
+void ET_File_List_Number (void)
+{
+    GList *list = NULL;
+    guint i = 1;
+    
+    list = g_list_first(ETFileList);
+    while (list)
+    {
+        ((ET_File *)list->data)->IndexKey = i++;
+        list = list->next;
+    }
+}
+
+/*
+ * Returns the list of selected files in the BrowserClist (equal to ETFileList filtrated)
+ */
+GList *ET_File_List_Get_Selection (void)
+{
+    ET_Free_Selection_File_List();
+
+    ETFileSelectionList = Browser_List_Get_Selection();
+    // Sort the list using IndexKey (FIX ME : should be also sorted after sorting the main list)
+    ETFileSelectionList = g_list_sort(ETFileSelectionList,(GCompareFunc)ET_Comp_Func_Sort_File_By_Ascending_Index_Key);
+    ETFileSelectionList_Length = g_list_length(ETFileSelectionList);
+
+    //ET_Debug_Print_File_List(ETFileSelectionList,__FILE__,__LINE__,"");
+
+    return ETFileSelectionList;
+}
+
 
 
 
@@ -1112,17 +1238,19 @@ guint ET_File_List_Get_Length (void)
  */
 gboolean ET_Free_File_List (void)
 {
+    GList *list = NULL;
+
     if (!ETFileList) return FALSE;
     
-    ETFileList = g_list_last(ETFileList);
-    while (ETFileList)
+    list = g_list_last(ETFileList);
+    while (list)
     {
-        ET_Free_File_List_Item((ET_File *)ETFileList->data);
-        if (!ETFileList->prev) break;
-        ETFileList = ETFileList->prev;
+        ET_Free_File_List_Item((ET_File *)list->data);
+        if (!list->prev) break;
+        list = list->prev;
     }
 
-    g_list_free(ETFileList);
+    g_list_free(list);
     ETFileList = (GList *)NULL;
     return TRUE;
 }
@@ -1276,6 +1404,19 @@ gboolean ET_Free_History_File_List (void)
 }
 
 
+gboolean ET_Free_Selection_File_List (void)
+{
+    ETFileSelectionList = g_list_first(ETFileSelectionList);
+    if (ETFileSelectionList)
+    {
+        g_list_free(ETFileSelectionList);
+        ETFileSelectionList = (GList *)NULL;
+    }
+    return TRUE;
+}
+
+
+
 /*********************
  * Copying functions *
  *********************/
@@ -1410,7 +1551,7 @@ gboolean ET_Copy_File_Tag_Item (ET_File *ETFile, File_Tag *FileTag)
  * Set the value of a field of a FileName item (for ex, value of FileName->value)
  * Must be used only for the 'gchar *' components (Only used for the filename)
  */
-gboolean ET_Set_Field_File_Name_Item (gint *FileNameField, gchar *value)
+gboolean ET_Set_Field_File_Name_Item (gchar **FileNameField, gchar *value)
 {
     return ET_Set_Field_File_Tag_Item(FileNameField,value);
 }
@@ -1420,18 +1561,18 @@ gboolean ET_Set_Field_File_Name_Item (gint *FileNameField, gchar *value)
  * Set the value of a field of a FileTag item (for ex, value of FileTag->title)
  * Must be used only for the 'gchar *' components
  */
-gboolean ET_Set_Field_File_Tag_Item (gint *FileTagField, gchar *value)
+gboolean ET_Set_Field_File_Tag_Item (gchar **FileTagField, gchar *value)
 {
     if (!FileTagField) return FALSE;
     
-    if ((gchar *)*FileTagField != NULL)
+    if (*FileTagField != NULL)
     {
-        g_free((gchar *)*FileTagField);
-        (gchar *)*FileTagField = NULL;
+        g_free(*FileTagField);
+        *FileTagField = NULL;
     }
     
     if (value != NULL)
-        (gchar *)*FileTagField = g_strdup(value);
+        *FileTagField = g_strdup(value);
 
     return TRUE;
 }
@@ -1443,6 +1584,7 @@ gboolean ET_Set_Field_File_Tag_Item (gint *FileTagField, gchar *value)
 
 /*
  * Display informations of the file (Position + Header + Tag) to the user interface.
+ * Before doing it, it saves data of the file currently displayed
  */
 void ET_Display_File_Data_To_UI (ET_File *ETFile)
 {
@@ -1457,12 +1599,15 @@ void ET_Display_File_Data_To_UI (ET_File *ETFile)
     new_filename = ((File_Name *)((GList *)ETFile->FileNameNew)->data)->value;
     cur_filename = ((File_Name *)((GList *)ETFile->FileNameCur)->data)->value;
     ETFileDescription = ETFile->ETFileDescription;
+
+    /* Save the current displayed file */
+    ETFileDisplayed = ETFile;
     
-    /* Display position in list + show/hide icon if file writable/read_only */
-    ET_Display_File_And_List_Status_To_UI(cur_filename);
+    /* Display position in list + show/hide icon if file writable/read_only (cur_filename) */
+    ET_Display_File_And_List_Status_To_UI(ETFile);
     
-    /* Display filename (and his path) */
-    ET_Display_Filename_To_UI(new_filename);
+    /* Display filename (and his path) (new_filename) */
+    ET_Display_Filename_To_UI(ETFile);
     
     /* Display tag data */
     switch (ETFileDescription->TagType)
@@ -1473,10 +1618,14 @@ void ET_Display_File_Data_To_UI (ET_File *ETFile)
             break;
 #ifdef ENABLE_OGG
         case OGG_TAG:
-            gtk_frame_set_label(GTK_FRAME(TagFrame),_("OGG Vorbis Tag"));
+            gtk_frame_set_label(GTK_FRAME(TagFrame),_("Ogg Vorbis Tag"));
             ET_Display_File_Tag_To_UI(ETFile);
             break;
 #endif
+        case APE_TAG:
+            gtk_frame_set_label(GTK_FRAME(TagFrame),_("APE Tag"));
+            ET_Display_File_Tag_To_UI(ETFile);
+            break;
         case UNKNOWN_TAG:
         default:
             gtk_frame_set_label(GTK_FRAME(TagFrame),_("Tag"));
@@ -1498,19 +1647,26 @@ void ET_Display_File_Data_To_UI (ET_File *ETFile)
             break;
 #ifdef ENABLE_OGG
         case OGG_FILE:
-            gtk_frame_set_label(GTK_FRAME(FileFrame),_("OGG File"));
+            gtk_frame_set_label(GTK_FRAME(FileFrame),_("Ogg Vorbis File"));
             Ogg_Header_Display_File_Info_To_UI(cur_filename,ETFile->ETFileInfo);
             break;
 #endif
         case FLAC_FILE:
             gtk_frame_set_label(GTK_FRAME(FileFrame),_("FLAC File"));
             Flac_Header_Display_File_Info_To_UI(cur_filename,ETFile->ETFileInfo);
-            // Default displaying
-            //ET_Display_File_Info_To_UI(ETFile->ETFileInfo);
+            break;
+        case MPC_FILE:
+            gtk_frame_set_label(GTK_FRAME(FileFrame),_("MusePack File"));
+            Mpc_Header_Display_File_Info_To_UI(cur_filename,ETFile->ETFileInfo);
+            break;
+        case MAC_FILE:
+            gtk_frame_set_label(GTK_FRAME(FileFrame),_("Monkey's Audio File"));
+            Mac_Header_Display_File_Info_To_UI(cur_filename,ETFile->ETFileInfo);
             break;
         case UNKNOWN_FILE:
         default:
             gtk_frame_set_label(GTK_FRAME(FileFrame),_("File"));
+            // Default displaying
             ET_Display_File_Info_To_UI(ETFile->ETFileInfo);
             g_print("ETFileInfo: Undefined file type %d for file %s.\n",ETFileDescription->FileType,cur_filename);
             break;
@@ -1526,13 +1682,19 @@ void ET_Display_File_Data_To_UI (ET_File *ETFile)
  * Toggle visibility of the small status icon if filename is read-only or not found.
  * Show the position of the current file in the list, by using the index and list length.
  */
-void ET_Display_File_And_List_Status_To_UI (gchar *filename)
+void ET_Display_File_And_List_Status_To_UI (ET_File *ETFile)
 {
     FILE *file;
     gchar *text;
+    gchar *cur_filename;
+
+
+    if (!ETFile) return;
+
+    cur_filename = ((File_Name *)((GList *)ETFile->FileNameCur)->data)->value;
 
     /* Show/hide 'AccessStatusIcon' */
-    if ( (file=fopen(filename,"r+b"))!=NULL )
+    if ( (file=fopen(cur_filename,"r+b"))!=NULL )
     {
         gtk_widget_hide(ReadOnlyStatusIconBox);
         gtk_widget_hide(BrokenStatusIconBox);
@@ -1556,22 +1718,25 @@ void ET_Display_File_And_List_Status_To_UI (gchar *filename)
     }
 
     /* Show position of current file in list */
-    text = g_strdup_printf("%d/%d:",ETFileList_Index,ETFileList_Length);
+    text = g_strdup_printf("%d/%d:",ETFile->IndexKey,ETFileList_Length);
     gtk_label_set_text(GTK_LABEL(FileIndex),text);
     g_free(text);
 }
 
 
-void ET_Display_Filename_To_UI (gchar *filename)
+void ET_Display_Filename_To_UI (ET_File *ETFile)
 {
-    gchar *text, *pos;
+    gchar *text, *text1, *pos, *new_filename;
 
-    if (!filename) return;
+
+    if (!ETFile) return;
+
+    new_filename = ((File_Name *)((GList *)ETFile->FileNameNew)->data)->value;
 
     /*
      * Set filename into FileEntry
      */
-    text = g_strdup(g_basename(filename));
+    text = g_strdup(g_basename(new_filename));
     // Remove the extension
     if ((pos=strrchr(text,'.'))!=NULL) *pos = 0;
     gtk_entry_set_text(GTK_ENTRY(FileEntry),text);
@@ -1582,9 +1747,13 @@ void ET_Display_Filename_To_UI (gchar *filename)
     /*
      * Set the path to the file into BrowserEntry (dirbrowser)
      */
-    text = g_dirname(filename);
+    text = g_dirname(new_filename);
     Browser_Entry_Set_Text(text);
+    // And refresh the number of files in this directory
+    text1 = g_strdup_printf(_("%u file(s)"),ET_Get_Number_Of_Files_In_Directory(text));
+    Browser_Label_Set_Text(text1);
     g_free(text);
+    g_free(text1);
 }
 
 
@@ -1736,16 +1905,17 @@ void ET_Save_File_Data_From_UI (ET_File *ETFile)
     ET_File_Description *ETFileDescription;
     File_Name *FileName;
     File_Tag  *FileTag;
-    gulong     undo_key;
+    guint      undo_key;
     gchar     *cur_filename;
 
 
-    if (!ETFile) return;
+    if (!ETFile || !ETFile->FileNameCur || !ETFile->FileNameCur->data) return;
 
     cur_filename = ((File_Name *)((GList *)ETFile->FileNameCur)->data)->value;
     ETFileDescription = ETFile->ETFileDescription;
     undo_key = ET_Undo_Key_New();
 
+
     /*
      * Save filename and generate undo for filename
      */
@@ -1767,6 +1937,7 @@ void ET_Save_File_Data_From_UI (ET_File *ETFile)
 #ifdef ENABLE_OGG
         case OGG_TAG:
 #endif
+        case APE_TAG:
             ET_Save_File_Tag_From_UI(FileTag);
             ET_Copy_File_Tag_Item_Other_Field(ETFile,FileTag);
             break;
@@ -1957,7 +2128,7 @@ gboolean ET_Save_File_Tag_From_UI (File_Tag *FileTag)
     if ( strlen(buffer) > 0  )
     {
         if (NUMBER_TRACK_FORMATED)
-            FileTag->track = g_strdup_printf("%.2d",atoi(buffer));
+            FileTag->track = g_strdup_printf("%.*d",NUMBER_TRACK_FORMATED_SPIN_BUTTON,atoi(buffer));
         else
             FileTag->track = g_strdup(buffer);
     } else
@@ -1973,7 +2144,7 @@ gboolean ET_Save_File_Tag_From_UI (File_Tag *FileTag)
     if ( strlen(buffer) > 0  )
     {
         if (NUMBER_TRACK_FORMATED)
-            FileTag->track_total = g_strdup_printf("%.2d",atoi(buffer));
+            FileTag->track_total = g_strdup_printf("%.*d",NUMBER_TRACK_FORMATED_SPIN_BUTTON,atoi(buffer));
         else
             FileTag->track_total = g_strdup(buffer);
     } else
@@ -2073,7 +2244,7 @@ gboolean ET_Save_File_Tag_Internal (ET_File *ETFile, File_Tag *FileTag)
         gchar *tmp_str;
 
         if (NUMBER_TRACK_FORMATED)
-            FileTag->track = g_strdup_printf("%.2d",atoi(FileTagCur->track));
+            FileTag->track = g_strdup_printf("%.*d",NUMBER_TRACK_FORMATED_SPIN_BUTTON,atoi(FileTagCur->track));
         else
             FileTag->track = g_strdup(FileTagCur->track);
         // This field must contain only digits
@@ -2091,7 +2262,7 @@ gboolean ET_Save_File_Tag_Internal (ET_File *ETFile, File_Tag *FileTag)
     if ( FileTagCur->track_total && strlen(FileTagCur->track_total)>0 )
     {
         if (NUMBER_TRACK_FORMATED)
-            FileTag->track_total = g_strdup_printf("%.2d",atoi(FileTagCur->track_total));
+            FileTag->track_total = g_strdup_printf("%.*d",NUMBER_TRACK_FORMATED_SPIN_BUTTON,atoi(FileTagCur->track_total));
         else
             FileTag->track_total = g_strdup(FileTagCur->track_total);
     } else
@@ -2160,6 +2331,9 @@ gboolean ET_Save_File_Tag_To_HD (ET_File *ETFile)
             state = Ogg_Tag_Write_File_Tag(ETFile);
             break;
 #endif
+        case APE_TAG:
+            state = Ape_Tag_Write_File_Tag(ETFile);
+            break;
         case UNKNOWN_TAG:
         default:
             g_print("Saving to HD: Undefined function for tag type '%d' (file %s).\n",
@@ -2458,14 +2632,14 @@ gboolean ET_Add_File_To_History_List (ET_File *ETFile)
 
 
 /*
- * Applies one undo to the ETFile data (returns previous data).
+ * Applies one undo to the ETFile data (to reload the previous data).
  * Returns TRUE if an undo had been applied.
  */
 gboolean ET_Undo_File_Data (ET_File *ETFile)
 {
     gboolean has_filename_undo_data = FALSE;
     gboolean has_filetag_undo_data  = FALSE;
-    gulong   filename_key, filetag_key, undo_key;
+    guint    filename_key, filetag_key, undo_key;
     
     if (!ETFile)
         return FALSE;
@@ -2526,7 +2700,7 @@ gboolean ET_Redo_File_Data (ET_File *ETFile)
 {
     gboolean has_filename_redo_data = FALSE;
     gboolean has_filetag_redo_data  = FALSE;
-    gulong   filename_key, filetag_key, undo_key;
+    guint    filename_key, filetag_key, undo_key;
     
     if (!ETFile)
         return FALSE;
@@ -2535,11 +2709,11 @@ gboolean ET_Redo_File_Data (ET_File *ETFile)
     if (ETFile->FileNameNew->next && ETFile->FileNameNew->next->data)
         filename_key = ((File_Name *)ETFile->FileNameNew->next->data)->key;
     else
-        filename_key = (gulong)~0; // To have the max value for gulong
+        filename_key = (guint)~0; // To have the max value for guint
     if (ETFile->FileTag->next && ETFile->FileTag->next->data)
         filetag_key = ((File_Tag *)ETFile->FileTag->next->data)->key;
     else
-        filetag_key = (gulong)~0; // To have the max value for gulong
+        filetag_key = (guint)~0; // To have the max value for guint
     // The key to use
     undo_key = MIN(filename_key,filetag_key);
 
@@ -2581,23 +2755,24 @@ gboolean ET_File_Data_Has_Redo_Data (ET_File *ETFile)
 
 
 /*
- * Execute one 'undo' in the main undo list
+ * Execute one 'undo' in the main undo list (it selects the last ETFile changed,
+ * before to apply an undo action)
  */
-gboolean ET_Undo_History_File_Data (void)
+ET_File *ET_Undo_History_File_Data (void)
 {
     ET_File *ETFile;
     ET_History_File *ETHistoryFile;
     
-    if (!ETHistoryFileList || !ET_History_File_List_Has_Undo_Data()) return FALSE;
+    if (!ETHistoryFileList || !ET_History_File_List_Has_Undo_Data()) return NULL;
 
     ETHistoryFile = (ET_History_File *)ETHistoryFileList->data;
-    ET_File_List_Nth_By_Key(ETHistoryFile->ETFile->ETFileKey);
-    ETFile = (ET_File *)ETFileList->data;
+    ETFile        = (ET_File *)ETHistoryFile->ETFile;
+    ET_File_List_By_Etfile(ETFile);
     ET_Undo_File_Data(ETFile);
     
     if (ETHistoryFileList->prev)
         ETHistoryFileList = ETHistoryFileList->prev;
-    return TRUE;
+    return ETFile;
 }
 
 
@@ -2616,21 +2791,21 @@ gboolean ET_History_File_List_Has_Undo_Data (void)
 /*
  * Execute one 'redo' in the main undo list
  */
-gboolean ET_Redo_History_File_Data (void)
+ET_File *ET_Redo_History_File_Data (void)
 {
     ET_File *ETFile;
     ET_History_File *ETHistoryFile;
 
-    if (!ETHistoryFileList || !ET_History_File_List_Has_Redo_Data()) return FALSE;
+    if (!ETHistoryFileList || !ET_History_File_List_Has_Redo_Data()) return NULL;
 
     ETHistoryFile = (ET_History_File *)ETHistoryFileList->next->data;
-    ET_File_List_Nth_By_Key(ETHistoryFile->ETFile->ETFileKey);
-    ETFile = (ET_File *)ETFileList->data;
+    ETFile        = (ET_File *)ETHistoryFile->ETFile;
+    ET_File_List_By_Etfile(ETFile);
     ET_Redo_File_Data(ETFile);
  
     if (ETHistoryFileList->next)
         ETHistoryFileList = ETHistoryFileList->next;
-    return TRUE;
+    return ETFile;
 }
 
 
@@ -2665,8 +2840,10 @@ gboolean ET_Check_If_File_Is_Saved (ET_File *ETFile)
 
     if (!ETFile) return TRUE;
     
-    if (ETFile->FileTag)     FileTag = ETFile->FileTag->data;
-    if (ETFile->FileNameNew) FileNameNew = ETFile->FileNameNew->data;
+    if (ETFile->FileTag)
+        FileTag     = ETFile->FileTag->data;
+    if (ETFile->FileNameNew)
+        FileNameNew = ETFile->FileNameNew->data;
     
     // Check if the tag has been changed
     if ( FileTag && FileTag->saved != TRUE )
@@ -2728,7 +2905,7 @@ void ET_Mark_File_Tag_As_Saved (ET_File *ETFile)
     GList *FileTagList;
 
     FileTag     = (File_Tag *)ETFile->FileTag->data;    // The current FileTag, to set to TRUE
-    FileTagList = ((ET_File *)ETFileList->data)->FileTagList;
+    FileTagList = ETFile->FileTagList;
     g_list_foreach(FileTagList,(GFunc)Set_Saved_Value_Of_File_Tag,FALSE); // All other FileTag set to FALSE
     FileTag->saved = TRUE; // The current FileTag set to TRUE
 }
@@ -2748,7 +2925,7 @@ void ET_Mark_File_Name_As_Saved (ET_File *ETFile)
     GList *FileNameList;
 
     FileNameNew  = (File_Name *)ETFile->FileNameNew->data;    // The current FileName, to set to TRUE
-    FileNameList = ((ET_File *)ETFileList->data)->FileNameList;
+    FileNameList = ETFile->FileNameList;
     g_list_foreach(FileNameList,(GFunc)Set_Saved_Value_Of_File_Tag,FALSE);
     FileNameNew->saved = TRUE;
 }
@@ -2785,6 +2962,31 @@ gboolean ET_Read_File_Info (gchar *filename, ET_File_Info *ETFileInfo)
 }
 
 
+/*
+ * This function generate a new file name using path of the old file and the new name
+ * - ETFile -> old_file_name : "/path_to_file/old_name.ext"
+ * - new_file_name : "new_name.ext"
+ * Returns "/path_to_file/new_name.ext" into allocated data!
+ */
+gchar *ET_Generate_File_Name (ET_File *ETFile, gchar *new_file_name)
+{
+    gchar *dirname;
+
+    if (ETFile && ETFile->FileNameNew->data && new_file_name
+    && (dirname=g_dirname(((File_Name *)ETFile->FileNameNew->data)->value)) )
+    {
+        if ( strcmp(dirname,"/")==0 ) // Root directory?
+            return g_strconcat(dirname,new_file_name,ETFile->ETFileDescription->Extension,NULL);
+        else
+            return g_strconcat(dirname,"/",new_file_name,ETFile->ETFileDescription->Extension,NULL);
+    }else
+    {
+        return NULL;
+    }
+
+}
+    
+    
 /*
  * Make a copy of the ETFIle item passed in parameter. The copy
  * is generated as if it was saved from the UI.
@@ -2796,38 +2998,6 @@ gboolean ET_Read_File_Info (gchar *filename, ET_File_Info *ETFileInfo)
 }*/
     
     
-/*
- * Delete the corresponding file. Return TRUE if deleted.
- */
-gboolean ET_Delete_File (ET_File *ETFile)
-{
-    gchar *cur_filename;
-    
-    if (!ETFile) return FALSE;
-
-    // Filename of the file to delete
-    cur_filename = ((File_Name *)(ETFile->FileNameCur)->data)->value;
-    if (Delete_File(cur_filename)==TRUE)
-    {
-        // Now, we delete the file in the list
-        GList *list = g_list_first(ETFileList);
-        while(list)
-        {
-            if ( (ET_File *)list->data == ETFile )
-            {
-                ET_File_List_Remove(list);
-                break;
-            }
-            list = list->next;
-        }
-        return TRUE;
-    }else
-    {
-        return FALSE;
-    }
-}
-
-
 /*
  * Used to replace the illegal characters in the filename (it doesn't contain the path).
  */
@@ -2868,18 +3038,58 @@ gboolean ET_File_Name_Convert_Character (gchar *filename)
 }
 
 
+/*
+ * Returns the number of file in the directory of the selected file.
+ */
+guint ET_Get_Number_Of_Files_In_Directory (gchar *path)
+{
+    GList *etfilelist;
+    guint  count = 0;
+    
+    if (!path) return count;
+        
+    etfilelist = g_list_first(ETFileList);
+    while (etfilelist)
+    {
+        ET_File *ETFile = (ET_File *)etfilelist->data;
+        gchar *cur_filename = ((File_Name *)((GList *)ETFile->FileNameCur)->data)->value;
+        gchar *dirname = g_dirname(cur_filename);
+        if (strcmp(dirname,path) == 0)
+            count++;
+        g_free(dirname);
+
+        etfilelist = etfilelist->next;
+    }
+    
+    return count;
+}
+    
+    
 
 /***********************
  * Debugging functions *
  ***********************/
 
+void ET_Debug_Print_File (ET_File *ETFile, gchar *file, gint line, gchar *function)
+{
+    GList *etfilelist = NULL;
+    
+    if (ETFile)
+        etfilelist = g_list_append(etfilelist,ETFile);
+    ET_Debug_Print_File_List(etfilelist,file,line,function);
+    g_list_free(etfilelist);
+}
+
+
 /*
  * Functions for DEBUGGING ONLY
  * ET_Print_File_List => show list of filename
- * Parameters: file = __FILE__
+ * Parameters: ETFileList : the list of files to display
+ *             file = __FILE__
  *             line = __LINE__
+ *             function : a text to display (for example the function that calls it)
  */
-void ET_Debug_Print_File_List (gchar *file, gint line, gchar *function)
+void ET_Debug_Print_File_List (GList *ETFileList, gchar *file, gint line, gchar *function)
 {
     gint efl_item = 1;
     gint fnl_item = 1;
@@ -2903,6 +3113,7 @@ void ET_Debug_Print_File_List (gchar *file, gint line, gchar *function)
         g_print("####################\n");
         g_print("# file_cur : '%s'\n",((File_Name *)((ET_File *)etfilelist->data)->FileNameCur->data)->value);
         g_print("# file_new : '%s'\n",((File_Name *)((ET_File *)etfilelist->data)->FileNameNew->data)->value);
+        g_print("# saved    : '%d'\n",((File_Name *)((ET_File *)etfilelist->data)->FileNameNew->data)->saved);
 
         filenamelist = g_list_first( ((ET_File *)etfilelist->data)->FileNameList );
         filenamelist_length = g_list_length(filenamelist);
@@ -2911,7 +3122,7 @@ void ET_Debug_Print_File_List (gchar *file, gint line, gchar *function)
         {
             g_print("  ### sub item filename : %3d/%3d ### %s\n",fnl_item,filenamelist_length,
                                                       (filenamelist==((ET_File *)etfilelist->data)->FileNameNew)?"!! CURRENT !!":"");
-            g_print("  # key     : '%lu'\n",((File_Name *)filenamelist->data)->key);
+            g_print("  # key     : '%d'\n",((File_Name *)filenamelist->data)->key);
             g_print("  # filename: '%s'\n",((File_Name *)filenamelist->data)->value);
 
             if (!filenamelist->next) break;
@@ -2927,8 +3138,9 @@ void ET_Debug_Print_File_List (gchar *file, gint line, gchar *function)
         while (filetaglist)
         {
             g_print("  ### sub item tag : %3d/%3d ### %s\n",ftl_item,filetaglist_length,
-                                                      (filetaglist==((ET_File *)etfilelist->data)->FileTag)?"!! CURRENT !!":"");
-            g_print("  # key     : '%lu'\n",((File_Tag *)filetaglist->data)->key);
+                (filetaglist==((ET_File *)etfilelist->data)->FileTag)?"!! CURRENT !!":"");
+            g_print("  # key     : '%d'\n",((File_Tag *)filetaglist->data)->key);
+            g_print("  # saved   : '%d'\n",((File_Tag *)filetaglist->data)->saved);
             g_print("  # title   : '%s'\n",((File_Tag *)filetaglist->data)->title);
             g_print("  # artist  : '%s'\n",((File_Tag *)filetaglist->data)->artist);
             g_print("  # album   : '%s'\n",((File_Tag *)filetaglist->data)->album);
@@ -2947,7 +3159,5 @@ void ET_Debug_Print_File_List (gchar *file, gint line, gchar *function)
         etfilelist = g_list_next(etfilelist);
         efl_item++;
     }
-    g_print("#### Undo list from %s:%d - end   ####\n",file,line);
+    g_print("#### File list from %s:%d - end   ####\n",file,line);
 }
-
-
