@@ -1,7 +1,7 @@
 /* id3tag.c - 2001/02/16 */
 /*
- *  EasyTAG - Tag editor for MP3 and OGG files
- *  Copyright (C) 2001-2002  Jerome Couderc <j.couderc@ifrance.com>
+ *  EasyTAG - Tag editor for MP3 and Ogg Vorbis files
+ *  Copyright (C) 2001-2003  Jerome Couderc <j.couderc@ifrance.com>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -46,7 +46,7 @@
 /**************
  * Prototypes *
  **************/
-gchar *Id3tag_Get_Error_Message(ID3_Err error);
+gchar *Id3tag_Get_Error_Message (ID3_Err error);
 gint   Id3tag_Get_Id3v2_Version (gchar *filename);
 ID3_C_EXPORT size_t ID3Tag_Link_1       (ID3Tag *id3tag, const char *filename);
 ID3_C_EXPORT size_t ID3Field_GetASCII_1 (const ID3Field *field, char *buffer, size_t maxChars, size_t itemNum);
@@ -98,6 +98,14 @@ gboolean Id3tag_Read_File_Tag (gchar *filename, File_Tag *FileTag)
                 FileTag->saved = FALSE;
         }
 
+        /* If the specified tags (in preferences window) aren't in the file => we save it
+         * (Note: the file must be linked with the both tags) */ 
+#       if ( (ID3LIB_MAJOR >= 3) && (ID3LIB_MINOR >= 8) && (ID3LIB_PATCH >= 1) )
+            if (ID3Tag_HasTagType(id3_tag,ID3TT_ID3V1) != WRITE_ID3V1_TAG
+            ||  ID3Tag_HasTagType(id3_tag,ID3TT_ID3V2) != WRITE_ID3V2_TAG)
+                FileTag->saved = FALSE;
+#       endif
+
         /* Protection against invalid ID3v2 tag, for example ID3v2.4 in
          * id3lib-3.8.0 : if offset is not nul and the tag contains no
          * frame, may be an invalid id3v2 tag. So we read the id3v1 tag */
@@ -247,10 +255,10 @@ gboolean Id3tag_Read_File_Tag (gchar *filename, File_Tag *FileTag)
                         {
                             if (string2)
                             {
-                                FileTag->track_total = g_strdup_printf("%.2d",atoi(string2+1)); // Just to have numbers like this : '01', '05', '12', ...
+                                FileTag->track_total = g_strdup_printf("%.*d",NUMBER_TRACK_FORMATED_SPIN_BUTTON,atoi(string2+1)); // Just to have numbers like this : '01', '05', '12', ...
                                 *string2 = '\0';
                             }
-                            FileTag->track = g_strdup_printf("%.2d",atoi(string1)); // Just to have numbers like this : '01', '05', '12', ...
+                            FileTag->track = g_strdup_printf("%.*d",NUMBER_TRACK_FORMATED_SPIN_BUTTON,atoi(string1)); // Just to have numbers like this : '01', '05', '12', ...
                         }else
                         {
                             if (string2)
@@ -268,10 +276,10 @@ gboolean Id3tag_Read_File_Tag (gchar *filename, File_Tag *FileTag)
                         {
                             if (string2)
                             {
-                                FileTag->track_total = g_strdup_printf("%.2d",atoi(string2+1)); // Just to have numbers like this : '01', '05', '12', ...
+                                FileTag->track_total = g_strdup_printf("%.*d",NUMBER_TRACK_FORMATED_SPIN_BUTTON,atoi(string2+1)); // Just to have numbers like this : '01', '05', '12', ...
                                 *string2 = '\0';
                             }
-                            FileTag->track = g_strdup_printf("%.2d",atoi(string)); // Just to have numbers like this : '01', '05', '12', ...
+                            FileTag->track = g_strdup_printf("%.*d",NUMBER_TRACK_FORMATED_SPIN_BUTTON,atoi(string)); // Just to have numbers like this : '01', '05', '12', ...
                         }else
                             {
                             if (string2)
@@ -296,6 +304,10 @@ gboolean Id3tag_Read_File_Tag (gchar *filename, File_Tag *FileTag)
             {
                 /*
                  * We manipulate only the name of the genre
+                 * Genre is written like this :
+                 *    - "(<genre_id>)"              -> "(3)"
+                 *    - "<genre_name>"              -> "Dance"
+                 *    - "(<genre_id>)<refinement>"  -> "(3)EuroDance"
                  */
                 if ( (num_chars=ID3Field_GetASCII_1(id3_field,string,ID3V2_MAX_STRING_LEN,field_num)) > 0
                      && string != NULL )
@@ -307,7 +319,7 @@ gboolean Id3tag_Read_File_Tag (gchar *filename, File_Tag *FileTag)
                     if ( (string[0]=='(') && (tmp=strchr(string,')')) && (strlen((tmp+1))>0) )
                     {
     
-                        /* Convert a genre written as '(3)Dance' into 'Dance' */
+                        /* Convert a genre written as '(3)EuroDance' into 'EuroDance' */
                         if (USE_CHARACTER_SET_TRANSLATION)
                         {
                             string1 = convert_from_file_to_user(tmp+1);
@@ -414,13 +426,13 @@ gboolean Id3tag_Write_File_Tag (ET_File *ETFile)
     ID3_Err   error_update_id3v2 = ID3E_NoError;
     gint error = 0;
     gint number_of_frames;
-    gboolean has_title   = 1;
-    gboolean has_artist  = 1;
-    gboolean has_album   = 1;
-    gboolean has_year    = 1;
-    gboolean has_track   = 1;
-    gboolean has_genre   = 1;
-    gboolean has_comment = 1;
+    gboolean has_title    = 1;
+    gboolean has_artist   = 1;
+    gboolean has_album    = 1;
+    gboolean has_year     = 1;
+    gboolean has_track    = 1;
+    gboolean has_genre    = 1;
+    gboolean has_comment  = 1;
     gboolean has_song_len = 1;
 
 
@@ -438,6 +450,7 @@ gboolean Id3tag_Write_File_Tag (ET_File *ETFile)
     }
     fclose(file);
 
+
     /* We get again the tag from the file to keep also unused data (by EasyTAG), then
      * we replace the changed data */
     if ( (id3_tag = ID3Tag_New()) )
@@ -607,7 +620,11 @@ gboolean Id3tag_Write_File_Tag (ET_File *ETFile)
 
         /*********
          * Genre *
-         *********/
+         *********
+         * Genre is written like this :
+         *    - "(<genre_id>)"              -> "(3)"
+         *    - "(<genre_id>)<refinement>"  -> "(3)EuroDance"
+         */
         if (FileTag->genre && strlen(FileTag->genre)>0 )
         {
             if ( (id3_frame = ID3Tag_FindFrameWithID(id3_tag,ID3FID_CONTENTTYPE)) )
@@ -617,20 +634,30 @@ gboolean Id3tag_Write_File_Tag (ET_File *ETFile)
 
             if ((id3_field = ID3Frame_GetField(id3_frame,ID3FN_TEXT)))
             {
-                gchar *tmp_genre;
+                gchar *genre_string_tmp;
+                guchar genre_value;
 
                 if (USE_CHARACTER_SET_TRANSLATION)
                 {
                     string = convert_from_user_to_file(FileTag->genre);
-                    tmp_genre = g_strdup_printf("(%d)%s",Id3tag_String_To_Genre(string),string);
-                    ID3Field_SetASCII(id3_field,tmp_genre);
+                    genre_value = Id3tag_String_To_Genre(string);
+                    /* If genre not defined don't write genre value between brackets! (priority problem noted with some tools) */
+                    if (genre_value == ID3_INVALID_GENRE)
+                        genre_string_tmp = g_strdup_printf("%s",string);
+                    else
+                        genre_string_tmp = g_strdup_printf("(%d)",genre_value);
+                    ID3Field_SetASCII(id3_field,genre_string_tmp);
                     g_free(string);
                 }else
                 {
-                    tmp_genre = g_strdup_printf("(%d)%s",Id3tag_String_To_Genre(FileTag->genre),FileTag->genre);
-                    ID3Field_SetASCII(id3_field,tmp_genre);
+                    genre_value = Id3tag_String_To_Genre(FileTag->genre);
+                    if (genre_value == ID3_INVALID_GENRE)
+                        genre_string_tmp = g_strdup_printf("%s",FileTag->genre);
+                    else
+                        genre_string_tmp = g_strdup_printf("(%d)",genre_value);
+                    ID3Field_SetASCII(id3_field,genre_string_tmp);
                 }
-                g_free(tmp_genre);
+                g_free(genre_string_tmp);
             }
 
         } else
@@ -802,7 +829,7 @@ gboolean Id3tag_Write_File_Tag (ET_File *ETFile)
 gint Id3tag_Get_Id3v2_Version (gchar *filename)
 {
     FILE  *file;
-	guchar tmp[4];
+    guchar tmp[4];
 
     if ( filename!=NULL && (file=fopen(filename,"r"))!=NULL )
     {
@@ -875,7 +902,7 @@ gchar *Id3tag_Get_Error_Message(ID3_Err error)
  */
 guchar Id3tag_String_To_Genre (gchar *genre)
 {
-    gint i;
+    guint i;
 
     for (i=0; i<GENRE_MAX; i++)
         if (strcasecmp(genre,id3_genres[i])==0)
@@ -914,7 +941,14 @@ ID3_C_EXPORT size_t ID3Tag_Link_1 (ID3Tag *id3tag, const char *filename)
 {
     size_t offset;
 
-#   if ( (ID3LIB_MAJOR >= 3) && (ID3LIB_MINOR >= 8) )
+#   if ( (ID3LIB_MAJOR >= 3) && (ID3LIB_MINOR >= 8) && (ID3LIB_PATCH >= 1) ) // Same test used in Id3tag_Read_File_Tag to use ID3Tag_HasTagType
+        /* No problem of priority, so we link the file with the both tags
+         * to manage => ID3Tag_HasTagType works correctly */
+        offset = ID3Tag_LinkWithFlags(id3tag,filename,ID3TT_ID3V1 | ID3TT_ID3V2);
+#   elif ( (ID3LIB_MAJOR >= 3) && (ID3LIB_MINOR >= 8) )
+        /* Version 3.8.0pre2 gives priority to tag id3v1 instead of id3v2, so we
+         * try to fix it by linking the file with the id3v2 tag first. This bug 
+         * was fixed in the final version of 3.8.0 but we can't know it... */
         /* First, try to get the ID3v2 tags */
         offset = ID3Tag_LinkWithFlags(id3tag,filename,ID3TT_ID3V2);
         if (offset == 0)
@@ -966,3 +1000,89 @@ ID3_C_EXPORT size_t ID3Field_GetASCII_1(const ID3Field *field, char *buffer, siz
 #    endif
 }
 
+
+
+/*
+ * This function calculates the CRC-32 value of audio data (It doesn't read
+ * the ID3v2 and ID3v1 tags).
+ */
+/***********
+#include <sys/types.h>
+#include <sys/stat.h>
+#include <fcntl.h>
+#include "crc32.h"
+gulong Id3tag_Get_Crc32_Value (gchar *filename)
+{
+    gint     fd;
+    gint     id3v2size = 0;
+    guchar   tmp_id3v1[128];
+    guchar   tmp_id3v2[4];
+    gboolean has_id3v1 = FALSE;
+    gboolean has_id3v2 = FALSE;
+    gulong   crc32_value;
+
+        
+    // Open the file on read/write mode
+    // Note : the file descriptor is an 'int' instead of 'FILE *' as I doesn't
+    // know the equivalent of ftruncate()
+    if ( (fd=open(filename, O_RDWR)) == -1 )
+        return 0;
+
+    // Go to the beginning for the ID3v2 tag
+    lseek(fd, 0L, SEEK_SET);
+    if (read(fd, &tmp_id3v2, 4) != 4)
+        return 0;
+    // Calculate ID3v2 length
+    if (tmp_id3v2[0] == 'I' && tmp_id3v2[1] == 'D' && tmp_id3v2[2] == '3' && tmp_id3v2[3] < 0xFF)
+    {
+        has_id3v2 = TRUE;
+        // id3v2 tag skipeer $49 44 33 yy yy xx zz zz zz zz [zz size]
+        lseek(fd, 2, SEEK_CUR); // Size is 6-9 position
+        if (read(fd, &tmp_id3v2, 4) != 4)
+            return 0;
+        id3v2size = 10 + ( (long)(tmp_id3v2[3])        | ((long)(tmp_id3v2[2]) << 7)
+                           | ((long)(tmp_id3v2[1]) << 14) | ((long)(tmp_id3v2[0]) << 21) );
+    }
+
+
+    // Now we remove the ID3v1 tag temporarly
+    // Skip data of ID3v1.x tag
+    lseek(fd, -128, SEEK_END);
+    if (read(fd, &tmp_id3v1, 128) != 128)
+        return 0;
+    if (tmp_id3v1[0] == 'T' && tmp_id3v1[1] == 'A' && tmp_id3v1[2] == 'G')
+    {
+        int len;
+            
+           has_id3v1 = TRUE;
+        len = lseek(fd, -128, SEEK_END);
+        // Remove the tag
+        if (ftruncate(fd, len)!=0)
+        {        
+            g_print(_("Warning the CRC32 value may be wrong (Can't remove ID3v1 tag of '%s') (%s).\n"),filename,g_strerror(errno));
+        }
+    }
+        
+    // We return at the beginning of the file, after the id3v2 tag before calculating CRC-32
+    if (has_id3v2 == TRUE)
+        lseek(fd, id3v2size, SEEK_SET);
+    else
+        lseek(fd, 0L, SEEK_SET);
+        
+    // Calculate the CRC-32 value
+    if (crc32_easytag(fd, &crc32_value))
+        g_print(_("Error while calculating CRC value on file: '%s' (%s).\n\a"),filename,g_strerror(errno));
+    //g_print("CRC-32: %.8lx\n",crc32_value);
+
+    // Write again the ID3v1 tag
+    if (has_id3v1 == TRUE)
+    {
+        lseek(fd, 0, SEEK_END);
+        write(fd, &tmp_id3v1, sizeof(tmp_id3v1));
+    }
+
+    close(fd);
+
+    return crc32_value;
+}
+***********/
