@@ -1,7 +1,7 @@
 /* misc.c - 2000/06/28 */
 /*
- *  EasyTAG - Tag editor for MP3 and OGG files
- *  Copyright (C) 2000-2002  Jerome Couderc <j.couderc@ifrance.com>
+ *  EasyTAG - Tag editor for MP3 and Ogg Vorbis files
+ *  Copyright (C) 2000-2003  Jerome Couderc <j.couderc@ifrance.com>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -85,14 +85,15 @@ void     Destroy_Write_Playlist_Window   (void);
 void     Playlist_Write_Button_Pressed   (void);
 gboolean Write_Playlist                  (gchar *play_list_name);
 gboolean Playlist_Check_Content_Mask     (GtkObject *widget_to_show_hide, GtkEntry *widget_source);
+void     Playlist_Convert_Forwardslash_Into_Backslash (gchar *string);
 
-
-void Open_Search_File_Window         (void);
-void Destroy_Search_File_Window      (void);
-void Search_File_Window_Key_Press    (GtkWidget *window, GdkEvent *event);
-void Search_File                     (GtkWidget *search_button);
-void Add_Row_To_Search_Result_List   (ET_File *ETFile,gchar *string_to_search);
-void Search_Result_List_Row_Selected (GtkCList *clist, gint row, gint column, GdkEventButton *event, gpointer data);
+void Open_Search_File_Window           (void);
+void Destroy_Search_File_Window        (void);
+void Search_File_Window_Key_Press      (GtkWidget *window, GdkEvent *event);
+void Search_File                       (GtkWidget *search_button);
+void Add_Row_To_Search_Result_List     (ET_File *ETFile,gchar *string_to_search);
+void Search_Result_List_Row_Selected   (GtkCList *clist, gint row, gint column, GdkEventButton *event, gpointer data);
+void Search_Result_List_Row_Unselected (GtkCList *clist, gint row, gint column, GdkEventButton *event, gpointer data);
 
 
 void Open_Load_Filename_Window      (void);
@@ -125,6 +126,7 @@ void Load_Filename_Edit_Text_Line               (GtkWidget *entry, gint row, gin
  * Buttons creation with pixmap
  */
 #include "../pixmaps/apply.xpm"
+#include "../pixmaps/browse.xpm"
 #include "../pixmaps/cancel.xpm"
 #include "../pixmaps/close.xpm"
 #include "../pixmaps/execute.xpm"
@@ -195,6 +197,11 @@ GtkWidget *Create_Button_With_Pixmap (guint button_type)
             pixmap = gdk_pixmap_create_from_xpm_d(MainWindow->window,&mask,NULL,search_xpm);
             break;
 
+        case BUTTON_BROWSE:
+            Label = gtk_label_new(_(" Browse... "));
+            pixmap = gdk_pixmap_create_from_xpm_d(MainWindow->window,&mask,NULL,browse_xpm);
+            break;
+
         default:
             Button = gtk_button_new_with_label("Unknown button");
             return Button;
@@ -300,7 +307,7 @@ GtkWidget *Create_Button_With_Icon_And_Label (gchar **pixmap_name, gchar *label)
 
     /* Display a warning message if the both parameters are NULL */
     if (pixmap_name==NULL && label==NULL)
-        g_warning("Empty button created 'adr=%x' (no icon and no label)!",(gint)Button);
+        g_warning("Empty button created 'adr=%p' (no icon and no label)!",Button);
 
     return Button;
 }
@@ -361,7 +368,7 @@ GList *Add_String_To_Glist (GList *list, gchar *string)
     list = g_list_prepend(list,string);
     
     /* Cut end of list if length exceeds HISTORY_MAX_LENGTH items */
-    if (g_list_length(list)>HISTORY_MAX_LENGTH)
+    if (g_list_length(list)>(guint)HISTORY_MAX_LENGTH)
     {
         tmp_list = g_list_nth(list,HISTORY_MAX_LENGTH);
         tmp_list->prev->next = NULL;
@@ -383,7 +390,7 @@ void Sort_File_list_By_Ascending_Filename (void)
 {
     if (!ETFileList) return;
     
-    ET_Save_File_Data_From_UI((ET_File *)ETFileList->data);
+    ET_Save_File_Data_From_UI(ETFileDisplayed);
 
     /* Go to the first item of the list */
     ET_File_List_First();
@@ -392,7 +399,7 @@ void Sort_File_list_By_Ascending_Filename (void)
     /* Reload files in browser list */
     Browser_List_Load_Files();
 
-    ET_Display_File_Data_To_UI((ET_File *)ETFileList->data);
+    ET_Display_File_Data_To_UI(ETFileDisplayed);
 
     Update_Command_Buttons_Sensivity();
 }
@@ -404,7 +411,7 @@ void Sort_File_list_By_Descending_Filename (void)
 {
     if (!ETFileList) return;
     
-    ET_Save_File_Data_From_UI((ET_File *)ETFileList->data);
+    ET_Save_File_Data_From_UI(ETFileDisplayed);
 
     /* Go to the first item of the list */
     ET_File_List_First();
@@ -413,7 +420,7 @@ void Sort_File_list_By_Descending_Filename (void)
     /* Reload files in browser list */
     Browser_List_Load_Files();
 
-    ET_Display_File_Data_To_UI((ET_File *)ETFileList->data);
+    ET_Display_File_Data_To_UI(ETFileDisplayed);
 
     Update_Command_Buttons_Sensivity();
 }
@@ -483,19 +490,32 @@ void Set_Tips_Color (GtkTooltips *tips)
  * else only first digits.
  */
 #include <ctype.h>
-void Insert_Only_Digit (GtkEditable *editable, const gchar *text, gint length,
-            gint *position, gpointer data)
+void Insert_Only_Digit (GtkEditable *editable, const gchar *inserted_text, gint length,
+                        gint *position, gpointer data)
 {
-    gint i;
+    gint i, i_start = 0;
+    gchar *wid_text = NULL;
     gchar *result = g_new (gchar, length);
 
-    for (i=0; i<length; i++)
-        result[i] = isdigit(text[i])?text[i]:(gchar)NULL;
+    if (length<=0 || !inserted_text)
+        return;
+
+    // To insert negatives values ('-' only for the first character)
+    wid_text = gtk_editable_get_chars(GTK_EDITABLE(editable),0,-1);
+    if ( inserted_text[0]=='-' && wid_text && strlen(wid_text)<1 )
+    {
+        result[0] = inserted_text[0];
+        i_start = 1;
+    }
+    if (wid_text) g_free(wid_text);
+    
+    for (i=i_start; i<length; i++)
+        result[i] = isdigit(inserted_text[i])?inserted_text[i]:(gchar)NULL;
     gtk_signal_handler_block_by_func(GTK_OBJECT(editable),GTK_SIGNAL_FUNC(Insert_Only_Digit),data);
     gtk_editable_insert_text (editable, result, length, position);
     gtk_signal_handler_unblock_by_func(GTK_OBJECT(editable),GTK_SIGNAL_FUNC(Insert_Only_Digit),data);
     gtk_signal_emit_stop_by_name(GTK_OBJECT(editable),"insert_text");
-    g_free (result);
+    g_free(result);
 }
 
 
@@ -552,7 +572,7 @@ void Parse_Genre (GtkEntry *entry, GdkEvent *event, GtkCombo *combo)
     cursor_position = gtk_editable_get_position(GTK_EDITABLE(entry));
 
     // Don't use autocompletion if cursor is not at the end.
-    if ( (cursor_position < strlen(gtk_entry_get_text(entry))) )
+    if ( (cursor_position < (gint)strlen(gtk_entry_get_text(entry))) )
         return;
 
     typed_string = gtk_editable_get_chars(GTK_EDITABLE(entry),0,cursor_position);
@@ -648,7 +668,7 @@ int Compare_Two_Genres (gchar *genre1,gchar *genre2)
 void Load_Genres_List_To_UI(void)
 {
     GList *genrelist = NULL;
-    gint i;
+    guint i;
 
     if (!GenreEntry) return;
 
@@ -675,8 +695,8 @@ void Load_Genres_List_To_UI(void)
 void Load_Track_List_To_UI (void)
 {
     GList *tracklist = NULL;
-    gint len;
-    gint i;
+    guint len;
+    guint i;
 
     if (!ETFileList || !TrackEntry) return;
 
@@ -688,7 +708,13 @@ void Load_Track_List_To_UI (void)
     {
         tracklist = g_list_append(tracklist," ");
         for (i=1; i<=len; i++)
-            tracklist = g_list_append(tracklist,g_strdup_printf("%.2d",i));
+            if (NUMBER_TRACK_FORMATED)
+            {
+                tracklist = g_list_append(tracklist,g_strdup_printf("%.*d",NUMBER_TRACK_FORMATED_SPIN_BUTTON,i));
+            }else
+            {
+                tracklist = g_list_append(tracklist,g_strdup_printf("%.2d",i));
+            }
     }
 
     // Connect the combo and the list
@@ -710,7 +736,7 @@ void Destroy_Mouse_Cursor (void)
 {
     if (MouseCursor)
     {
-            gdk_cursor_destroy(MouseCursor);
+        gdk_cursor_destroy(MouseCursor);
         MouseCursor = NULL;
     }
 }
@@ -731,11 +757,20 @@ void Set_Unbusy_Cursor (void)
 
 
 
+/*
+ * Callback to the event "focus-in-event" when an entry receive the focus : we select all the text
+ */
+void Focus_Select_Text_In_Editable (GtkWidget *widget)
+{
+    gtk_editable_select_region(GTK_EDITABLE(widget),0,-1);
+}
+
+
 
 /*
  * Run the audio player and load files of the current dir
  */
-void Run_Audio_Player_With_File_List (void)
+void Run_Audio_Player_Using_File_List (AP_Play_Type type)
 {
     pid_t pid;
 
@@ -763,7 +798,13 @@ void Run_Audio_Player_With_File_List (void)
             // Number of arguments into 'argv_user'
             for (argv_user_number=0;argv_user[argv_user_number];argv_user_number++);
 
-            argv = g_new0(gchar *,argv_user_number + ETFileList_Length + 1);
+            // The list of files to play
+            if (type == PLAY_DIRECTORY)
+                etfilelist = g_list_first(ETFileList);
+            else
+                etfilelist = ET_File_List_Get_Selection();
+
+            argv = g_new0(gchar *,argv_user_number + g_list_length(etfilelist) + 1);
 
             // Load 'user' arguments (program name and more...)
             while (argv_user[argv_index])
@@ -772,7 +813,6 @@ void Run_Audio_Player_With_File_List (void)
                 argv_index++;
             }
             // Load files as arguments
-            etfilelist = ET_File_List_First();
             while (etfilelist)
             {
                 etfile   = (ET_File *)etfilelist->data;
@@ -787,68 +827,20 @@ void Run_Audio_Player_With_File_List (void)
             }
             g_free(argv);
             g_strfreev(argv_user);
+            _exit(1);
             break;
         }
         default:
             break;
     }
 }
-/*
- * Run the audio player with the current file
- */
-void Run_Audio_Player_With_One_File (void)
+void Run_Audio_Player_Using_Directory (void)
 {
-    pid_t pid;
-
-    // Exit if no program selected...
-    if (strlen(g_strstrip(AUDIO_FILE_PLAYER))<1)
-        return;
-
-    pid = fork();
-    switch(pid)
-    {
-        case -1:
-            g_warning(_("Can't fork another process!\n"));
-            break;
-        case 0:
-        {
-            gchar **argv;
-            gint    argv_index = 0;
-            gchar **argv_user;
-            gint    argv_user_number;
-
-            argv_user = g_strsplit(AUDIO_FILE_PLAYER," ",0); // the string may contains arguments, space is the delimiter
-            // Number of arguments into 'argv_user'
-            //for (argv_user_number=1;argv_user[argv_user_number-1];argv_user_number++);
-            for (argv_user_number=0;argv_user[argv_user_number];argv_user_number++);
-
-            argv = g_new0(gchar *,argv_user_number + ETFileList_Length + 1);
-
-            // Load 'user' arguments (program name and more...)
-            while (argv_user[argv_index])
-            {
-                argv[argv_index] = argv_user[argv_index];
-                argv_index++;
-            }
-            // Load the current file as arguments
-            if (ETFileList)
-            {
-                ET_File *etfile = (ET_File *)ETFileList->data;
-                gchar *filename = ((File_Name *)etfile->FileNameCur->data)->value;
-                argv[argv_index++] = filename;
-            }
-            argv[argv_index] = NULL; // Ends the list of arguments
-            if (execvp(argv[0],argv) == -1)
-            {
-                g_warning(_("Can't execute %s (%s)!\n"),argv[0],g_strerror(errno));
-            }
-            g_free(argv);
-            g_strfreev(argv_user);
-            break;
-        }
-        default:
-            break;
-    }
+    Run_Audio_Player_Using_File_List(PLAY_DIRECTORY);
+}
+void Run_Audio_Player_Using_Selection (void)
+{
+    Run_Audio_Player_Using_File_List(PLAY_SELECTION);
 }
 
 
@@ -862,7 +854,7 @@ gchar *Convert_Size (gfloat size)
     gchar *Units_Tab[] = { N_("B"), N_("KB"), N_("MB"), N_("GB"), N_("TB")};
     gint i = 0;
 
-    while ( (gint)size/1024 && i<(sizeof(Units_Tab)/sizeof(Units_Tab[0])-1) )
+    while ( (gint)size/1024 && i<(gint)(sizeof(Units_Tab)/sizeof(Units_Tab[0])-1) )
     {
         size = size/1024;
         i++;
@@ -874,7 +866,7 @@ gchar *Convert_Size_1 (gfloat size)
     gchar *data=NULL;
     /* Units Tab of file size (bytes,kilobytes,...) */
     gchar *Units_Tab[] = { N_("B"), N_("KB"), N_("MB"), N_("GB"), N_("TB")};
-    gint i = 0;
+    guint i = 0;
 
     while ( (gint)size/1024 && i<(sizeof(Units_Tab)/sizeof(Units_Tab[0])-1) )
     {
@@ -1089,6 +1081,23 @@ gtk_widget_set_sensitive(GTK_WIDGET(Button),FALSE);
     gtk_box_pack_start(GTK_BOX(vbox),playlist_relative_path,FALSE,FALSE,0);
     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(playlist_full_path),PLAYLIST_FULL_PATH);
     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(playlist_relative_path),PLAYLIST_RELATIVE_PATH);
+    // Separator line
+    Separator = gtk_hseparator_new();
+    gtk_box_pack_start(GTK_BOX(vbox),Separator,FALSE,FALSE,0);
+
+    // Create playlist in parent directory
+    playlist_create_in_parent_dir = gtk_check_button_new_with_label(_("Create playlist in the parent directory"));
+    gtk_box_pack_start(GTK_BOX(vbox),playlist_create_in_parent_dir,FALSE,FALSE,0);
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(playlist_create_in_parent_dir),PLAYLIST_CREATE_IN_PARENT_DIR);
+    gtk_tooltips_set_tip(Tips,playlist_create_in_parent_dir,_("If activated, the playlist will be created "
+        "in the parent directory."),NULL);
+
+    // DOS Separator
+    playlist_use_dos_separator = gtk_check_button_new_with_label(_("Use DOS directory separator"));
+    gtk_box_pack_start(GTK_BOX(vbox),playlist_use_dos_separator,FALSE,FALSE,0);
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(playlist_use_dos_separator),PLAYLIST_USE_DOS_SEPARATOR);
+    gtk_tooltips_set_tip(Tips,playlist_use_dos_separator,_("This option replaces the UNIX directory "
+        "separator '/' into DOS separator '\\'."),NULL);
 
     /* Playlist content */
     Frame = gtk_frame_new(_("Playlist Content"));
@@ -1216,7 +1225,7 @@ void Write_Playlist_Window_Key_Press (GtkWidget *window, GdkEvent *event)
 void Playlist_Write_Button_Pressed (void)
 {
     gchar *playlist_name = NULL;
-    gchar *playlist_path;
+    gchar *playlist_path, *playlist_path_tmp;
     FILE  *file;
     gchar *msg;
     GtkWidget *msgbox;
@@ -1225,38 +1234,66 @@ void Playlist_Write_Button_Pressed (void)
 
     // Check if playlist name was filled
     if ( gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(playlist_use_mask_name)) 
-         && strlen(gtk_entry_get_text_1(PlayListNameEntry))<=0 )
+    &&   strlen(gtk_entry_get_text_1(PlayListNameEntry))<=0 )
         gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(playlist_use_dir_name),TRUE);
 
     /* Also set in the function 'Apply_Changes' */
-    PLAYLIST_NAME               = g_strdup(gtk_entry_get_text_1(PlayListNameEntry));
-    PLAYLIST_USE_MASK_NAME      = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(playlist_use_mask_name));
-    PLAYLIST_USE_DIR_NAME       = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(playlist_use_dir_name));
+    PLAYLIST_NAME                 = g_strdup(gtk_entry_get_text_1(PlayListNameEntry));
+    PLAYLIST_USE_MASK_NAME        = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(playlist_use_mask_name));
+    PLAYLIST_USE_DIR_NAME         = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(playlist_use_dir_name));
 
-    PLAYLIST_FULL_PATH          = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(playlist_full_path));
-    PLAYLIST_RELATIVE_PATH      = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(playlist_relative_path));
+    PLAYLIST_FULL_PATH            = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(playlist_full_path));
+    PLAYLIST_RELATIVE_PATH        = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(playlist_relative_path));
+    PLAYLIST_CREATE_IN_PARENT_DIR = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(playlist_create_in_parent_dir));
+    PLAYLIST_USE_DOS_SEPARATOR    = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(playlist_use_dos_separator));
 
-    PLAYLIST_CONTENT_NONE       = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(playlist_content_none));
-    PLAYLIST_CONTENT_FILENAME   = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(playlist_content_filename));
-    PLAYLIST_CONTENT_MASK       = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(playlist_content_mask));
-    PLAYLIST_CONTENT_MASK_VALUE = g_strdup(gtk_entry_get_text_1(PlayListContentMaskEntry));
+    PLAYLIST_CONTENT_NONE         = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(playlist_content_none));
+    PLAYLIST_CONTENT_FILENAME     = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(playlist_content_filename));
+    PLAYLIST_CONTENT_MASK         = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(playlist_content_mask));
+    PLAYLIST_CONTENT_MASK_VALUE   = g_strdup(gtk_entry_get_text_1(PlayListContentMaskEntry));
 
-    // Note : End of path contains '/'
-    playlist_path = Browser_Get_Current_Path();
+    // Path of the playlist file
+    // Note : End of path must contains '/' and playlist_path musn't be freed
+    playlist_path_tmp = g_strdup(Browser_Get_Current_Path());
+    if (PLAYLIST_CREATE_IN_PARENT_DIR)
+    {
+        if ( (strcmp(playlist_path_tmp,"/") != 0) )
+        {
+            gchar *tmp;
+            if (playlist_path_tmp[strlen(playlist_path_tmp)-1]=='/')
+                playlist_path_tmp[strlen(playlist_path_tmp)-1] = '\0';
+            if ( (tmp=strrchr(playlist_path_tmp,'/')) != NULL )
+                *(tmp + 1) = '\0';
+        }
+    }
+    playlist_path = g_strdup(playlist_path_tmp);
+    if (playlist_path_tmp) g_free(playlist_path_tmp);
     
-    // Build the playlist name
+    // Build the playlist file name
     if (PLAYLIST_USE_MASK_NAME)
     {
         gchar *filename_generated = NULL;
 
         if (!ETFileList) return;
         
-        // Generate filename from tag of the current selected file
-        filename_generated = Scan_Generate_New_Filename_From_Mask((ET_File *)ETFileList->data,PLAYLIST_NAME);
+        // Generate filename from tag of the current selected file (hummm FIX ME)
+        filename_generated = Scan_Generate_New_Filename_From_Mask(ETFileDisplayed,PLAYLIST_NAME);
+
+        // Replace Characters (with scanner)
+        if (CONVERT_UNDERSCORE_AND_P20_INTO_SPACE)
+        {
+            Scan_Convert_Underscore_Into_Space(filename_generated);
+            Scan_Convert_P20_Into_Space(filename_generated);
+        }
+        if (CONVERT_SPACE_INTO_UNDERSCORE)
+        {
+            Scan_Convert_Space_Into_Undescore(filename_generated);
+        }
+
         playlist_name = g_strconcat(playlist_path,filename_generated,".m3u",NULL);
-        
+
         if (filename_generated) g_free(filename_generated);
-    }else
+    }else // PLAYLIST_USE_DIR_NAME
     {
         gchar *extract_dir_name;
         
@@ -1356,6 +1393,17 @@ gboolean Playlist_Check_Content_Mask (GtkObject *widget_to_show_hide, GtkEntry *
         return TRUE;
 }
 
+/*
+ * Function to replace UNIX ForwardSlash with a DOS BackSlash
+ */
+void Playlist_Convert_Forwardslash_Into_Backslash (gchar *string)
+{
+    gchar *tmp;
+
+    while ((tmp=strchr(string,'/'))!=NULL)
+        *tmp = '\\';
+}
+
 
 /*
  * Write a playlist
@@ -1376,8 +1424,12 @@ gboolean Write_Playlist (gchar *play_list_name)
          */
         basedir = g_dirname(play_list_name);
 
-        // First line of the file
-        fprintf(file,"#EXTM3U\n");
+        // 1) First line of the file (if playlist content is not set to "write only list of files")
+        if (!PLAYLIST_CONTENT_NONE)
+        {
+            fprintf(file,"#EXTM3U\n");
+        }
+        
         etfilelist = g_list_first(ETFileList);
         while (etfilelist)
         {
@@ -1389,28 +1441,67 @@ gboolean Write_Playlist (gchar *play_list_name)
                 // Keep only files in this directory and sub-dirs
                 if ( strncmp(filename,basedir,strlen(basedir))==0 )
                 {
-                    if (PLAYLIST_CONTENT_FILENAME)
+                    // 2) Write the header
+                    if (PLAYLIST_CONTENT_NONE)
                     {
+                        // No header written
+                    }else if (PLAYLIST_CONTENT_FILENAME)
+                    {
+                        // Header uses only filename
                         fprintf(file,"#EXTINF:%d,%s\n",duration,g_basename(filename));
                     }else if (PLAYLIST_CONTENT_MASK)
                     {
+                        // Header uses generated filename from a mask
                         gchar *mask = g_strdup(gtk_entry_get_text_1(PlayListContentMaskEntry));
                         gchar *filename_generated = Scan_Generate_New_Filename_From_Mask(etfile,mask);
                         fprintf(file,"#EXTINF:%d,%s\n",duration,filename_generated);
                         if (mask)               g_free(mask);
                         if (filename_generated) g_free(filename_generated);
                     }
-                    fprintf(file,"%s\n",filename+strlen(basedir)+1);
+                    
+                    // 3) Write the file path
+                    if (PLAYLIST_USE_DOS_SEPARATOR)
+                    {
+                        gchar *filename_conv = g_strdup(filename+strlen(basedir)+1);
+                        Playlist_Convert_Forwardslash_Into_Backslash(filename_conv);
+                        fprintf(file,"%s\n",filename_conv);
+                        if (filename_conv) g_free(filename_conv);
+                    }else
+                    {
+                        fprintf(file,"%s\n",filename+strlen(basedir)+1);
+                    }
                 }
-            }else
+            }else // PLAYLIST_FULL_PATH
             {
-                if (PLAYLIST_CONTENT_FILENAME)
+                // 2) Write the header
+                if (PLAYLIST_CONTENT_NONE)
                 {
+                    // No header written
+                }else if (PLAYLIST_CONTENT_FILENAME)
+                {
+                    // Header uses only filename
                     fprintf(file,"#EXTINF:%d,%s\n",duration,g_basename(filename));
                 }else if (PLAYLIST_CONTENT_MASK)
                 {
+                    // Header uses generated filename from a mask
+                    gchar *mask = g_strdup(gtk_entry_get_text_1(PlayListContentMaskEntry));
+                    gchar *filename_generated = Scan_Generate_New_Filename_From_Mask(etfile,mask);
+                    fprintf(file,"#EXTINF:%d,%s\n",duration,filename_generated);
+                    if (mask)               g_free(mask);
+                    if (filename_generated) g_free(filename_generated);
+                }
+                
+                // 3) Write the file path
+                if (PLAYLIST_USE_DOS_SEPARATOR)
+                {
+                    gchar *filename_conv = g_strdup(filename);
+                    Playlist_Convert_Forwardslash_Into_Backslash(filename_conv);
+                    fprintf(file,"%s\n",filename_conv);
+                    if (filename_conv) g_free(filename_conv);
+                }else
+                {
+                    fprintf(file,"%s\n",filename);
                 }
-                fprintf(file,"%s\n",filename);
             }
             etfilelist = etfilelist->next;
         }
@@ -1498,6 +1589,10 @@ void Open_Search_File_Window (void)
         Add_To_Combo_Box_History,GTK_OBJECT(SearchStringEntry));
     gtk_entry_set_text_1(SearchStringEntry,"");
 
+    // Set content of the clipboard if available
+    gtk_editable_paste_clipboard(GTK_EDITABLE(GTK_COMBO(SearchStringEntry)->entry));
+    gtk_signal_connect(GTK_OBJECT(GTK_COMBO(SearchStringEntry)->entry),"grab-focus",Focus_Select_Text_In_Editable,NULL);
+
     // Where...
     Label = gtk_label_new(_("In :"));
     gtk_misc_set_alignment(GTK_MISC(Label),1.0,0.5);
@@ -1526,10 +1621,12 @@ void Open_Search_File_Window (void)
     gtk_container_add(GTK_CONTAINER(ScrollWindow),SearchResultList);
     //gtk_clist_set_column_auto_resize(GTK_CLIST(BrowserList),0,TRUE);
     gtk_clist_set_reorderable(GTK_CLIST(SearchResultList),FALSE);
-    gtk_clist_set_selection_mode(GTK_CLIST(SearchResultList),GTK_SELECTION_SINGLE);
+    //gtk_clist_set_selection_mode(GTK_CLIST(SearchResultList),GTK_SELECTION_SINGLE);
+    gtk_clist_set_selection_mode(GTK_CLIST(SearchResultList),GTK_SELECTION_EXTENDED);
     gtk_clist_column_titles_passive(GTK_CLIST(SearchResultList));
     gtk_widget_set_sensitive(GTK_WIDGET(SearchResultList),FALSE);
     gtk_signal_connect(GTK_OBJECT(SearchResultList),"select_row",(GtkSignalFunc)Search_Result_List_Row_Selected,NULL);
+    gtk_signal_connect(GTK_OBJECT(SearchResultList),"unselect_row",(GtkSignalFunc)Search_Result_List_Row_Unselected,NULL);
 
     for (i=0;i<8;i++)
     {
@@ -1773,6 +1870,7 @@ void Add_Row_To_Search_Result_List(ET_File *ETFile,gchar *string_to_search)
                 GdkColor RED = {0, 0xffff, 0x0000, 0x0000};
                 GtkStyle *style = gtk_style_copy(gtk_widget_get_style(SearchResultList));
                 style->fg[GTK_STATE_NORMAL] = RED;
+                style->bg[GTK_STATE_SELECTED] = RED;
                 gtk_clist_set_cell_style(GTK_CLIST(SearchResultList),new_row,column,style);
             }
         }else
@@ -1792,6 +1890,7 @@ void Add_Row_To_Search_Result_List(ET_File *ETFile,gchar *string_to_search)
                 GdkColor RED = {0, 0xffff, 0x0000, 0x0000};
                 GtkStyle *style = gtk_style_copy(gtk_widget_get_style(SearchResultList));
                 style->fg[GTK_STATE_NORMAL] = RED;
+                style->bg[GTK_STATE_SELECTED] = RED;
                 gtk_clist_set_cell_style(GTK_CLIST(SearchResultList),new_row,column,style);
             }
             if (list_text) g_free(list_text);
@@ -1807,11 +1906,30 @@ void Add_Row_To_Search_Result_List(ET_File *ETFile,gchar *string_to_search)
  * Callback to select-row event
  */
 void Search_Result_List_Row_Selected (GtkCList *clist, gint row, gint column, GdkEventButton *event, gpointer data)
+{
+    GList   *SelectedRow = NULL;
+    ET_File *ETFile;
+    
+    Browser_List_Unselect_All_Files();
+
+    SelectedRow = ((GtkCList*)clist)->selection;
+    while (SelectedRow)
+    {
+        ETFile = gtk_clist_get_row_data(GTK_CLIST(clist),GPOINTER_TO_INT(SelectedRow->data));
+        Browser_List_Select_File(ETFile,TRUE);
+        Action_Select_Nth_File_By_Etfile(ETFile);
+        SelectedRow = SelectedRow->next;
+    }
+}
+/*
+ * Callback to unselect-row event
+ */
+void Search_Result_List_Row_Unselected (GtkCList *clist, gint row, gint column, GdkEventButton *event, gpointer data)
 {
     ET_File *ETFile;
     
     ETFile = gtk_clist_get_row_data(GTK_CLIST(clist),row);
-    Action_Select_Nth_File_By_Key(ETFile->ETFileKey);
+    Browser_List_Unselect_File(ETFile);
 }
 
 
@@ -1823,6 +1941,7 @@ void Search_Result_List_Row_Selected (GtkCList *clist, gint row, gint column, Gd
 /*
  * The window to load the filenames from a txt.
  */
+#include "../pixmaps/add.xpm"
 void Open_Load_Filename_Window (void)
 {
     GtkWidget *VBox, *hbox;
@@ -1890,13 +2009,14 @@ void Open_Load_Filename_Window (void)
         gtk_entry_set_text_1(FileToLoad,path);
     // the 'changed' signal is attached below to enable/disable the button to load
     // Button 'browse'
-    Button = gtk_button_new_with_label(_(" Browse... "));
+    Button = Create_Button_With_Pixmap(BUTTON_BROWSE);
     gtk_box_pack_start(GTK_BOX(hbox),Button,FALSE,FALSE,0);
     gtk_object_set_data(GTK_OBJECT(FileToLoad),"Parent_Window",LoadFilenameWindow);
     gtk_signal_connect_object(GTK_OBJECT(Button),"clicked",(GtkSignalFunc)File_Selection_Window_For_File,GTK_OBJECT(FileToLoad));
     // Button 'load'
     // the signal attached to this button, to load the file, is placed after the LoadFileContentList definition
-    ButtonLoad = gtk_button_new_with_label(_(" Load "));
+    ButtonLoad = Create_Button_With_Icon_And_Label(add_xpm,_(" Load "));
+    //ButtonLoad = gtk_button_new_with_label(_(" Load "));
     gtk_box_pack_start(GTK_BOX(hbox),ButtonLoad,FALSE,FALSE,0);
     gtk_signal_connect_object(GTK_OBJECT(GTK_COMBO(FileToLoad)->entry),"changed",(GtkSignalFunc)Button_Load_Set_Sensivity,GTK_OBJECT(ButtonLoad));
 
@@ -1910,7 +2030,7 @@ void Open_Load_Filename_Window (void)
 
     // Label of file content
     Label = gtk_label_new(_("Loaded File Content :"));
-    gtk_table_attach_defaults(GTK_TABLE(Table),Label,0,1,0,1);
+    gtk_table_attach(GTK_TABLE(Table),Label,0,1,0,1,GTK_FILL,GTK_FILL,0,2);
 
     // Content of the loaded file
     ScrollWindow = gtk_scrolled_window_new(NULL,NULL);
@@ -1930,7 +2050,7 @@ void Open_Load_Filename_Window (void)
 
     // Label of current list
     Label = gtk_label_new(_("Files Name List :"));
-    gtk_table_attach_defaults(GTK_TABLE(Table),Label,1,2,0,1);
+    gtk_table_attach(GTK_TABLE(Table),Label,1,2,0,1,GTK_FILL,GTK_FILL,0,2);
 
     // List of current filenames
     ScrollWindow = gtk_scrolled_window_new(NULL,NULL);
@@ -2144,18 +2264,41 @@ void Load_File_List (void)
 void Load_Filename_Select_Row_In_Other_Clist (GtkWidget *clist_target, gint row, gint column, GdkEventButton *event, gpointer clist_emit)
 {
     gchar *text;
+    GtkAdjustment *ct_adj, *ce_adj;
 
     if (!clist_emit || !clist_target)
         return;
 
-    gtk_clist_unselect_all(GTK_OBJECT(clist_target));
+    gtk_clist_unselect_all(GTK_CLIST(clist_target));
+
+    // Synchronize the two clist
+    ce_adj = gtk_clist_get_vadjustment(GTK_CLIST(clist_emit));
+    ct_adj = gtk_clist_get_vadjustment(GTK_CLIST(clist_target));
+
+    if (GTK_ADJUSTMENT(ct_adj)->upper >= GTK_ADJUSTMENT(ct_adj)->page_size
+    &&  GTK_ADJUSTMENT(ce_adj)->upper >= GTK_ADJUSTMENT(ce_adj)->page_size)
+    {
+        // Rules are displayed in the both clist
+        if (GTK_ADJUSTMENT(ce_adj)->value <= GTK_ADJUSTMENT(ct_adj)->upper - GTK_ADJUSTMENT(ct_adj)->page_size)
+        {
+            gtk_adjustment_set_value(GTK_ADJUSTMENT(ct_adj),GTK_ADJUSTMENT(ce_adj)->value);
+        }else
+        {
+            gtk_adjustment_set_value(GTK_ADJUSTMENT(ct_adj),GTK_ADJUSTMENT(ct_adj)->upper - GTK_ADJUSTMENT(ct_adj)->page_size);
+            if (row <= GTK_CLIST(clist_target)->rows-1)
+                gtk_adjustment_set_value(GTK_ADJUSTMENT(ce_adj),GTK_ADJUSTMENT(ct_adj)->value);
+        }
+    }else if (GTK_ADJUSTMENT(ct_adj)->upper < GTK_ADJUSTMENT(ct_adj)->page_size) // Target Clist rule not visible
+    {
+        if (row <= GTK_CLIST(clist_target)->rows-1)
+            gtk_adjustment_set_value(GTK_ADJUSTMENT(ce_adj),GTK_ADJUSTMENT(ct_adj)->value);
+    }
 
     // Must block the select signal of the target to avoid looping
     gtk_signal_handler_block_by_func(GTK_OBJECT(clist_target),GTK_SIGNAL_FUNC(Load_Filename_Select_Row_In_Other_Clist),GTK_OBJECT(clist_emit));
     gtk_clist_get_text(GTK_CLIST(clist_emit),row,column,&text);
     gtk_clist_select_row(GTK_CLIST(clist_target),row,column);
     gtk_signal_handler_unblock_by_func(GTK_OBJECT(clist_target),GTK_SIGNAL_FUNC(Load_Filename_Select_Row_In_Other_Clist),GTK_OBJECT(clist_emit));
-
 }
 /*
  * Set the new file name of each file
@@ -2167,14 +2310,13 @@ void Load_Filename_Set_Filenames (void)
     File_Name *FileName;
     gchar *clist_text;
     gchar *clist_text_tmp = NULL;
-    gchar *dirname;
     gchar *filename_new;
     
     if ( !ETFileList || !LoadFileContentList || !LoadFileNameList)
         return;
 
     // Save current file
-    ET_Save_File_Data_From_UI((ET_File *)ETFileList->data);
+    ET_Save_File_Data_From_UI(ETFileDisplayed);
 
     for (row=0;(row<GTK_CLIST(LoadFileContentList)->rows)&&(row<GTK_CLIST(LoadFileNameList)->rows);row++)
     {
@@ -2186,34 +2328,25 @@ void Load_Filename_Set_Filenames (void)
             ET_File_Name_Convert_Character(clist_text_tmp); // Replace invalid characters
 
             /* Build the filename with the path */
-            dirname = g_dirname(((File_Name *)ETFile->FileNameNew->data)->value);
-            if (dirname)
-            {
-                if ( strcmp(dirname,"/")==0 ) // Root directory?
-                    filename_new = g_strconcat("/",clist_text_tmp,ETFile->ETFileDescription->Extension,NULL);
-                else
-                    filename_new = g_strconcat(dirname,"/",clist_text_tmp,ETFile->ETFileDescription->Extension,NULL);
-            }else
-            {
-                break;
-            }
+            filename_new = ET_Generate_File_Name(ETFile,clist_text_tmp);
             g_free(clist_text_tmp);
 
-
             /* Set the new filename */
             // Create a new 'File_Name' item
             FileName = ET_File_Name_Item_New();
             // Save changes of the 'File_Name' item
-            ET_Set_Field_File_Name_Item((gint *)&FileName->value,filename_new);
+            ET_Set_Field_File_Name_Item(&FileName->value,filename_new);
             ET_Manage_Changes_Of_File_Data(ETFile,FileName,NULL);
             
+            if (filename_new) g_free(filename_new);
+
             // Then run current scanner if asked...
             if (ScannerWindow && gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(LoadFileRunScanner)) )
                 Scan_Select_Mode_And_Run_Scanner(ETFile);
         }
     }
     Browser_List_Refresh_Whole_List();
-    ET_Display_File_Data_To_UI((ET_File *)ETFileList->data);
+    ET_Display_File_Data_To_UI(ETFileDisplayed);
 }
 /*
  * Create and attach a popup menu on the two clist of the LoadFileWindow
@@ -2271,7 +2404,7 @@ void Load_Filename_Clist_Insert_Blank_Line (GtkCList *clist)
     if (!clist) return;
 
     if ( (SelectedRow=((GtkCList*)clist)->selection) )
-        row = (gint)SelectedRow->data;
+        row = GPOINTER_TO_INT(SelectedRow->data);
     gtk_clist_insert(GTK_CLIST(clist),row,&text);
     gtk_clist_set_background(GTK_CLIST(clist),row,&LIGHT_GREY);
 }
@@ -2307,7 +2440,7 @@ void Load_Filename_Clist_Delete_Line (GtkCList *clist)
     if (!clist) return;
 
     if ( (SelectedRow=((GtkCList*)clist)->selection) )
-        gtk_clist_remove(GTK_CLIST(clist),(gint)SelectedRow->data);
+        gtk_clist_remove(GTK_CLIST(clist),GPOINTER_TO_INT(SelectedRow->data));
 }
 
 void Load_Filename_Clist_Reload (GtkCList *clist)
@@ -2335,7 +2468,7 @@ void Load_Filename_Update_Text_Line (GtkWidget *clist, GtkWidget *entry)
     SelectedRow = ((GtkCList*)clist)->selection;
     if (SelectedRow && SelectedRow->data!=NULL)
     {
-        gint row = (gint)SelectedRow->data;
+        gint row = GPOINTER_TO_INT(SelectedRow->data);
         gchar *text = gtk_entry_get_text_1(entry);
 
         gtk_clist_set_text(GTK_CLIST(clist),row,0,text);
@@ -2361,6 +2494,3 @@ void Load_Filename_Edit_Text_Line (GtkWidget *entry, gint row, gint column, GdkE
     gtk_entry_set_text_1(entry,text);
 //    gtk_signal_handler_unblock_by_func(GTK_OBJECT(entry),GTK_SIGNAL_FUNC(Update_Text_Line),NULL);
 }
-
-
-
