@@ -1,7 +1,7 @@
 /* ogg_tag.c - 2001/11/08 */
 /*
- *  EasyTAG - Tag editor for MP3 and OGG files
- *  Copyright (C) 2001-2002  Jerome Couderc <j.couderc@ifrance.com>
+ *  EasyTAG - Tag editor for MP3 and Ogg Vorbis files
+ *  Copyright (C) 2001-2003  Jerome Couderc <j.couderc@ifrance.com>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -47,7 +47,7 @@
 
 #define MULTIFIELD_SEPARATOR " - "
 
-/* OGG fields names :
+/* Ogg Vorbis fields names :
  *  - TITLE        : Track name
  *  - VERSION      : The version field may be used to differentiate multiple version of the same track title in a single collection. (e.g. remix info)
  *  - ALBUM        : The collection name to which this track belongs
@@ -80,7 +80,7 @@ gboolean Ogg_Tag_Write_File (FILE *file_in, gchar *filename_in, vcedit_state *st
  *************/
 
 /*
- * Read tag data into an ogg file.
+ * Read tag data into an Ogg Vorbis file.
  * Note:
  *  - if field is found but contains no info (strlen(str)==0), we don't read it
  */
@@ -105,6 +105,40 @@ gboolean Ogg_Tag_Read_File_Tag (gchar *filename, File_Tag *FileTag)
         return FALSE;
     }
 
+    {
+    // Skip the id3v2 tag
+    guchar tmp_id3[4];
+    gulong id3v2size;
+    
+    // Check if there is an ID3v2 tag...
+    fseek(file, 0L, SEEK_SET);
+    if (fread(tmp_id3, 1, 4, file) == 4)
+    {
+        // Calculate ID3v2 length
+        if (tmp_id3[0] == 'I' && tmp_id3[1] == 'D' && tmp_id3[2] == '3' && tmp_id3[3] < 0xFF)
+        {
+            // id3v2 tag skipeer $49 44 33 yy yy xx zz zz zz zz [zz size]
+            fseek(file, 2, SEEK_CUR); // Size is 6-9 position
+            if (fread(tmp_id3, 1, 4, file) == 4)
+            {
+                id3v2size = 10 + ( (long)(tmp_id3[3])        | ((long)(tmp_id3[2]) << 7)
+                                   | ((long)(tmp_id3[1]) << 14) | ((long)(tmp_id3[0]) << 21) );
+                fseek(file, id3v2size, SEEK_SET);
+                g_print(_("Warning : The Ogg Vorbis file '%s' contains an ID3v2 tag.\n"),filename);
+            }else
+            {
+                fseek(file, 0L, SEEK_SET);
+            }
+        }else
+        {
+            fseek(file, 0L, SEEK_SET);
+        }
+    }else
+    {
+        fseek(file, 0L, SEEK_SET);
+    }
+    }
+
     state = vcedit_new_state();    // Allocate memory for 'state'
     if ( vcedit_open(state,file) < 0 )
     {
@@ -116,7 +150,7 @@ gboolean Ogg_Tag_Read_File_Tag (gchar *filename, File_Tag *FileTag)
 
     /* Get data from tag */
     vc = vcedit_comments(state);
-    //{gint i; for (i=0;i<vc->comments;i++) g_print("%s -> ogg vc:'%s'\n",g_basename(filename),vc->user_comments[i]);}
+    //{gint i; for (i=0;i<vc->comments;i++) g_print("%s -> Ogg vc:'%s'\n",g_basename(filename),vc->user_comments[i]);}
 
     /*********
      * Title *
@@ -196,10 +230,10 @@ gboolean Ogg_Tag_Read_File_Tag (gchar *filename, File_Tag *FileTag)
         {
             if (string1)
             {
-                FileTag->track_total = g_strdup_printf("%.2d",atoi(string1+1));
+                FileTag->track_total = g_strdup_printf("%.*d",NUMBER_TRACK_FORMATED_SPIN_BUTTON,atoi(string1+1));
                 *string1 = '\0';
             }
-            FileTag->track = g_strdup_printf("%.2d",atoi(string));
+            FileTag->track = g_strdup_printf("%.*d",NUMBER_TRACK_FORMATED_SPIN_BUTTON,atoi(string));
         }else
         {
             if (string1)
@@ -265,7 +299,7 @@ gboolean Ogg_Tag_Read_File_Tag (gchar *filename, File_Tag *FileTag)
     /***************************
      * Save unsupported fields *
      ***************************/
-    for (i=0;i<vc->comments;i++)
+    for (i=0;i<(guint)vc->comments;i++)
     {
         if ( strncasecmp(vc->user_comments[i],"title=",       6) != 0
           && strncasecmp(vc->user_comments[i],"artist=",      7) != 0
@@ -314,6 +348,39 @@ gboolean Ogg_Tag_Write_File_Tag (ET_File *ETFile)
         return FALSE;
     }
 
+    {
+    // Skip the id3v2 tag
+    guchar tmp_id3[4];
+    gulong id3v2size;
+    
+    // Check if there is an ID3v2 tag...
+    fseek(file_in, 0L, SEEK_SET);
+    if (fread(tmp_id3, 1, 4, file_in) == 4)
+    {
+        // Calculate ID3v2 length
+        if (tmp_id3[0] == 'I' && tmp_id3[1] == 'D' && tmp_id3[2] == '3' && tmp_id3[3] < 0xFF)
+        {
+            // id3v2 tag skipeer $49 44 33 yy yy xx zz zz zz zz [zz size]
+            fseek(file_in, 2, SEEK_CUR); // Size is 6-9 position
+            if (fread(tmp_id3, 1, 4, file_in) == 4)
+            {
+                id3v2size = 10 + ( (long)(tmp_id3[3])        | ((long)(tmp_id3[2]) << 7)
+                                   | ((long)(tmp_id3[1]) << 14) | ((long)(tmp_id3[0]) << 21) );
+                fseek(file_in, id3v2size, SEEK_SET);
+            }else
+            {
+                fseek(file_in, 0L, SEEK_SET);
+            }
+        }else
+        {
+            fseek(file_in, 0L, SEEK_SET);
+        }
+    }else
+    {
+        fseek(file_in, 0L, SEEK_SET);
+    }
+    }
+
     state = vcedit_new_state();    // Allocate memory for 'state'
     if ( vcedit_open(state,file_in) < 0 )
     {
@@ -628,4 +695,3 @@ gboolean Ogg_Header_Display_File_Info_To_UI (gchar *filename, ET_File_Info *ETFi
 }
 
 #endif /* ENABLE_OGG */
-
