@@ -1,7 +1,7 @@
 /* prefs.c - 2000/05/06 */
 /*
- *  EasyTAG - Tag editor for MP3 and OGG files
- *  Copyright (C) 2000-2002  Jerome Couderc <j.couderc@ifrance.com>
+ *  EasyTAG - Tag editor for MP3 and Ogg Vorbis files
+ *  Copyright (C) 2000-2003  Jerome Couderc <j.couderc@ifrance.com>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -75,6 +75,7 @@ void Set_Cddb_Window_Size_Toggled          (void);
 void Set_Cddb_Pane_Handle_Position_Toggled (void);
 
 void Set_Default_Comment_Check_Button_Toggled (void);
+void Number_Track_Formated_Toggled            (void);
 void Use_Character_Set_Translation_Toggled    (void);
 void Scanner_Convert_Check_Button_Toggled_1   (GtkObject *object_rec, GtkObject *object_emi);
 void Cddb_Use_Proxy_Toggled                   (void);
@@ -216,7 +217,7 @@ void Open_OptionsWindow (void)
         Add_To_Combo_Box_History,GTK_OBJECT(DefaultPathToMp3));
 
     // Button browse
-    Button = gtk_button_new_with_label(_(" Browse... "));
+    Button = Create_Button_With_Pixmap(BUTTON_BROWSE);
     gtk_box_pack_start(GTK_BOX(HBox),Button,FALSE,FALSE,0);
     gtk_object_set_data(GTK_OBJECT(DefaultPathToMp3),"Parent_Window",OptionsWindow);
     gtk_signal_connect_object(GTK_OBJECT(Button),"clicked",
@@ -514,17 +515,28 @@ void Open_OptionsWindow (void)
         "only the last numerals of the date (for instance, if the current year is 2005: "
         "5 => 2005, 4 => 2004, 6 => 1996, 95 => 1995, ...)."),NULL);
 
-    NumberTrackFormated = gtk_check_button_new_with_label(_("Write the track field on two characters ('05', '09', '10', ...)"));
-    gtk_box_pack_start(GTK_BOX(vbox),NumberTrackFormated,FALSE,FALSE,0);
+    hbox = gtk_hbox_new(FALSE,0);
+    gtk_box_pack_start(GTK_BOX(vbox),hbox,FALSE,FALSE,0);
+
+    NumberTrackFormated = gtk_check_button_new_with_label(_("Write the track field with severals characters :"));
+    //gtk_box_pack_start(GTK_BOX(vbox),NumberTrackFormated,FALSE,FALSE,0);
+    gtk_box_pack_start(GTK_BOX(hbox),NumberTrackFormated,FALSE,FALSE,0);
     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(NumberTrackFormated),NUMBER_TRACK_FORMATED);
     gtk_tooltips_set_tip(Tips,NumberTrackFormated,_("If activated, the track field is written with "
-        "two characters. Else it keeps the 'raw' track value."),NULL);
+        "'n' characters (Ex. with two characters : '05', '09', '10',...). Else it keeps the 'raw' "
+        "track value."),NULL);
+    
+    NumberTrackFormatedSpinButton = gtk_spin_button_new((GtkAdjustment *)gtk_adjustment_new(2.0,2.0,6.0,1.0,1.0,1.0),1.0,0);
+    gtk_box_pack_start(GTK_BOX(hbox),NumberTrackFormatedSpinButton,FALSE,FALSE,0);
+    gtk_spin_button_set_value(GTK_SPIN_BUTTON(NumberTrackFormatedSpinButton),(gfloat)NUMBER_TRACK_FORMATED_SPIN_BUTTON);
+    gtk_signal_connect(GTK_OBJECT(NumberTrackFormated),"toggled",(GtkSignalFunc)Number_Track_Formated_Toggled,NULL);
+    gtk_signal_emit_by_name(GTK_OBJECT(NumberTrackFormated),"toggled");
 
-    OggTagWriteXmmsComment = gtk_check_button_new_with_label(_("OGG Files : Write also the comment to the XMMS format"));
+    OggTagWriteXmmsComment = gtk_check_button_new_with_label(_("Ogg Vorbis Files : Write also the comment to the XMMS format"));
     gtk_box_pack_start(GTK_BOX(vbox),OggTagWriteXmmsComment,FALSE,FALSE,0);
     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(OggTagWriteXmmsComment),OGG_TAG_WRITE_XMMS_COMMENT);
     gtk_tooltips_set_tip(Tips,OggTagWriteXmmsComment,_("XMMS doesn't make use of the right way to "
-        "identify a comment in OGG files as other apps do. In fact, this field is usually labeled "
+        "identify a comment in Ogg Vorbis files as other apps do. In fact, this field is usually labeled "
         "with 'comment=', whereas XMMS uses only `='. Please, uncheck this option if you don't want "
         "other apps to complain about an unknown field. Comments won't be shown in XMMS, though."),NULL);
 
@@ -766,7 +778,7 @@ void Open_OptionsWindow (void)
         Add_To_Combo_Box_History,GTK_OBJECT(FilePlayerEntry));
 
     // Button browse
-    Button = gtk_button_new_with_label(_(" Browse... "));
+    Button = Create_Button_With_Pixmap(BUTTON_BROWSE);
     gtk_box_pack_start(GTK_BOX(hbox),Button,FALSE,FALSE,0);
     gtk_object_set_data(GTK_OBJECT(FilePlayerEntry),"Parent_Window",OptionsWindow);
     gtk_signal_connect_object(GTK_OBJECT(Button),"clicked",
@@ -810,10 +822,10 @@ void Open_OptionsWindow (void)
         CONVERT_UNDERSCORE_AND_P20_INTO_SPACE);
     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(ConvertSpaceIntoUnderscore),
         CONVERT_SPACE_INTO_UNDERSCORE);
-    gtk_tooltips_set_tip(Tips,ConvertUnderscoreAndP20IntoSpace,_("If activated, the conversion "
-        "will take place when applying a mask."),NULL);
-    gtk_tooltips_set_tip(Tips,ConvertSpaceIntoUnderscore,_("If activated, the conversion "
-        "will take place when applying a mask."),NULL);
+    gtk_tooltips_set_tip(Tips,ConvertUnderscoreAndP20IntoSpace,_("If activated, this conversion "
+        "will be used when applying a mask from the scanner for tags or filenames."),NULL);
+    gtk_tooltips_set_tip(Tips,ConvertSpaceIntoUnderscore,_("If activated, this conversion "
+        "will be used when applying a mask from the scanner for tags or filenames."),NULL);
 
 
     /* Properties of the scanner window */
@@ -851,12 +863,12 @@ void Open_OptionsWindow (void)
 
     Label = gtk_label_new (_("X :"));
     gtk_box_pack_start(GTK_BOX(hbox),Label,FALSE,FALSE,0);
-    ScannerWindowX = gtk_entry_new_with_max_length(4);
+    ScannerWindowX = gtk_entry_new_with_max_length(5);
     gtk_box_pack_start(GTK_BOX(hbox),ScannerWindowX,FALSE,FALSE,0);
     gtk_widget_set_usize(ScannerWindowX,40,-1);
     Label = gtk_label_new (_("Y :"));
     gtk_box_pack_start(GTK_BOX(hbox),Label,FALSE,FALSE,0);
-    ScannerWindowY = gtk_entry_new_with_max_length(4);
+    ScannerWindowY = gtk_entry_new_with_max_length(5);
     gtk_box_pack_start(GTK_BOX(hbox),ScannerWindowY,FALSE,FALSE,0);
     gtk_widget_set_usize(ScannerWindowY,40,-1);
     ScannerWindowButton = Create_Button_With_Icon_And_Label(grab_xpm,_(" Get current position "));
@@ -890,7 +902,7 @@ void Open_OptionsWindow (void)
     gtk_tooltips_set_tip(Tips,OverwriteTagField,_("If activated, the scanner will replace existing text "
         "in fields by the new one. If deactivated, only blank fields of the tag will be filled."),NULL);
 
-    // Set a default comment text
+    // Set a default comment text or CRC-32 checksum
     hbox = gtk_hbox_new(FALSE,2);
     gtk_box_pack_start(GTK_BOX(vbox),hbox,FALSE,FALSE,0);
     SetDefaultComment = gtk_check_button_new_with_label(_("Set this text as default comment :"));
@@ -917,6 +929,17 @@ void Open_OptionsWindow (void)
         Add_To_Combo_Box_History,GTK_OBJECT(DefaultComment));
     gtk_signal_connect_object(GTK_OBJECT(GTK_ENTRY(GTK_COMBO(DefaultComment)->entry)),"focus_out_event",
         Add_To_Combo_Box_History,GTK_OBJECT(DefaultComment));
+    // CRC32 comment
+    Crc32Comment = gtk_check_button_new_with_label(_("Use CRC32 as the default "
+        "comment (for files with ID3 tags only)."));
+    gtk_box_pack_start(GTK_BOX(vbox),Crc32Comment,FALSE,FALSE,0);
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(Crc32Comment),SET_CRC32_COMMENT);
+    gtk_tooltips_set_tip(Tips,Crc32Comment,_("Calculates the CRC-32 value of the file "
+        "and writes it into the comment field when using the 'Fill Tag' scanner."),NULL);
+    gtk_signal_connect_object(GTK_OBJECT(SetDefaultComment), "toggled",
+        Scanner_Convert_Check_Button_Toggled_1,GTK_OBJECT(Crc32Comment));
+    gtk_signal_connect_object(GTK_OBJECT(Crc32Comment), "toggled",
+        Scanner_Convert_Check_Button_Toggled_1,GTK_OBJECT(SetDefaultComment));
 
 
     /*
@@ -1014,22 +1037,28 @@ void Open_OptionsWindow (void)
     gtk_signal_connect(GTK_OBJECT(CddbProxyPort),"insert_text",GTK_SIGNAL_FUNC(Insert_Only_Digit),NULL);
     gtk_signal_connect(GTK_OBJECT(CddbUseProxy),"toggled",GTK_SIGNAL_FUNC(Cddb_Use_Proxy_Toggled),NULL);
     Label = gtk_label_new(_("User Name :"));
+gtk_widget_set_sensitive(Label,FALSE);
     gtk_table_attach(GTK_TABLE(Table),Label,0,1,2,3,GTK_FILL,GTK_FILL,0,0);
     gtk_misc_set_alignment(GTK_MISC(Label),1,0.5);
     CddbProxyUserName = gtk_entry_new();
+gtk_widget_set_sensitive(CddbProxyUserName,FALSE);
     gtk_table_attach(GTK_TABLE(Table),CddbProxyUserName,1,2,2,3,GTK_FILL,GTK_FILL,0,0);
     gtk_tooltips_set_tip(Tips,CddbProxyUserName,_("Name of user for the the proxy server."),NULL);
+    Label = gtk_label_new(_("(Not available!)"));
 gtk_widget_set_sensitive(Label,FALSE);
-gtk_widget_set_sensitive(CddbProxyUserName,FALSE);
+    gtk_table_attach(GTK_TABLE(Table),Label,2,5,2,3,GTK_FILL,GTK_FILL,0,0);
     Label = gtk_label_new(_("User Password :"));
+gtk_widget_set_sensitive(Label,FALSE);
     gtk_table_attach(GTK_TABLE(Table),Label,0,1,3,4,GTK_FILL,GTK_FILL,0,0);
     gtk_misc_set_alignment(GTK_MISC(Label),1,0.5);
     CddbProxyUserPassword = gtk_entry_new();
+gtk_widget_set_sensitive(CddbProxyUserPassword,FALSE);
     gtk_table_attach(GTK_TABLE(Table),CddbProxyUserPassword,1,2,3,4,GTK_FILL,GTK_FILL,0,0);
     gtk_entry_set_visibility(GTK_ENTRY(CddbProxyUserPassword),FALSE);
     gtk_tooltips_set_tip(Tips,CddbProxyUserPassword,_("Password of user for the the proxy server."),NULL);
+    Label = gtk_label_new(_("(Not available!)"));
 gtk_widget_set_sensitive(Label,FALSE);
-gtk_widget_set_sensitive(CddbProxyUserPassword,FALSE);
+    gtk_table_attach(GTK_TABLE(Table),Label,2,5,3,4,GTK_FILL,GTK_FILL,0,0);
     Cddb_Use_Proxy_Toggled();
 
 
@@ -1122,6 +1151,10 @@ gtk_widget_set_sensitive(CddbProxyUserPassword,FALSE);
     gtk_box_pack_start(GTK_BOX(VBox),ConfirmRenameFile,FALSE,FALSE,0);
     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(ConfirmRenameFile),CONFIRM_RENAME_FILE);
 
+    ConfirmDeleteFile = gtk_check_button_new_with_label(_("Confirm deleting of file"));
+    gtk_box_pack_start(GTK_BOX(VBox),ConfirmDeleteFile,FALSE,FALSE,0);
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(ConfirmDeleteFile),CONFIRM_DELETE_FILE);
+
     ConfirmWritePlayList = gtk_check_button_new_with_label(_("Confirm writing of playlist"));
     gtk_box_pack_start(GTK_BOX(VBox),ConfirmWritePlayList,FALSE,FALSE,0);
     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(ConfirmWritePlayList),CONFIRM_WRITE_PLAYLIST);
@@ -1200,6 +1233,10 @@ void Set_Default_Comment_Check_Button_Toggled (void)
     gtk_widget_set_sensitive(DefaultComment,GTK_TOGGLE_BUTTON(SetDefaultComment)->active);
 }
 
+void Number_Track_Formated_Toggled (void)
+{
+    gtk_widget_set_sensitive(NumberTrackFormatedSpinButton,GTK_TOGGLE_BUTTON(NumberTrackFormated)->active);
+}
 
 void Use_Character_Set_Translation_Toggled (void)
 {
@@ -1559,6 +1596,7 @@ void Get_Scanner_Window_Position (void)
 
 /*
  * Manage Check buttons into Scanner tab: conversion group
+ * This reproduces "something" like the behaviour of radio buttons with check buttons
  */
 void Scanner_Convert_Check_Button_Toggled_1 (GtkObject *object_rec, GtkObject *object_emi)
 {
@@ -1613,5 +1651,3 @@ void Get_Cddb_Pane_Handle_Position (void)
         gtk_entry_set_text(GTK_ENTRY(CddbPaneHandlePosition),temp);
     }
 }
-
-
