@@ -1,7 +1,7 @@
 /* config.c - 2000/06/21 */
 /*
- *  EasyTAG - Tag editor for MP3 and OGG files
- *  Copyright (C) 2000-2002  Jerome Couderc <j.couderc@ifrance.com>
+ *  EasyTAG - Tag editor for MP3 and Ogg Vorbis files
+ *  Copyright (C) 2000-2003  Jerome Couderc <j.couderc@ifrance.com>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -44,7 +44,12 @@
 /***************
  * Declaration *
  ***************/
-// Nota : no trailing slashes on directory name to avoid problem with NetBSD's mkdir(2).
+
+/*
+ * Nota :
+ *  - no trailing slashes on directory name to avoid problem with
+ *    NetBSD's mkdir(2).
+ */
 
 // Base directory created into home dir 
 gchar *EASYTAG_DIR                                 = ".easytag";
@@ -116,27 +121,29 @@ tConfigVariable Config_Variables[] =
     {"use_colored_tooltips",           CV_TYPE_BOOL,    &USE_COLORED_TOOLTIPS       },
 
     {"replace_illegal_character_in_filename",CV_TYPE_BOOL,    &REPLACE_ILLEGAL_CHARACTERS_IN_FILENAME},
-    {"date_auto_completion",                 CV_TYPE_BOOL,    &DATE_AUTO_COMPLETION           },
-    {"number_track_formated",                CV_TYPE_BOOL,    &NUMBER_TRACK_FORMATED          },
-    {"ogg_tag_write_xmms_comment",           CV_TYPE_BOOL,    &OGG_TAG_WRITE_XMMS_COMMENT     },
-    {"sorting_file_mode",                    CV_TYPE_BOOL,    &SORTING_FILE_MODE              },
-    {"sorting_file_case_sensitive",          CV_TYPE_BOOL,    &SORTING_FILE_CASE_SENSITIVE    },
+    {"date_auto_completion",                 CV_TYPE_BOOL,    &DATE_AUTO_COMPLETION                 },
+    {"number_track_formated",                CV_TYPE_BOOL,    &NUMBER_TRACK_FORMATED                },
+    {"number_track_formated_spin_button",    CV_TYPE_INT,     &NUMBER_TRACK_FORMATED_SPIN_BUTTON    },
+    {"ogg_tag_write_xmms_comment",           CV_TYPE_BOOL,    &OGG_TAG_WRITE_XMMS_COMMENT           },
+    {"sorting_file_mode",                    CV_TYPE_BOOL,    &SORTING_FILE_MODE                    },
+    {"sorting_file_case_sensitive",          CV_TYPE_BOOL,    &SORTING_FILE_CASE_SENSITIVE          },
 
-    {"use_character_set_translation",        CV_TYPE_BOOL,    &USE_CHARACTER_SET_TRANSLATION  },
-    {"file_character_set",                   CV_TYPE_STRING,  &FILE_CHARACTER_SET             },
-    {"user_character_set",                   CV_TYPE_STRING,  &USER_CHARACTER_SET             },
+    {"use_character_set_translation",        CV_TYPE_BOOL,    &USE_CHARACTER_SET_TRANSLATION        },
+    {"file_character_set",                   CV_TYPE_STRING,  &FILE_CHARACTER_SET                   },
+    {"user_character_set",                   CV_TYPE_STRING,  &USER_CHARACTER_SET                   },
 
-    {"message_box_position_none",            CV_TYPE_BOOL,    &MESSAGE_BOX_POSITION_NONE      },
-    {"message_box_position_center",          CV_TYPE_BOOL,    &MESSAGE_BOX_POSITION_CENTER    },
-    {"message_box_position_mouse",           CV_TYPE_BOOL,    &MESSAGE_BOX_POSITION_MOUSE     },
+    {"message_box_position_none",            CV_TYPE_BOOL,    &MESSAGE_BOX_POSITION_NONE            },
+    {"message_box_position_center",          CV_TYPE_BOOL,    &MESSAGE_BOX_POSITION_CENTER          },
+    {"message_box_position_mouse",           CV_TYPE_BOOL,    &MESSAGE_BOX_POSITION_MOUSE           },
 
-    {"audio_file_player",                    CV_TYPE_STRING,  &AUDIO_FILE_PLAYER              },
+    {"audio_file_player",                    CV_TYPE_STRING,  &AUDIO_FILE_PLAYER                    },
 
     {"convert_underscore_and_p20_into_space",CV_TYPE_BOOL,    &CONVERT_UNDERSCORE_AND_P20_INTO_SPACE},
     {"convert_space_into_underscore",        CV_TYPE_BOOL,    &CONVERT_SPACE_INTO_UNDERSCORE        },
     {"overwrite_tag_field",                  CV_TYPE_BOOL,    &OVERWRITE_TAG_FIELD                  },
     {"set_default_comment",                  CV_TYPE_BOOL,    &SET_DEFAULT_COMMENT                  },
     {"default_comment",                      CV_TYPE_STRING,  &DEFAULT_COMMENT                      },
+    {"crc32_comment",                        CV_TYPE_BOOL,    &SET_CRC32_COMMENT                    },
     {"open_scanner_window_on_startup",       CV_TYPE_BOOL,    &OPEN_SCANNER_WINDOW_ON_STARTUP       },
     {"scanner_window_on_top",                CV_TYPE_BOOL,    &SCANNER_WINDOW_ON_TOP                },
     {"set_scanner_window_position",          CV_TYPE_BOOL,    &SET_SCANNER_WINDOW_POSITION          },
@@ -147,6 +154,7 @@ tConfigVariable Config_Variables[] =
     {"confirm_write_tag",                    CV_TYPE_BOOL,    &CONFIRM_WRITE_TAG                    },
     {"confirm_rename_file",                  CV_TYPE_BOOL,    &CONFIRM_RENAME_FILE                  },
     {"confirm_write_playlist",               CV_TYPE_BOOL,    &CONFIRM_WRITE_PLAYLIST               },
+    {"confirm_delete_file",                  CV_TYPE_BOOL,    &CONFIRM_DELETE_FILE                  },
     {"options_notebook_page",                CV_TYPE_INT,     &OPTIONS_NOTEBOOK_PAGE                },
     {"process_filename_field",               CV_TYPE_BOOL,    &PROCESS_FILENAME_FIELD               },
     {"process_title_field",                  CV_TYPE_BOOL,    &PROCESS_TITLE_FIELD                  },
@@ -169,6 +177,8 @@ tConfigVariable Config_Variables[] =
     {"playlist_use_dir_name",                CV_TYPE_BOOL,    &PLAYLIST_USE_DIR_NAME                },
     {"playlist_full_path",                   CV_TYPE_BOOL,    &PLAYLIST_FULL_PATH                   },
     {"playlist_relative_path",               CV_TYPE_BOOL,    &PLAYLIST_RELATIVE_PATH               },
+    {"playlist_create_in_parent_dir",        CV_TYPE_BOOL,    &PLAYLIST_CREATE_IN_PARENT_DIR        },
+    {"playlist_use_dos_separator",           CV_TYPE_BOOL,    &PLAYLIST_USE_DOS_SEPARATOR           },
     {"playlist_content_none",                CV_TYPE_BOOL,    &PLAYLIST_CONTENT_NONE                },
     {"playlist_content_filename",            CV_TYPE_BOOL,    &PLAYLIST_CONTENT_FILENAME            },
     {"playlist_content_mask",                CV_TYPE_BOOL,    &PLAYLIST_CONTENT_MASK                },
@@ -239,8 +249,8 @@ void Init_Config_Variables (void)
      */
     LOAD_ON_STARTUP               = 1;
     DEFAULT_PATH_TO_MP3           = g_strdup(HOME_VARIABLE);
-    BROWSER_LINE_STYLE            = GTK_CTREE_LINES_DOTTED;
-    BROWSER_EXPANDER_STYLE        = GTK_CTREE_EXPANDER_SQUARE;
+    BROWSER_LINE_STYLE            = GTK_CTREE_LINES_NONE;
+    BROWSER_EXPANDER_STYLE        = GTK_CTREE_EXPANDER_TRIANGLE;
     WRITE_ID3V1_TAG               = 1;
     WRITE_ID3V2_TAG               = 1;
     STRIP_TAG_WHEN_EMPTY_FIELDS   = 1;
@@ -271,6 +281,7 @@ void Init_Config_Variables (void)
     OVERWRITE_TAG_FIELD                   = 0;
     SET_DEFAULT_COMMENT                   = 0;
     DEFAULT_COMMENT                       = g_strdup("Tagged with EasyTAG");
+    SET_CRC32_COMMENT                     = 0;
     OPEN_SCANNER_WINDOW_ON_STARTUP        = 1;
     SCANNER_WINDOW_ON_TOP                 = 1;
     SET_SCANNER_WINDOW_POSITION           = 0;
@@ -281,20 +292,21 @@ void Init_Config_Variables (void)
      * Misc
      */
     REPLACE_ILLEGAL_CHARACTERS_IN_FILENAME = 1;
-    DATE_AUTO_COMPLETION          = 1;
-    NUMBER_TRACK_FORMATED         = 1;
-    OGG_TAG_WRITE_XMMS_COMMENT    = 1;
-    SORTING_FILE_MODE             = SORTING_BY_ASCENDING_FILENAME;
-    SORTING_FILE_CASE_SENSITIVE   = 0;
-    USE_CHARACTER_SET_TRANSLATION = 0;
-    FILE_CHARACTER_SET            = g_strdup("UTF-8");
-    USER_CHARACTER_SET            = g_strdup("ISO-8859-15");
+    DATE_AUTO_COMPLETION                   = 1;
+    NUMBER_TRACK_FORMATED                  = 1;
+    NUMBER_TRACK_FORMATED_SPIN_BUTTON      = 2;
+    OGG_TAG_WRITE_XMMS_COMMENT             = 1;
+    SORTING_FILE_MODE                      = SORTING_BY_ASCENDING_FILENAME;
+    SORTING_FILE_CASE_SENSITIVE            = 0;
+    USE_CHARACTER_SET_TRANSLATION          = 0;
+    FILE_CHARACTER_SET                     = g_strdup("UTF-8");
+    USER_CHARACTER_SET                     = g_strdup("ISO-8859-15");
 
-    MESSAGE_BOX_POSITION_NONE     = 0;
-    MESSAGE_BOX_POSITION_CENTER   = 1;
-    MESSAGE_BOX_POSITION_MOUSE    = 0;
+    MESSAGE_BOX_POSITION_NONE              = 0;
+    MESSAGE_BOX_POSITION_CENTER            = 1;
+    MESSAGE_BOX_POSITION_MOUSE             = 0;
 
-    AUDIO_FILE_PLAYER             = g_strdup("xmms -p");
+    AUDIO_FILE_PLAYER                      = g_strdup("xmms -p");
 
     /*
      * Confirmation
@@ -302,6 +314,7 @@ void Init_Config_Variables (void)
     CONFIRM_BEFORE_EXIT    = 1;
     CONFIRM_WRITE_TAG      = 1;
     CONFIRM_RENAME_FILE    = 1;
+    CONFIRM_DELETE_FILE    = 1;
     CONFIRM_WRITE_PLAYLIST = 1;
 
     /*
@@ -326,15 +339,17 @@ void Init_Config_Variables (void)
     /*
      * Playlist window
      */
-    PLAYLIST_NAME             = g_strdup("playlist_%a_-_%t");
-    PLAYLIST_USE_MASK_NAME    = 0;
-    PLAYLIST_USE_DIR_NAME     = 1;
-    PLAYLIST_FULL_PATH        = 0;
-    PLAYLIST_RELATIVE_PATH    = 1;
-    PLAYLIST_CONTENT_NONE     = 0;
-    PLAYLIST_CONTENT_FILENAME = 1;
-    PLAYLIST_CONTENT_MASK     = 0;
-    PLAYLIST_CONTENT_MASK_VALUE = g_strdup("%n/%l - %a - %b - %t");
+    PLAYLIST_NAME                   = g_strdup("playlist_%a_-_%t");
+    PLAYLIST_USE_MASK_NAME          = 0;
+    PLAYLIST_USE_DIR_NAME           = 1;
+    PLAYLIST_FULL_PATH              = 0;
+    PLAYLIST_RELATIVE_PATH          = 1;
+    PLAYLIST_CREATE_IN_PARENT_DIR   = 0;
+    PLAYLIST_USE_DOS_SEPARATOR      = 0;
+    PLAYLIST_CONTENT_NONE           = 0;
+    PLAYLIST_CONTENT_FILENAME       = 1;
+    PLAYLIST_CONTENT_MASK           = 0;
+    PLAYLIST_CONTENT_MASK_VALUE     = g_strdup("%n/%l - %a - %b - %t");
 
     /*
      * Load File window
@@ -350,32 +365,32 @@ void Init_Config_Variables (void)
     CDDB_USE_PROXY       = 0;
     CDDB_PROXY_NAME      = g_strdup("localhost");
     CDDB_PROXY_PORT      = 8080;
-//    CDDB_PROXY_USER_NAME          = NULL;
-//    CDDB_PROXY_USER_PASSWORD      = NULL;
+    //CDDB_PROXY_USER_NAME          = NULL;
+    //CDDB_PROXY_USER_PASSWORD      = NULL;
     SET_CDDB_WINDOW_SIZE          =  0;
     CDDB_WINDOW_HEIGHT            = -1;
     CDDB_WINDOW_WIDTH             = -1;
     SET_CDDB_PANE_HANDLE_POSITION =  0;
     CDDB_PANE_HANDLE_POSITION     = -1;
 
-    CDDB_SEARCH_IN_ALL_FIELDS       = 0;
-    CDDB_SEARCH_IN_ARTIST_FIELD     = 1;
-    CDDB_SEARCH_IN_TITLE_FIELD      = 1;
-    CDDB_SEARCH_IN_TRACK_NAME_FIELD = 0;
-    CDDB_SEARCH_IN_OTHER_FIELD      = 0;
-    CDDB_SHOW_CATEGORIES            = 0;
+    CDDB_SEARCH_IN_ALL_FIELDS           = 0;
+    CDDB_SEARCH_IN_ARTIST_FIELD         = 1;
+    CDDB_SEARCH_IN_TITLE_FIELD          = 1;
+    CDDB_SEARCH_IN_TRACK_NAME_FIELD     = 0;
+    CDDB_SEARCH_IN_OTHER_FIELD          = 0;
+    CDDB_SHOW_CATEGORIES                = 0;
 
-    CDDB_SEARCH_IN_ALL_CATEGORIES      = 1;
-    CDDB_SEARCH_IN_BLUES_CATEGORY      = 0;
-    CDDB_SEARCH_IN_CLASSICAL_CATEGORY  = 0;
-    CDDB_SEARCH_IN_COUNTRY_CATEGORY    = 0;
-    CDDB_SEARCH_IN_FOLK_CATEGORY       = 0;
-    CDDB_SEARCH_IN_JAZZ_CATEGORY       = 0;
-    CDDB_SEARCH_IN_MISC_CATEGORY       = 1;
-    CDDB_SEARCH_IN_NEWAGE_CATEGORY     = 1;
-    CDDB_SEARCH_IN_REGGAE_CATEGORY     = 0;
-    CDDB_SEARCH_IN_ROCK_CATEGORY       = 1;
-    CDDB_SEARCH_IN_SOUNDTRACK_CATEGORY = 0;
+    CDDB_SEARCH_IN_ALL_CATEGORIES       = 1;
+    CDDB_SEARCH_IN_BLUES_CATEGORY       = 0;
+    CDDB_SEARCH_IN_CLASSICAL_CATEGORY   = 0;
+    CDDB_SEARCH_IN_COUNTRY_CATEGORY     = 0;
+    CDDB_SEARCH_IN_FOLK_CATEGORY        = 0;
+    CDDB_SEARCH_IN_JAZZ_CATEGORY        = 0;
+    CDDB_SEARCH_IN_MISC_CATEGORY        = 1;
+    CDDB_SEARCH_IN_NEWAGE_CATEGORY      = 1;
+    CDDB_SEARCH_IN_REGGAE_CATEGORY      = 0;
+    CDDB_SEARCH_IN_ROCK_CATEGORY        = 1;
+    CDDB_SEARCH_IN_SOUNDTRACK_CATEGORY  = 0;
 
     CDDB_SET_TO_ALL_FIELDS  = 1;
     CDDB_SET_TO_TITLE       = 1;
@@ -419,25 +434,26 @@ void Apply_Changes (void)
         SAVE_CONFIG_BEFORE_EXIT       = GTK_TOGGLE_BUTTON(SaveConfigBeforeExit)->active;
 
         /* User interface */
-        SET_MAIN_WINDOW_SIZE      = GTK_TOGGLE_BUTTON(SetMainWindowSize)->active;
-        MAIN_WINDOW_HEIGHT        = atoi(gtk_entry_get_text_1(MainWindowHeight));
-        MAIN_WINDOW_WIDTH         = atoi(gtk_entry_get_text_1(MainWindowWidth));
-        SET_PANE_HANDLE_POSITION1 = GTK_TOGGLE_BUTTON(SetPaneHandlePosition1)->active;
-        PANE_HANDLE_POSITION1     = atoi(gtk_entry_get_text_1(PaneHandlePosition1));
-        SET_PANE_HANDLE_POSITION2 = GTK_TOGGLE_BUTTON(SetPaneHandlePosition2)->active;
-        PANE_HANDLE_POSITION2     = atoi(gtk_entry_get_text_1(PaneHandlePosition2));
-        SHOW_HEADER_INFO          = GTK_TOGGLE_BUTTON(ShowHeaderInfos)->active;
-        USE_COLORED_TOOLTIPS      = GTK_TOGGLE_BUTTON(UseColoredTooltips)->active;
+        SET_MAIN_WINDOW_SIZE          = GTK_TOGGLE_BUTTON(SetMainWindowSize)->active;
+        MAIN_WINDOW_HEIGHT            = atoi(gtk_entry_get_text_1(MainWindowHeight));
+        MAIN_WINDOW_WIDTH             = atoi(gtk_entry_get_text_1(MainWindowWidth));
+        SET_PANE_HANDLE_POSITION1     = GTK_TOGGLE_BUTTON(SetPaneHandlePosition1)->active;
+        PANE_HANDLE_POSITION1         = atoi(gtk_entry_get_text_1(PaneHandlePosition1));
+        SET_PANE_HANDLE_POSITION2     = GTK_TOGGLE_BUTTON(SetPaneHandlePosition2)->active;
+        PANE_HANDLE_POSITION2         = atoi(gtk_entry_get_text_1(PaneHandlePosition2));
+        SHOW_HEADER_INFO              = GTK_TOGGLE_BUTTON(ShowHeaderInfos)->active;
+        USE_COLORED_TOOLTIPS          = GTK_TOGGLE_BUTTON(UseColoredTooltips)->active;
 
         /* Misc */
         REPLACE_ILLEGAL_CHARACTERS_IN_FILENAME = GTK_TOGGLE_BUTTON(ReplaceIllegalCharactersInFilename)->active;
-        DATE_AUTO_COMPLETION          = GTK_TOGGLE_BUTTON(DateAutoCompletion)->active;
-        NUMBER_TRACK_FORMATED         = GTK_TOGGLE_BUTTON(NumberTrackFormated)->active;
-        OGG_TAG_WRITE_XMMS_COMMENT    = GTK_TOGGLE_BUTTON(OggTagWriteXmmsComment)->active;
-        SORTING_FILE_CASE_SENSITIVE   = GTK_TOGGLE_BUTTON(SortingFileCaseSensitive)->active;
-        USE_CHARACTER_SET_TRANSLATION = GTK_TOGGLE_BUTTON(UseCharacterSetTranslation)->active;
-        FILE_CHARACTER_SET     = Charset_Get_Name_From_Title(gtk_entry_get_text_1(FileCharacterSetEntry));
-        USER_CHARACTER_SET     = Charset_Get_Name_From_Title(gtk_entry_get_text_1(UserCharacterSetEntry));
+        DATE_AUTO_COMPLETION                   = GTK_TOGGLE_BUTTON(DateAutoCompletion)->active;
+        NUMBER_TRACK_FORMATED                  = GTK_TOGGLE_BUTTON(NumberTrackFormated)->active;
+        NUMBER_TRACK_FORMATED_SPIN_BUTTON      = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(NumberTrackFormatedSpinButton));
+        OGG_TAG_WRITE_XMMS_COMMENT             = GTK_TOGGLE_BUTTON(OggTagWriteXmmsComment)->active;
+        SORTING_FILE_CASE_SENSITIVE            = GTK_TOGGLE_BUTTON(SortingFileCaseSensitive)->active;
+        USE_CHARACTER_SET_TRANSLATION          = GTK_TOGGLE_BUTTON(UseCharacterSetTranslation)->active;
+        FILE_CHARACTER_SET                     = Charset_Get_Name_From_Title(gtk_entry_get_text_1(FileCharacterSetEntry));
+        USER_CHARACTER_SET                     = Charset_Get_Name_From_Title(gtk_entry_get_text_1(UserCharacterSetEntry));
 
         MESSAGE_BOX_POSITION_NONE   = GTK_TOGGLE_BUTTON(MessageBoxPositionNone)->active;
         MESSAGE_BOX_POSITION_CENTER = GTK_TOGGLE_BUTTON(MessageBoxPositionCenter)->active;
@@ -452,6 +468,7 @@ void Apply_Changes (void)
         OVERWRITE_TAG_FIELD = GTK_TOGGLE_BUTTON(OverwriteTagField)->active;
         SET_DEFAULT_COMMENT = GTK_TOGGLE_BUTTON(SetDefaultComment)->active;
         DEFAULT_COMMENT     = g_strdup(gtk_entry_get_text_1(DefaultComment));
+        SET_CRC32_COMMENT   = GTK_TOGGLE_BUTTON(Crc32Comment)->active;
 
         OPEN_SCANNER_WINDOW_ON_STARTUP = GTK_TOGGLE_BUTTON(OpenScannerWindowOnStartup)->active;
         SCANNER_WINDOW_ON_TOP          = GTK_TOGGLE_BUTTON(ScannerWindowOnTop)->active;
@@ -478,6 +495,7 @@ void Apply_Changes (void)
         CONFIRM_BEFORE_EXIT    = GTK_TOGGLE_BUTTON(ConfirmBeforeExit)->active;
         CONFIRM_WRITE_TAG      = GTK_TOGGLE_BUTTON(ConfirmWriteTag)->active;
         CONFIRM_RENAME_FILE    = GTK_TOGGLE_BUTTON(ConfirmRenameFile)->active;
+        CONFIRM_DELETE_FILE    = GTK_TOGGLE_BUTTON(ConfirmDeleteFile)->active;
         CONFIRM_WRITE_PLAYLIST = GTK_TOGGLE_BUTTON(ConfirmWritePlayList)->active;
 
         /* Get the last visible notebook page */
@@ -498,6 +516,8 @@ void Apply_Changes (void)
 /*    if (SET_WINDOW_SIZE) gdk_window_resize(MainWindow->window,WINDOW_HEIGHT,WINDOW_WIDTH);
  *    else                 gdk_window_resize(MainWindow->window,0,0);
  */
+    /* Reload if number of character changed for track list */
+    //Load_Track_List_To_UI();
 
     if (ScannerWindow)
     {
@@ -514,15 +534,17 @@ void Apply_Changes (void)
     {
         /* Playlist */
         /* Also set in the function 'Playlist_Write_Button_Pressed' */
-        PLAYLIST_NAME               = g_strdup(gtk_entry_get_text_1(PlayListNameEntry));
-        PLAYLIST_USE_MASK_NAME      = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(playlist_use_mask_name));
-        PLAYLIST_USE_DIR_NAME       = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(playlist_use_dir_name));
-        PLAYLIST_FULL_PATH          = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(playlist_full_path));
-        PLAYLIST_RELATIVE_PATH      = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(playlist_relative_path));
-        PLAYLIST_CONTENT_NONE       = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(playlist_content_none));
-        PLAYLIST_CONTENT_FILENAME   = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(playlist_content_filename));
-        PLAYLIST_CONTENT_MASK       = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(playlist_content_mask));
-        PLAYLIST_CONTENT_MASK_VALUE = g_strdup(gtk_entry_get_text_1(PlayListContentMaskEntry));
+        PLAYLIST_NAME                 = g_strdup(gtk_entry_get_text_1(PlayListNameEntry));
+        PLAYLIST_USE_MASK_NAME        = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(playlist_use_mask_name));
+        PLAYLIST_USE_DIR_NAME         = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(playlist_use_dir_name));
+        PLAYLIST_FULL_PATH            = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(playlist_full_path));
+        PLAYLIST_RELATIVE_PATH        = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(playlist_relative_path));
+        PLAYLIST_CREATE_IN_PARENT_DIR = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(playlist_create_in_parent_dir));
+        PLAYLIST_USE_DOS_SEPARATOR    = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(playlist_use_dos_separator));
+        PLAYLIST_CONTENT_NONE         = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(playlist_content_none));
+        PLAYLIST_CONTENT_FILENAME     = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(playlist_content_filename));
+        PLAYLIST_CONTENT_MASK         = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(playlist_content_mask));
+        PLAYLIST_CONTENT_MASK_VALUE   = g_strdup(gtk_entry_get_text_1(PlayListContentMaskEntry));
     }
 
     if (LoadFilenameWindow)
@@ -794,6 +816,105 @@ void Display_Config (void)
 
 
 
+/*
+ * Create the main directory with empty history files
+ */
+void Setting_Create_Files (void)
+{
+    gchar *home_path = NULL;
+    gchar *file_path = NULL;
+    FILE  *file;
+    
+    /* The file to write */
+    if (!HOME_VARIABLE)
+        return;
+        
+    if ( Create_Easytag_Directory()==FALSE )
+        return;
+    
+    home_path = g_strconcat(HOME_VARIABLE,
+                            HOME_VARIABLE[strlen(HOME_VARIABLE)-1]!='/'?"/":"",
+                            NULL);
+    
+    file_path = g_strconcat(home_path,CONFIG_FILE,NULL);
+    if ( (file=fopen(file_path,"a+")) == 0 )
+        fclose(file);
+    g_free(file_path);
+    
+    file_path = g_strconcat(home_path,SCAN_TAG_MASKS_FILE,NULL);
+    if ( (file=fopen(file_path,"a+")) == 0 )
+        fclose(file);
+    g_free(file_path);
+    
+    file_path = g_strconcat(home_path,RENAME_FILE_MASKS_FILE,NULL);
+    if ( (file=fopen(file_path,"a+")) == 0 )
+        fclose(file);
+    g_free(file_path);
+    
+    file_path = g_strconcat(home_path,DEFAULT_PATH_TO_MP3_HISTORY_FILE,NULL);
+    if ( (file=fopen(file_path,"a+")) == 0 )
+        fclose(file);
+    g_free(file_path);
+    
+    file_path = g_strconcat(home_path,DEFAULT_TAG_COMMENT_HISTORY_FILE,NULL);
+    if ( (file=fopen(file_path,"a+")) == 0 )
+        fclose(file);
+    g_free(file_path);
+    
+    file_path = g_strconcat(home_path,PATH_ENTRY_HISTORY_FILE,NULL);
+    if ( (file=fopen(file_path,"a+")) == 0 )
+        fclose(file);
+    g_free(file_path);
+
+    file_path = g_strconcat(home_path,PLAY_LIST_NAME_HISTORY_FILE,NULL);
+    if ( (file=fopen(file_path,"a+")) == 0 )
+        fclose(file);
+    g_free(file_path);
+
+    file_path = g_strconcat(home_path,RUN_PROGRAM_WITH_DIRECTORY_HISTORY_FILE,NULL);
+    if ( (file=fopen(file_path,"a+")) == 0 )
+        fclose(file);
+    g_free(file_path);
+    
+    file_path = g_strconcat(home_path,RUN_PROGRAM_WITH_FILE_HISTORY_FILE,NULL);
+    if ( (file=fopen(file_path,"a+")) == 0 )
+        fclose(file);
+    g_free(file_path);
+    
+    file_path = g_strconcat(home_path,AUDIO_FILE_PLAYER_HISTORY_FILE,NULL);
+    if ( (file=fopen(file_path,"a+")) == 0 )
+        fclose(file);
+    g_free(file_path);
+    
+    file_path = g_strconcat(home_path,SEARCH_FILE_HISTORY_FILE,NULL);
+    if ( (file=fopen(file_path,"a+")) == 0 )
+        fclose(file);
+    g_free(file_path);
+    
+    file_path = g_strconcat(home_path,FILE_TO_LOAD_HISTORY_FILE,NULL);
+    if ( (file=fopen(file_path,"a+")) == 0 )
+        fclose(file);
+    g_free(file_path);
+    
+    file_path = g_strconcat(home_path,PLAYLIST_CONTENT_HISTORY_FILE,NULL);
+    if ( (file=fopen(file_path,"a+")) == 0 )
+        fclose(file);
+    g_free(file_path);
+    
+    file_path = g_strconcat(home_path,CDDB_SEARCH_STRING_HISTORY_FILE,NULL);
+    if ( (file=fopen(file_path,"a+")) == 0 )
+        fclose(file);
+    g_free(file_path);
+    
+    file_path = g_strconcat(home_path,CDDB_SEARCH_STRING_IN_RESULT_HISTORY_FILE,NULL);
+    if ( (file=fopen(file_path,"a+")) == 0 )
+        fclose(file);
+    g_free(file_path);
+    
+    g_free(home_path);
+}
+
+
 
 /*
  * Save a glist list into file
@@ -1064,11 +1185,12 @@ gboolean Create_Easytag_Directory (void)
         return FALSE;
     }
 
-    /* Directory to create (if doesn't exists) with absolute path */
+    /* Directory to create (if doesn't exists) with absolute path
+     * Note for NetBSD : avoid passing a trailing slash to mkdir() */
     easytag_path = g_strconcat(HOME_VARIABLE,
                                HOME_VARIABLE[strlen(HOME_VARIABLE)-1]!='/'?"/":"",
                                EASYTAG_DIR,
-                               EASYTAG_DIR[strlen(EASYTAG_DIR)-1]!='/'?"/":"",
+                               //EASYTAG_DIR[strlen(EASYTAG_DIR)-1]!='/'?"/":"",
                                NULL);
 
     if ( (dir=opendir(easytag_path)) == NULL )
